/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded;

import java.util.HashSet;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.pc.inventory.InventoryEventListener;

/**
 * @author Ian Springer
 */
public class LoggingInventoryEventListener implements InventoryEventListener
{
    private final Log log = LogFactory.getLog(this.getClass());

    public void resourcesAdded(Set<Resource> resources)
    {
        log.trace("The following Resources have been added: " + toString(resources));
        return;
    }

    public void resourcesRemoved(Set<Resource> resources)
    {
        log.debug("The following Resources have been removed: " + toString(resources));
        return;
    }

    public void resourceActivated(Resource resource)
    {
        Set resources = new HashSet();
        resources.add(resource);
        log.trace("The following Resource has been activated: " + toString(resources));
        return;
    }

    public void resourceDeactivated(Resource resource) {
        Set resources = new HashSet();
        resources.add(resource);
        log.trace("The following Resource has been deactivated: " + toString(resources));
        return;
    }

    private static String toString(Set<Resource> resources)
    {
        StringBuilder stringBuilder = new StringBuilder();
        for (Resource resource : resources) {
            stringBuilder.append("Resource[").append(resource.getId()).append("]: ").append(resource.getName()).append(", ");
        }
        stringBuilder.delete(stringBuilder.length() - 2, stringBuilder.length());
        return stringBuilder.toString();
    }
}
