/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded;

import java.util.HashMap;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import org.rhq.core.domain.configuration.Property;
import org.rhq.core.domain.configuration.PropertySimple;

/**
 * TODO
 */
public class TextareaMapConverter implements Converter {
    private static final String ATTRIBUTE_VALUE_SEPARATOR = "=";

    // from http://www.w3.org/TR/REC-html40/interact/forms.html#h-17.13.4
    //  'As with all MIME transmissions, "CR LF" (i.e., `%0D%0A') is used to separate lines of data.'
    //
    // despite this FF on Win and Lin only puts a \n between lines, IE7 puts \r\n.
    //
    // if we split on \n we need to trim the value to get rid of the \r which IE puts in
    private static final String LINE_SEPARATOR = "\n";

    /**
     * Adapted from com.jboss.jbossnetwork.ui.action.resource.common.manage.APLHelper.gatherMapParameter TODO add i18n
     * of messages
     */
    public Object getAsObject(FacesContext facesContext, UIComponent uiComponent, String valueAsString)
    {
        Map<String, Property> valueAsMap = null;
        if (valueAsString != null && !valueAsString.trim().equals(""))
        {
            String[] lines = valueAsString.split(LINE_SEPARATOR);

            valueAsMap = new HashMap<String, Property>();

            for (String line : lines)
            {
                // don't count empty lines
                if (line == null || line.trim().equals(""))
                {
                    continue;
                }

                // remove any whitespace from start or end of line, in particular this
                // resolves JBMANCON-28
                line = line.trim();

                // just split once, on the first occurrence of the separator,
                // therefore if the separator appears in the value portion we won't fall over
                //   "If the limit n is greater than zero then the pattern will be applied at most n - 1 times,
                //    the array's length will be no greater than n"
                String[] attributeValuePair = line.split(ATTRIBUTE_VALUE_SEPARATOR, 2);

                // so if we've only entered the value, or something without a separator ignore it
                // name=               GOOD
                // badenty             BAD
                // =value              BAD
                // name=2+2=4          GOOD
                // in the above example the last value for 'name' attribute will win, i.e. 2+2=4
                if (attributeValuePair.length == 2 && attributeValuePair[0] != null &&
                        !attributeValuePair[0].equals(""))
                {
                    PropertySimple property = new PropertySimple(attributeValuePair[0], attributeValuePair[1].trim());
                    valueAsMap.put(attributeValuePair[0], property);
                }
                else
                {
                    FacesMessage message = new FacesMessage();
                    message.setDetail("Invalid attribute/value pair read :" + line);
                    message.setSummary("Invalid attribute/value pair read :" + line);
                    message.setSeverity(FacesMessage.SEVERITY_ERROR);
                    throw new ConverterException(message);
                }
            }
        }

        return valueAsMap;
    }

    /**
     * Adapted from com.jboss.jbossnetwork.ui.action.resource.common.manage.APLHelper.prepareParameterValue(Parameter ,
     * Map )
     */
    public String getAsString(FacesContext facesContext, UIComponent uiComponent, Object valueAsMap)
    {
        String valueAsString = null;
        if (valueAsMap != null && valueAsMap instanceof Map)
        {
            Map<String, Property> map = (Map<String, Property>)valueAsMap;

            StringBuffer mapAsStringBuffer = new StringBuffer();
            for (String attributeName : map.keySet())
            {
                PropertySimple attributeValue = (PropertySimple)map.get(attributeName);

                mapAsStringBuffer.append(attributeName)
                        .append(ATTRIBUTE_VALUE_SEPARATOR)
                        .append(attributeValue.getStringValue())
                        .append(LINE_SEPARATOR);
            }
            valueAsString = mapAsStringBuffer.toString();
        }

        return valueAsString;
    }
}
