/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.bean;

import java.util.Map;
import java.util.Set;

/**
 * Utility class used to help with constructing/processing the drop down list of available configuration templates.
 */
public class ConfigTemplateSelectUtil
{
    private static final String SEPARATOR = "__";

    public static void populateSelectItemsMap(Map<String, String> selectItems, Set<String> templateNames, String resourceTypeName)
    {
        for (String templateName : templateNames)
        {
            String label = templateName + " (" + resourceTypeName + ")";
            String value = templateName + SEPARATOR + resourceTypeName;
            selectItems.put(label, value);
        }
    }

    public static String getTemplateName(String selectItemValue)
    {
        if (selectItemValue == null || selectItemValue.trim().equals(""))
        {
            throw new RuntimeException("value containing configuration template name was null.");
        }
        return getSectionOfTemplateSelectItemValue(selectItemValue, 0);
    }

    public static String getResourceTypeName(String selectItemValue)
    {
        if (selectItemValue == null || selectItemValue.trim().equals(""))
        {
            throw new RuntimeException("value containing resource type name was null.");
        }
        return getSectionOfTemplateSelectItemValue(selectItemValue, 1);
    }

    private static String getSectionOfTemplateSelectItemValue(String selectItemValue, int i)
    {
        String section = null;
        if (selectItemValue != null && !selectItemValue.trim().equals(""))
        {
            // just split once, on the first occurrence of the separator,
            // therefore if the separator appears in the value portion we won't fall over
            //   "If the limit n is greater than zero then the pattern will be applied at most n - 1 times,
            //    the array's length will be no greater than n"
            String[] templateNameAndTypeName = selectItemValue.split(SEPARATOR, 2);

            if (templateNameAndTypeName.length == 2 &&
                    templateNameAndTypeName[0] != null &&
                    !templateNameAndTypeName[0].trim().equals("") &&
                    templateNameAndTypeName[1] != null &&
                    !templateNameAndTypeName[1].trim().equals(""))
            {
                section = templateNameAndTypeName[i];
            }
        }

        return section;
    }
}
