/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.bean;

import java.util.Comparator;
import java.util.Map;

import org.rhq.core.domain.measurement.MeasurementData;
import org.rhq.core.domain.measurement.MeasurementDefinition;

/**
 * @author Mark Spritzler TODO evaluate whether this class is needed why cant we use the order defined by
 *         measurementDefinitionMap which should be ordered as the measurements are specified in the plugin descriptor
 */
public class MeasurementComparator implements Comparator<MeasurementData>
{

    /**
     * map of measurement definition names to the actual measurement definition
     */
    private Map<String, MeasurementDefinition> measurementDefinitionMap;

    public MeasurementComparator()
    {
        super();
    }

    /**
     * Constructor that will allow us to sort by displayName that isn't in the MeasurementData
     *
     * @param measurementDefinitionMap
     */
    public MeasurementComparator(Map<String, MeasurementDefinition> measurementDefinitionMap)
    {
        this.measurementDefinitionMap = measurementDefinitionMap;
    }

    public int compare(MeasurementData data1, MeasurementData data2)
    {
        if (measurementDefinitionMap != null)
        {
            MeasurementDefinition definition1 = measurementDefinitionMap.get(data1.getName());
            MeasurementDefinition definition2 = measurementDefinitionMap.get(data2.getName());

            if (definition1 != null && definition2 != null)
            {
                if (definition1.getDisplayOrder() > definition2.getDisplayOrder())
                {
                    return 1;
                }
                else if (definition1.getDisplayOrder() < definition2.getDisplayOrder())
                {
                    return -1;
                }

                return definition1.getDisplayName().compareTo(definition2.getDisplayName());
            }
        }
        // if trying to get display name fails in anyway, then we will sort by the
        // name of the metric
        return data1.getName().compareTo(data2.getName());
    }
}