/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.bean;

import java.io.Serializable;

import org.rhq.core.domain.measurement.Availability;
import org.rhq.core.domain.resource.Resource;

/**
 * Bean used to hold all of the data necessary to render a table of resources. Quick accessors are provided to the
 * nested attributes to minimize the chaining that needs to take place in the XHTML pages.
 *
 * @author Jason Dobies
 */
public class ResourceListItem implements Serializable {
    // Attributes  --------------------------------------------

    private Resource resource;
    private Availability availability;

    // Constructors  --------------------------------------------

    public ResourceListItem(Resource resource, Availability availability) {
        this.resource = resource;
        this.availability = availability;
    }

    // Public  --------------------------------------------

    public Resource getResource() {
        return resource;
    }

    public String getAvailability() {
        if (availability == null || availability.getAvailabilityType() == null) {
            return "UNKNOWN";
        } else {
            return availability.getAvailabilityType().toString();
        }
    }

    public String getName() {
        return resource.getName();
    }

    public String getType() {
        return resource.getResourceType().getName();
    }
}
