/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.bean.history.content;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.rhq.core.clientapi.server.content.ContentServerService;
import org.rhq.core.clientapi.server.content.ContentServiceResponse;
import org.rhq.core.domain.content.PackageDetailsKey;
import org.rhq.core.domain.content.composite.PackageVersionMetadataComposite;
import org.rhq.core.clientapi.server.content.ContentDiscoveryReport;
import org.rhq.core.domain.content.transfer.DeployPackagesResponse;
import org.rhq.core.domain.content.transfer.RemovePackagesResponse;
import org.rhq.core.domain.content.transfer.ResourcePackageDetails;
import org.rhq.core.domain.util.PageControl;
import org.rhq.core.domain.util.PageList;

import org.jboss.on.embedded.manager.history.content.ContentHistoryManager;

public class ContentServerServiceImpl implements ContentServerService
{
    private final Log log = LogFactory.getLog(ContentServerServiceImpl.class);

    private ContentHistoryManager historyBean;

    public ContentHistoryManager getHistoryBean()
    {
        return historyBean;
    }

    public void setHistoryBean(ContentHistoryManager historyBean)
    {
        this.historyBean = historyBean;
    }

    public void completeDeletePackageRequest(RemovePackagesResponse response)
    {
        log.debug("In completeDeletePackageRequest()");
    }

    public void completeDeployPackageRequest(DeployPackagesResponse response)
    {
        log.debug("In completeDeployPackageRequest()");
    }

    public void completeRetrievePackageBitsRequest(ContentServiceResponse response, InputStream contentStream)
    {
        log.debug("In completeRetrievePackageBitsRequest()");
    }

    public long downloadPackageBitsForChildResource(int parentResourceId, String resourceTypeName, PackageDetailsKey packageDetailsKey, OutputStream outputStream)
    {
        return downloadPackageBits(packageDetailsKey, outputStream);
    }

    public long downloadPackageBitsGivenResource(int resourceId, PackageDetailsKey packageDetailsKey, OutputStream outputStream)
    {
        return downloadPackageBits(packageDetailsKey, outputStream);
    }

    public long downloadPackageBitsRangeGivenResource(int resourceId, PackageDetailsKey packageDetailsKey, OutputStream outputStream, long startByte, long endByte)
    {
        log.debug("In downloadPackageBitsRangeGivenResource()");
        return 0;
    }

    public long getPackageBitsLength(int resourceId, PackageDetailsKey packageDetailsKey)
    {
        log.debug("In getPackageBitsLength()");
        long bitsLength = 0;
        String fileName = packageDetailsKey.getName();
        File file = new File(fileName);
        if (file.isFile() && file.exists())
        {
            bitsLength = file.length();
        }
        return bitsLength;
    }

    public PageList<PackageVersionMetadataComposite> getPackageVersionMetadata(int resourceId, PageControl pc)
    {
        log.debug("In getPackageVersionMetadata()");
        return null;
    }

    public String getResourceSubscriptionMD5(int resourceId)
    {
        log.debug("In getResourceSubscriptionMD5()");
        return null;
    }

    public Set<ResourcePackageDetails> loadDependencies(int requestId, Set<PackageDetailsKey> dependencyPackages)
    {
        log.debug("In loadDependencies()");
        return null;
    }

    public void mergeDiscoveredPackages(ContentDiscoveryReport report)
    {
        historyBean.addContentHistory(report);
    }

    public boolean preLoadRemoteContent(int i,org.rhq.core.domain.content.PackageDetailsKey pdk) 
    {
        return true;
    }

    private long downloadPackageBits(PackageDetailsKey packageDetailsKey, OutputStream outputStream)
    {
        log.debug("Downloading package bits for " + packageDetailsKey + "...");
        long bitSize;
        String fileName = packageDetailsKey.getName();
        File file = new File(fileName);
        InputStream inputStream = null;
        try
        {
            inputStream = new BufferedInputStream(new FileInputStream(file));
            byte[] fileBytes = new byte[(int)file.length()];
            bitSize = inputStream.read(fileBytes);
            inputStream.close();
            outputStream.write(fileBytes);
        }
        catch (IOException e)
        {
            throw new RuntimeException("Unable to download package bits for " + "" + " child Resource.", e);
        }
        finally
        {
            if (inputStream != null)
                try
                {
                    inputStream.close();
                }
                catch (IOException e)
                {
                    log.error("Failed to close input stream.", e);
                }
        }
        return bitSize;
    }
}
