/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.bean.history.operation;

import java.util.Date;

import org.rhq.core.clientapi.server.operation.OperationServerService;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.operation.OperationHistory;
import org.rhq.core.domain.operation.OperationRequestStatus;
import org.rhq.core.domain.operation.ResourceOperationHistory;
import org.rhq.core.util.exception.ExceptionPackage;

import org.jboss.on.embedded.manager.history.operation.OperationHistoryManager;

/**
 * @author Mark Spritzler
 */
public class OperationServerServiceImpl implements OperationServerService {
    private OperationHistoryManager historyBean;

    public OperationHistoryManager getHistoryBean() {
        return historyBean;
    }

    public void setHistoryBean(OperationHistoryManager historyBean) {
        this.historyBean = historyBean;
    }

    public void operationSucceeded(String jobId, Configuration results, long invocationTime, long completionTime) {
        OperationHistory history = this.historyBean.getHistory(jobId);
        history.setStatus(OperationRequestStatus.SUCCESS);
        if (history instanceof ResourceOperationHistory) {
            if (results != null)
                results.setId(history.getId());
            ((ResourceOperationHistory) history).setResults(results);
        }
    }

    public void operationFailed(String jobId, Configuration results, ExceptionPackage error, long invocationTime,
        long completionTime) {
        OperationHistory operationHistory = this.historyBean.getHistory(jobId);
        operationHistory.setStatus(OperationRequestStatus.FAILURE);
        if (operationHistory instanceof ResourceOperationHistory) {
            if (results != null)
                results.setId(operationHistory.getId());
            ((ResourceOperationHistory) operationHistory).setResults(results);
        }
        if (error != null) {
            if (error.getExceptionName().equals(UnsupportedOperationException.class.getName()))
                operationHistory.setErrorMessage("The '" + operationHistory.getOperationDefinition().getName()
                    + "' operation is not supported by Embedded Jopr: " + error.getMessage());
            else
                operationHistory.setErrorMessage(error.getStackTraceString());
        } else {
            operationHistory.setErrorMessage("Failed for an unknown reason at " + new Date(completionTime));
        }
    }

    public void operationTimedOut(String jobId, long invocationTime, long timeoutTime) {
        OperationHistory history = this.historyBean.getHistory(jobId);
        if (history.getStatus() == OperationRequestStatus.INPROGRESS) {
            history.setStatus(OperationRequestStatus.FAILURE);
            history.setErrorMessage("Timed Out");
        }
    }
}
