/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.manager.history;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

public abstract class AbstractHistoryManager<T> implements HistoryManager<T>
{

    protected Map<ResourceType, Collection<T>> historyByResourceType =
            new HashMap<ResourceType, Collection<T>>();

    protected Map<Resource, Collection<T>> historyByResource =
            new HashMap<Resource, Collection<T>>();

    protected Map<String, T> historyByJobId =
            new HashMap<String, T>();

    protected abstract String getJobId(T History);

    protected void addToMaps(T history, Resource resource, ResourceType resourceType)
    {
        // When we are able to create Requests then we will have a jobId, until then code not needed
        String jobId = getJobId(history);
        historyByJobId.put(jobId, history);
        Collection<T> listOfHistoryByResourceTypes = historyByResourceType.get(resourceType);

        if (listOfHistoryByResourceTypes == null)
        {
            listOfHistoryByResourceTypes = new ArrayList<T>();
            historyByResourceType.put(resourceType, listOfHistoryByResourceTypes);
        }

        listOfHistoryByResourceTypes.add(history);
        Collection<T> listOfHistoryByResources = historyByResource.get(resource);

        if (listOfHistoryByResources == null)
        {
            listOfHistoryByResources = new ArrayList<T>();
            historyByResource.put(resource, listOfHistoryByResources);
        }

        listOfHistoryByResources.add(history);
    }

    public T getHistory(String historyId)
    {
        return historyByJobId.get(historyId);
    }

    public Collection<T> getAllHistory()
    {
        return historyByJobId.values();
    }

    public Collection<T> getHistoryForResourceType(ResourceType resourceType)
    {
        return historyByResourceType.get(resourceType);
    }

    public Collection<T> getHistoryForResource(Resource resource)
    {
        return historyByResource.get(resource);
    }

}
