/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui;

import java.io.PrintWriter;

import org.rhq.core.pc.PluginContainer;
import org.rhq.core.pc.util.InventoryPrinter;

import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;

import org.jboss.on.embedded.manager.ResourceManagerFactory;

/**
 * A Seam component that contains actions used for debugging purposes.
 *
 * @author Ian Springer
 */
@Name("debugAction")
public class DebugAction
{
    private static final String JBMANCON_DEBUG_SYSPROP = "jbmancon.debug";

    @Out
    private boolean debug;

    public void init()
    {
        this.debug = Boolean.getBoolean(JBMANCON_DEBUG_SYSPROP);
    }

    public void dumpInventory()
    {
        System.out.println("*********************************** INVENTORY DUMP **************************************");
        InventoryPrinter.outputInventory(new PrintWriter(System.out), false);
        System.out.println("*****************************************************************************************");
    }

    public void runServerScan()
    {
        System.out.println("******************************* RUNNING SERVER SCAN... **********************************");
        PluginContainer.getInstance().getInventoryManager().executeServerScanImmediately();
    }

    public void runServiceScan()
    {
        System.out.println("******************************* RUNNING SERVICE SCAN... **********************************");
        ResourceManagerFactory.resourceManager().discoverServicesAsync();
    }

    public void testErrorPage()
    {
        try
        {
            throw new RuntimeException("Yikes!");
        }
        catch (RuntimeException re)
        {
            try
            {
                throw new IllegalStateException("Ugh!", re);
            }
            catch (IllegalStateException ise)
            {
                try
                {
                    throw new RuntimeException("Damn!", ise);
                }
                catch (NullPointerException npe)
                {
                    throw new IllegalArgumentException("Drat!", npe);
                }
            }
        }
    }
}
