/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui;

import javax.faces.application.FacesMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.faces.FacesMessages;

/**
 * ResourceAction handles all the interaction between the UI and the backend ResourceManager
 *
 * @author Jessica Sant
 * @author Charles Crouch
 */
@Name("resourceAction")
@Scope(ScopeType.EVENT)
// this is the default for Java Beans, but lets just be clear
public class ResourceAction
{
    private final Log LOG = LogFactory.getLog(ResourceAction.class);

    /**
     * facesMessages is a Seam Component that will handle the success/fail messages across a page redirect
     */
    @In
    private transient FacesMessages facesMessages;

    /**
     * the name of the column to sort by
     */
    private String sort;

    /**
     * true if the list should be sorted in ascending order, false for descending order
     */
    private boolean ascending = true;

    /**
     * Used to determine if the there are any globalMessages present. This is used to determine if the messages box
     * should be rendered (this is the panelGroup surrounding the <h:messages/> tag). I'd love to figure out how to do
     * this in the XHTML page itself, but until then... this method will work
     *
     * @return "true" if there are messages present, "false" otherwise
     */
    public String getHasGlobalMessages()
    {
        String result = "false";
        if (facesMessages != null && facesMessages.getCurrentGlobalMessages() != null)
        {
            result = (facesMessages.getCurrentGlobalMessages().size() > 0) + "";
        }
        return result;
    }

    /**
     * returns true IFF the current faces messages has only input-specific non-INFO severity messages in other words --
     * it will return false if there are any global messages, and it will return false if any of the input-specific
     * messages are of severity INFO
     *
     * @return true or false
     */
    public boolean getHasOnlyNonInfoNonGlobalMessages()
    {
        boolean result = false;

        if (facesMessages != null)
        {
            //return false if there's any global messages
            if (facesMessages.getCurrentGlobalMessages().size() == 0)
            {
                for (FacesMessage msg : facesMessages.getCurrentMessages())
                {
                    // there's no global messages, but there are other messages, so set the result to true
                    result = true;

                    // if any of the messages are INFO level, then return false
                    if (msg.getSeverity().equals(FacesMessage.SEVERITY_INFO))
                    {
                        result = false;
                        break;
                    }
                }
            }
        }
        return result;
    }

    /**
     * gets the column to be sorted by
     *
     * @return the sorting column
     */
    public String getSort()
    {
        return sort;
    }

    /**
     * sets the column to be sorted by
     *
     * @param sort the sorting column
     */
    public void setSort(String sort)
    {
        this.sort = sort;
    }

    /**
     * true if the list should be sorted in ascending order, false otherwise
     *
     * @return how the list should be sorted
     */
    public boolean isAscending()
    {
        return ascending;
    }

    /**
     * sets how the list should be sorted
     *
     * @param ascending how the list should be sorted
     */
    public void setAscending(boolean ascending)
    {
        this.ascending = ascending;
    }

}
