/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui;

import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Startup;

/**
 * A Seam component for exposing miscellaneous information specific to the current session.
 *
 * @author Ian Springer
 */
@Name("sessionInfoAction")
@Scope(ScopeType.SESSION)
@Startup
public class SessionInfoAction
{
    private final Log log = LogFactory.getLog(this.getClass());

    @Factory(value = "baseUrl")
    public String getBaseUrl()
    {
        FacesContext context = FacesContext.getCurrentInstance();
        HttpServletRequest request = (HttpServletRequest)context.getExternalContext().getRequest();
        int indexAfterContextRoot = request.getRequestURL().indexOf(request.getServletPath());
        String baseUrl = request.getRequestURL().substring(0, indexAfterContextRoot) + "/";
        HttpSession session = request.getSession();
        log.debug("Base URL for HTTP session with id " + session.getId() + ": " + baseUrl);
        return baseUrl;
    }
}
