/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui;

import javax.faces.application.FacesMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.NotNull;
import org.rhq.core.clientapi.agent.inventory.DeleteResourceResponse;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.faces.FacesMessages;

import org.jboss.on.embedded.manager.ResourceManager;
import org.jboss.on.embedded.manager.ResourceManagerFactory;

@Name("resourceCRUDAction")
@Scope(ScopeType.EVENT)
// this is the default for a JavaBean but lets just be clear
//@Scope(ScopeType.CONVERSATION) does not work here, when saving a resource you get
// org.jboss.seam.RequiredException: In attribute requires value for component: resourceCRUDAction.facesMessages
/**
 * Class which contains methods used to Create, Read, Update and Delete the configuration of a Single resource
 */
public class SingleResourceAction
{
    private static final Log log = LogFactory.getLog(SingleResourceAction.class);

    private ResourceManager resourceManager;

    @In
    private NavigationAction navigationAction;

    /**
     * facesMessages is a Seam Component that will handle the success/fail messages across a page redirect
     */
    @In
    private transient FacesMessages facesMessages;

    private Resource selectedResource;

    @Out(value = "selectedResource", required = false, scope = ScopeType.EVENT)
    public Resource getSelectedResource()
    {
        return selectedResource;
    }

    @In(value = "selectedResource", required = false)
    public void setSelectedResource(Resource selectedResource)
    {
        this.selectedResource = selectedResource;
    }

    @Out(value = "resourceType", required = false)
    public ResourceType resourceType;

    public SingleResourceAction()
    {
    }

    @Create
    public void create()
    {
        resourceManager = ResourceManagerFactory.resourceManager();
    }

    @Destroy
    public void destroy()
    {
        selectedResource = null;
    }

    @RequestParameter
    private Integer resourceId;

    public String removeResourceAction()
    {
        Resource resourceToBeDeleted = resourceManager.getResource(resourceId);
        if (resourceToBeDeleted == null)
        {
            facesMessages.add(FacesMessage.SEVERITY_ERROR, "Resource with id [" + resourceId + "] not found.");
            return FacesOutcomes.FAILURE;
        }
        return removeResourceAction(resourceToBeDeleted);
    }

    /**
     * calls the appropriate manager to remove this resource
     *
     * @param resource the resource to be removed
     * @return a string representing the outcome of this action (Constants.OUTCOME_SUCCESS, Constants.OUTCOME_FAILURE,
     *         etc)
     */
    public String removeResourceAction(@NotNull Resource resource)
    {
        log.debug("Removing Resource " + resource + "...");
        DeleteResourceResponse deleteResourceResponse = resourceManager.removeResource(resource);
        switch (deleteResourceResponse.getStatus())
        {
            case SUCCESS:
                facesMessages.addFromResourceBundle(FacesMessage.SEVERITY_INFO, "resource.status.remove.success",
                        resource.getResourceType().getName(), resource.getName());
                return FacesOutcomes.SUCCESS;
            default:
                String details = (deleteResourceResponse.getErrorMessage() != null) ?
                        deleteResourceResponse.getErrorMessage() : "Check server log for more details.";
                facesMessages.addFromResourceBundle(FacesMessage.SEVERITY_ERROR, "resource.status.remove.failure",
                        resource.getResourceType().getName(), resource.getName(), details);
                return FacesOutcomes.FAILURE;
        }
    }

    // TODO just a test
    @Observer("org.jboss.seam.validationFailed")
    public void validationFailed()
    {
        log.info("validation failed");
    }
}
