/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.configuration.resource;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.definition.ConfigurationDefinition;
import org.rhq.core.gui.RequestParameterNameConstants;
import org.rhq.core.gui.util.FacesContextUtility;

/**
 * This is a JSF managed bean for the resource configuration view and edit pages. The contents of the page are
 * dynamically generated based on the ConfigurationDefinition (metadata) and Configuration (data) for a specified
 * resource.
 * <p/>
 * REQUEST PARAMETERS: <li>id - a {@link org.rhq.core.domain.resource.Resource} id</li> <li>listName - the name of a
 * list property within the Configuration (only required if the mapPanel property is being accessed)</li> <li>listIndex
 * - the index of a map property within the above list property (only required if the mapPanel property is being
 * accessed)</li>
 *
 * @author Ian Springer
 */
public abstract class AbstractResourceConfigurationUIBean
{

    protected Configuration configuration;
    protected ConfigurationDefinition configurationDefinition;

    @Nullable
    public ConfigurationDefinition getConfigurationDefinition()
    {
        return this.configurationDefinition;
    }

    @Nullable
    public Configuration getConfiguration()
    {
        return this.configuration;
    }

    public void setConfigurationDefinition(@NotNull ConfigurationDefinition configurationDefinition)
    {
        this.configurationDefinition = configurationDefinition;
    }

    public void setConfiguration(Configuration configuration)
    {
        if (this.configuration == null)
        {
            this.configuration = configuration;
        }
    }

    @Nullable
    protected Integer getConfigurationId()
    {
        String configIdString = FacesContextUtility.getOptionalRequestParameter(RequestParameterNameConstants.CONFIG_ID_PARAM);
        return configIdString != null ? Integer.parseInt(configIdString) : null;
    }

    public String getNullConfigurationDefinitionMessage()
    {
        return "This resource does not expose a configuration.";
    }

    public String getNullConfigurationMessage()
    {
        return "This resource's configuration is null or has not been initialized.";
    }
}
