/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.configuration.resource;

import javax.faces.application.FacesMessage;

import org.rhq.core.domain.configuration.PropertyMap;
import org.rhq.core.domain.configuration.PropertySimple;
import org.rhq.core.gui.RequestParameterNameConstants;
import org.rhq.core.gui.util.FacesContextUtility;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.faces.FacesMessages;

/**
 * @author Ian Springer
 */
@Name("addNewOpenMap")
@Scope(ScopeType.EVENT)
public class AddNewOpenMapMemberPropertyToResourceConfigurationUIBean
{

    private static final String SUCCESS_OUTCOME = "success";
    private static final String FAILURE_OUTCOME = "failure";

    private String propertyName;
    private String propertyValue;

    @In(required = false)
    private transient FacesMessages facesMessages;

    @In(value = "resourceConfigurationUIBean", required = false)
    private ResourceConfigurationUIBean configUIBean;

    public String getPropertyName()
    {
        return propertyName;
    }

    public void setPropertyName(String propertyName)
    {
        this.propertyName = propertyName;
    }

    public String getPropertyValue()
    {
        return propertyValue;
    }

    public void setPropertyValue(String propertyValue)
    {
        this.propertyValue = propertyValue;
    }

    public void addProperty()
    {

        // We assume the config managed bean is in session, otherwise the changes we make here will be lost when the
        // user gets redirected back to the main config page.
        String mapName = FacesContextUtility.getRequiredRequestParameter(RequestParameterNameConstants.MAP_NAME_PARAM);
        PropertyMap propertyMap = configUIBean.getConfiguration().getMap(mapName);
        if (propertyMap == null)
        {
            facesMessages.addFromResourceBundle(FacesMessage.SEVERITY_ERROR, "resource.configuration.no.map.found", mapName);
            return;
        }
        PropertySimple propertySimple = new PropertySimple(this.propertyName, this.propertyValue);
        propertyMap.put(propertySimple);
        //clearProperty();
    }

    public void clearProperty()
    {
        setPropertyName(null);
        setPropertyValue(null);
    }

    public String cancel()
    {
        return SUCCESS_OUTCOME;
    }

}
