/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.configuration.resource;

import java.util.LinkedHashMap;
import java.util.Map;

import org.rhq.core.clientapi.agent.configuration.ConfigurationUtility;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.definition.ConfigurationTemplate;
import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Begin;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.web.RequestParameter;

import org.jboss.on.embedded.bean.ConfigTemplateSelectUtil;
import org.jboss.on.embedded.manager.ResourceManager;
import org.jboss.on.embedded.manager.ResourceManagerFactory;
import org.jboss.on.embedded.ui.NavigationAction;
import org.jboss.on.embedded.ui.nav.JONTreeNode;
import org.jboss.on.embedded.ui.nav.SubCategoryTreeNode;
import org.jboss.on.embedded.ui.nav.TreeNodeWithResourceType;

/**
 * @author Mark Spritzler
 */
@Name("templateDropDownPopulator")
public class TemplateDropDownPopulator
{
    private Map<String, String> templateMap;
    private ResourceManager configurationManager = ResourceManagerFactory.resourceManager();

    private String selectedTemplate;

    @In(value = "navigationAction", required = false)
    NavigationAction navigationAction;

    @Out(value = "serviceType", required = false, scope = ScopeType.CONVERSATION)
    private ResourceType serviceType;

    @Out(value = "templateConfiguration", required = false, scope = ScopeType.CONVERSATION)
    private Configuration template;

    @Out(value = "ancestorResource", required = false, scope = ScopeType.CONVERSATION)
    private Resource ancestorResource;

    /**
     * Takes the injected selectedTemplate value and uses it to look up the corresponding configuration which it then
     * outjects
     */
    public void populateFromTemplate()
    {
        String selectedTemplateName = ConfigTemplateSelectUtil.getTemplateName(selectedTemplate);
        String selectedResourceTypeName = ConfigTemplateSelectUtil.getResourceTypeName(selectedTemplate);

        // Grab the currently selected node and find the closest parent resource
        // in the hierarchy and store it away in the conversation.
        // We can grab it later to tell the ResourceManager what resource
        // to use as parent when creating this new resource.
        JONTreeNode currentNode = navigationAction.getSelectedNode();
        ancestorResource = currentNode.getClosestResource();

        serviceType = configurationManager.getResourceType(selectedResourceTypeName);
        Map<String, ConfigurationTemplate> templates = serviceType.getResourceConfigurationDefinition().getTemplates();
        Configuration originalTemplate = templates.get(selectedTemplateName).getConfiguration();

        // TODO this should be handled elsewhere
        // currently this is not a copy so we have to clone it so we dont go around updating the original one
        // associated with this configurtion definition
        template = originalTemplate.deepCopy();

        // populate the empty elements in the template with
        ConfigurationUtility.normalizeConfiguration(template, serviceType.getResourceConfigurationDefinition());
    }

    public String getSelectedTemplate()
    {
        return selectedTemplate;
    }

    public void setSelectedTemplate(String selectedTemplate)
    {
        this.selectedTemplate = selectedTemplate;
    }

    // the path of the selected category, e.g. path of the JMS Destination category
    @RequestParameter
    private String selectedSubCategoryPath;

    @Begin
    public void getTemplateDropDownEntriesBySubCategory()
    {

        templateMap = new LinkedHashMap<String, String>();
        templateMap.put("Select Template", "");

        JONTreeNode treeNode = navigationAction.findNode(selectedSubCategoryPath);
        if (treeNode instanceof SubCategoryTreeNode)
        {
            SubCategoryTreeNode subCategoryNode = (SubCategoryTreeNode)treeNode;
            for (JONTreeNode childNode : subCategoryNode.getChildNodes())
            {
                if (childNode instanceof TreeNodeWithResourceType)
                {
                    TreeNodeWithResourceType nodeWithResourceType = (TreeNodeWithResourceType)childNode;
                    ResourceType resourceType = nodeWithResourceType.getResourceType();
                    Map<String, ConfigurationTemplate> templates = resourceType.getResourceConfigurationDefinition().getTemplates();
                    ConfigTemplateSelectUtil.populateSelectItemsMap(templateMap, templates.keySet(), resourceType.getName());
                }
            }
        }
        else
        {
            throw new IllegalArgumentException("Path [" + selectedSubCategoryPath + "] should point to a SubCategoryTreeNode but did not.");
        }
    }

    @RequestParameter
    String selectedResourceTypeName;

    @Begin
    public void getTemplateDropDownEntriesByResourceType()
    {
        templateMap = new LinkedHashMap<String, String>();
        templateMap.put("Select Template", "");

        // this needs to work if we pass in a selectedResourceType
        ResourceType serviceType = configurationManager.getResourceType(selectedResourceTypeName);
        Map<String, ConfigurationTemplate> templates = serviceType.getResourceConfigurationDefinition().getTemplates();
        ConfigTemplateSelectUtil.populateSelectItemsMap(templateMap, templates.keySet(), serviceType.getName());
    }

    @Out(required = false, scope = ScopeType.CONVERSATION)
    public Map getTemplateMap()
    {
        return templateMap;
    }

    @In(required = false)
    public void setTemplateMap(Map<String, String> templateMap)
    {
        this.templateMap = templateMap;
    }

}
