/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.content;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import javax.faces.application.FacesMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.jboss.seam.annotations.In;
import org.jboss.seam.faces.FacesMessages;

import org.jboss.on.embedded.ui.BootstrapAction;
import org.jboss.on.embedded.ui.NavigationAction;

/**
 * @author Ian Springer
 */
public abstract class AbstractFileUploadAction
{
    protected static final String SUCCESS_OUTCOME = "success";
    protected static final String FAILURE_OUTCOME = "failure";

    protected static final String PACKAGE_ARCHITECTURE = "no-arch";

    private final Log log = LogFactory.getLog(this.getClass());

    private String fileName;
    private byte[] file;
    private String fileContentType;

    /**
     * FacesMessages is a Seam Component that will handle the success/fail messages across a page redirect.
     */
    @In
    protected transient FacesMessages facesMessages;

    @In
    protected NavigationAction navigationAction;

    @In
    private BootstrapAction bootstrapAction;

    protected File writeTempFile()
            throws IOException
    {
        File tempFile;
        File tempDir = this.bootstrapAction.getTempDir();
        File uploadsDir = new File(tempDir, "uploads");
        try
        {
            uploadsDir.mkdirs();

						String baseName = new File(this.fileName).getName();
            tempFile = new File(uploadsDir, baseName);

            log.debug("Writing uploaded file to '" + tempFile + "'...");
            FileOutputStream fos = new FileOutputStream(tempFile);
            fos.write(this.file);
            log.debug("Wrote " + this.file.length + " bytes to '" + tempFile + "'.");
            fos.close();
        }
        catch (IOException e)
        {
            facesMessages.addFromResourceBundle(FacesMessage.SEVERITY_ERROR,
                    "content.resourceInstance.create.unableToCreateTempFile");
            log.error("Unable to save uploaded file to temp directory " + tempDir + ".", e);
            throw e;
        }
        return tempFile;
    }

    public String getFileName()
    {
        return fileName;
    }

    public void setFileName(String fileName)
    {
        this.fileName = fileName;
    }

    public byte[] getFile()
    {
        return file;
    }

    public void setFile(byte[] file)
    {
        this.file = file;
    }

    public String getFileContentType()
    {
        return fileContentType;
    }

    public void setFileContentType(String fileContentType)
    {
        this.fileContentType = fileContentType;
    }
}
