/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import java.util.Collection;

import org.rhq.core.domain.resource.Resource;
import org.richfaces.model.TreeNode;

import org.jboss.on.embedded.ui.SummaryAction;
import org.jboss.on.embedded.ui.SummaryAction.SummaryActionOutcome;

/**
 * Interface which adds methods on top of TreeNode Aim is to avoid using direct references to the richfaces TreeNode
 * class
 */
public interface JONTreeNode extends TreeNode
{
    // helps avoid spreading TreeNode imports to clients
    JONTreeNode getParent();

    // this is the infinitely more useful version of TreeNode.getChildren()
    // That method actually must return an Iterator of type Iterator<Map.Entry<Object, TreeNode>>
    // where as this method returns what everyone other than the richfaces tree renderer wants:
    // a Collection of child JONTreeNodes
    Collection<JONTreeNode> getChildNodes();

    String getPath();

    String getName();

    String getDescription();

    void reInitializeChildrenMap();

    /**
     * Returns one of SummaryAction's inner classes to indicate what sort of summary page should be displayed when the
     * user clicks on a particular type of node
     */
    SummaryActionOutcome getSummaryActionOutcome(SummaryAction action);

    /**
     * Returns the closest resource associated to this node, i.e. the resource associated directly with the node, or the
     * resource associated with its parent node
     */
    Resource getClosestResource();

    // TODO this should be moved off of here
    JONTreeNode findNode(String path);
}
