/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import java.util.Set;

import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

import org.jboss.on.embedded.ui.SummaryAction;
import org.jboss.on.embedded.ui.SummaryAction.SummaryActionOutcome;

/**
 * Class representing a node in the nav tree which holds a resource type which is not a singleton.
 * <p/>
 * Supports having zero or more of each of the following as child nodes a) ResourceTreeNode, for resources associated
 * with this resource type
 *
 * @author ccrouch
 */
public class NonSingletonResourceTypeTreeNode extends ResourceTypeTreeNode
{
    public NonSingletonResourceTypeTreeNode(String key, ResourceType type, Resource parentResource)
    {
        super(key, type, parentResource);
    }

    protected void initChildrenMap()
    {
        // add as child nodes all resources of the selected type, which are also children of the parentResource
        Set<Resource> resources = getResourceManager().getResources(getResourceType(), getParentResource());
        for (Resource resource : resources)
        {
            addChild(new ResourceTreeNode(getUniqueKey(), resource));
        }
    }

    @Override
    public String toString()
    {
        return getPath();
    }

    public String getPath()
    {
        String resourceTypeName = getResourceType().getName();
        return getParent() == null ? resourceTypeName :
                getParent().getPath() + "/" + resourceTypeName;
    }

    public String getName()
    {
        return pluralize(getResourceType().getName());
    }

    public SummaryActionOutcome getSummaryActionOutcome(SummaryAction action)
    {
        return action.new NonSingletonResourceTypeOutcome(getResourceType(), getParentResource());
    }

    private static String pluralize(String noun)
    {
        if (noun.endsWith("y"))
            return noun.substring(0, noun.length() - 1) + "ies";
        else if (noun.endsWith("s"))
            return noun;
        else
            return noun + "s";
    }
}
