/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import java.util.Set;

import org.rhq.core.domain.resource.Resource;

/**
 * Class representing a node in the nav tree which holds a platform
 * <p/>
 * Supports having zero or more of each of the following as child nodes a) ResourceTypeNode, to capture child resources
 * of this platform that aren't defined in its plugin descriptor e.g. JBAS servers b) other child nodes as specified by
 * ResourceTreeNode, e.g. to pick up platform services
 *
 * @author ccrouch
 */
public class PlatformResourceTreeNode extends ResourceTreeNode {
    public PlatformResourceTreeNode(Resource resource) {
        super(getUniqueKey(), resource);
    }

    protected void initChildrenMap()
    {
        // Server resources, e.g. JBAS, hang directly off the platform without 
        // an intervening resource type.  In order to present these in a "nice way"
        // in the nav, we add nodes for the resource types of each of the child
        // resources underneath the platform, rather than just the resources themselves.
        // The resources in turn will show up under their corresponding resource type nodes
        Set<Resource> childResources = getResource().getChildResources();
        for (Resource childResource : childResources)
        {
            addChildResourceTypeNode(childResource.getResourceType(), getResource());
        }

        // do the standard initialization for a ResourceTreeNode
        super.initChildrenMap();
    }
}
