/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import org.jetbrains.annotations.NotNull;
import org.rhq.core.domain.resource.Resource;

import org.jboss.on.embedded.ui.SummaryAction;
import org.jboss.on.embedded.ui.SummaryAction.SummaryActionOutcome;

/**
 * Class representing a node in the nav tree which holds any non-platform resource.
 * <p/>
 * Supports having zero or more of each of the following as child nodes a) ResourceTypeTreeNode, for child types of this
 * resource's type b) SubCategoryTreeNode, for sub categories defined on this resource's type
 *
 * @author ccrouch
 */
public class ResourceTreeNode extends BaseTreeNode implements TreeNodeWithResource
{
    private Resource resource;

    public ResourceTreeNode(@NotNull String key, @NotNull Resource resource)
    {
        super(key);
        //noinspection ConstantConditions
        if (resource == null)
        {
            throw new IllegalArgumentException("ResourceTreeNode requires a Resource - null was passed into the constructor.");
        }
        this.resource = resource;
    }

    protected void initChildrenMap()
    {
        addChildResourceTypeOrSubCategoryNodes(getResource().getResourceType(), getResource());
    }

    public Resource getClosestResource()
    {
        return resource;
    }

    public String getPath()
    {
        // TODO can't change this without changing the corresponding methods on NavigationAction
        // e.g. getResourcePath(Integer resourceId), and the places that call that, e.g. resourceTypeSummary.xhtml
        return String.valueOf(getResource().getId());
    }

    public String getName()
    {
        return getResource().getName();
    }

    public String getDescription()
    {
        return getResource().getDescription();
    }

    public Resource getResource()
    {
        return this.resource;
    }

    public SummaryActionOutcome getSummaryActionOutcome(SummaryAction action)
    {
        return action.new ResourceOutcome(this.resource);
    }

    @Override
    public String toString()
    {
        return getPath();
    }
}
