/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

/**
 * Abstract class representing a node in the nav tree which holds a resource type.
 * <p/>
 * Which child nodes are supported is determined by the subclasses
 *
 * @author ccrouch
 */
public abstract class ResourceTypeTreeNode extends BaseTreeNode implements TreeNodeWithResourceType {
    private ResourceType resourceType;

    // The closest ancestor resource to this resource type.
    // This is used to make sure that when we retrieve resources matching the given resourceType
    // that we only return those that are children of the given parentResource
    private Resource parentResource;

    public ResourceTypeTreeNode(String key, ResourceType type, Resource parentResource) {
        super(key);
        if (type == null) {
            throw new IllegalArgumentException(
                "ResourceTypeTreeNode requires a ResourceType. Null was passed into the constructor");
        }
        if (parentResource == null) {
            throw new IllegalArgumentException(
                "ResourceTypeTreeNode requires a Resource. Null was passed into the constructor");
        }
        setResourceType(type);
        this.parentResource = parentResource;
    }

    public Resource getClosestResource() {
        return parentResource;
    }

    public void setResourceType(ResourceType resourceType) {
        this.resourceType = resourceType;
    }

    public ResourceType getResourceType() {
        return this.resourceType;
    }

    public String getDescription() {
        return getResourceType().getDescription();
    }

    public Resource getParentResource() {
        return parentResource;
    }
}
