/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

package org.jboss.on.embedded.ui.nav;

import org.rhq.core.domain.resource.Resource;
import org.rhq.core.domain.resource.ResourceType;

import org.jboss.on.embedded.ui.SummaryAction;
import org.jboss.on.embedded.ui.SummaryAction.SummaryActionOutcome;

/**
 * Class representing a node in the nav tree which holds a resource type which is a singleton.
 * <p/>
 * Supports having zero or more of each of the following as child nodes a) ResourceTypeTreeNode, for child types of this
 * resource's type b) SubCategoryTreeNode, for sub categories defined on this resource's type
 *
 * @author ccrouch
 */
public class SingletonResourceTypeTreeNode extends ResourceTypeTreeNode implements TreeNodeWithResource
{
    private Resource singletonInstance;

    public SingletonResourceTypeTreeNode(String key, ResourceType type, Resource singletonInstance,
                                         Resource parentResource)
    {
        super(key, type, parentResource);
        this.singletonInstance = singletonInstance;
    }

    protected void initChildrenMap()
    {
        addChildResourceTypeOrSubCategoryNodes(getResourceType(), getResource());
    }

    public Resource getClosestResource()
    {
        return singletonInstance;
    }

    public Resource getResource()
    {
        return singletonInstance;
    }

    @Override
    public String toString()
    {
        return getPath();
    }

    public String getPath()
    {
        return String.valueOf(singletonInstance.getId());
    }

    public String getName()
    {
        return singletonInstance.getName();
    }

    public SummaryActionOutcome getSummaryActionOutcome(SummaryAction action)
    {
        return action.new ResourceOutcome(this.singletonInstance);
    }
}
