/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.util;

import java.util.Set;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jetbrains.annotations.Nullable;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.definition.ConfigurationDefinition;
import org.rhq.core.domain.content.PackageType;
import org.rhq.core.domain.resource.ResourceCreationDataType;
import org.rhq.core.domain.resource.ResourceType;

/**
 * @author Ian Springer
 */
public abstract class ContentUtility
{
    private static final Log LOG = LogFactory.getLog(ContentUtility.class);

    private ContentUtility()
    {
    }

    public static boolean isContentBacked(ResourceType resourceType)
    {
        return resourceType.isCreatable() && resourceType.getCreationDataType() == ResourceCreationDataType.CONTENT;
    }

    @Nullable
    public static PackageType getCreationPackageType(ResourceType resourceType)
    {
        PackageType creationPackageType = null;
        Set<PackageType> packageTypes = resourceType.getPackageTypes();
        for (PackageType packageType : packageTypes)
        {
            if (packageType.isCreationData())
            {
                creationPackageType = packageType;
                break;
            }
        }
        LOG.debug("Creation package type for " + resourceType + " is " + creationPackageType + ".");
        return creationPackageType;
    }

    @Nullable
    public static Configuration getDefaultDeploymentConfiguration(PackageType packageType)
    {
        ConfigurationDefinition configurationDefinition = packageType.getDeploymentConfigurationDefinition();
        Configuration configuration = null;
        if (configurationDefinition != null)
        {
            if (configurationDefinition.getDefaultTemplate() != null)
                //noinspection ConstantConditions
                configuration = configurationDefinition.getDefaultTemplate().getConfiguration();
            else
                configuration = new Configuration();
        }
        return configuration;
    }
}
