/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.manager.test;

import org.rhq.core.clientapi.agent.configuration.ConfigurationUtility;
import org.rhq.core.domain.configuration.Configuration;
import org.rhq.core.domain.configuration.PropertyList;
import org.rhq.core.domain.configuration.PropertyMap;
import org.rhq.core.domain.configuration.PropertySimple;
import org.rhq.core.domain.configuration.definition.ConfigurationDefinition;
import org.rhq.core.domain.configuration.definition.PropertyDefinitionList;
import org.rhq.core.domain.configuration.definition.PropertyDefinitionMap;
import org.rhq.core.domain.configuration.definition.PropertyDefinitionSimple;
import org.rhq.core.domain.configuration.definition.PropertySimpleType;
import org.testng.annotations.Test;

/**
 * @author Jason Dobies
 */
public class TemplatePopulatorTest
{
    // Test Cases  --------------------------------------------

    @Test
    public void testPopulateEmptyConfiguration()
    {
        Configuration configuration = new Configuration();
        ConfigurationDefinition definition = createDefinition();

        ConfigurationUtility.normalizeConfiguration(configuration, definition);

        int numDefinitions = definition.getPropertyDefinitions().keySet().size();
        assert configuration.getProperties().size() == numDefinitions :
                "Incorrect number of properties in the value set. Expected: " + numDefinitions + "  Found: " + configuration.getProperties().size();

        /* Asserts both that the properties were created as simple properties and that they are null
         If the properties were built as the wrong type, a ClassCastException will arise.
        */
        assert configuration.getSimple("property1").getStringValue() == null;
        assert configuration.getSimple("property2").getStringValue() == null;
        assert configuration.getSimple("property3").getStringValue() == null;
        assert configuration.getSimple("property4").getStringValue() == null;
        assert configuration.getSimple("property5").getStringValue() == null;
        assert configuration.getSimple("property6").getStringValue() == null;
        assert configuration.getSimple("property7").getStringValue() == null;

        assert configuration.getMap("property20") != null;
        assert configuration.getList("property30") != null;
    }

    @Test
    public void testPopulateNonEmptyConfiguration()
    {
        Configuration configuration = new Configuration();
        configuration.put(new PropertySimple("property1", "foo"));

        PropertyMap map = new PropertyMap("property20");
        map.put(new PropertySimple("map1", "foo"));
        configuration.put(map);

        PropertyList list = new PropertyList("property30");
        list.add(new PropertySimple("list1", "foo"));
        configuration.put(list);

        ConfigurationDefinition definition = createDefinition();

        ConfigurationUtility.normalizeConfiguration(configuration, definition);

        int numDefinitions = definition.getPropertyDefinitions().keySet().size();
        assert configuration.getProperties().size() == numDefinitions :
                "Incorrect number of properties in the value set. Expected: " + numDefinitions + "  Found: " + configuration.getProperties().size();

        /* Asserts both that the properties were created as simple properties and that they are null
         If the properties were built as the wrong type, a ClassCastException will arise.
        */
        assert configuration.getSimple("property1").getStringValue().equals("foo");
        assert configuration.getSimple("property2").getStringValue() == null;
        assert configuration.getSimple("property3").getStringValue() == null;
        assert configuration.getSimple("property4").getStringValue() == null;
        assert configuration.getSimple("property5").getStringValue() == null;
        assert configuration.getSimple("property6").getStringValue() == null;
        assert configuration.getSimple("property7").getStringValue() == null;

        /* Make sure the populator didn't blow away the values already in these properties. */
        map = configuration.getMap("property20");
        assert map.get("map1") != null;

        list = configuration.getList("property30");
        assert list.getList().size() == 1;
    }

    // Private  --------------------------------------------

    private ConfigurationDefinition createDefinition()
    {
        ConfigurationDefinition definition = new ConfigurationDefinition("Test Definition", "Test Definition");
        definition.put(new PropertyDefinitionSimple("property1", "", true, PropertySimpleType.STRING));
        definition.put(new PropertyDefinitionSimple("property2", "", true, PropertySimpleType.INTEGER));
        definition.put(new PropertyDefinitionSimple("property3", "", true, PropertySimpleType.FLOAT));
        definition.put(new PropertyDefinitionSimple("property4", "", true, PropertySimpleType.LONG_STRING));
        definition.put(new PropertyDefinitionSimple("property5", "", true, PropertySimpleType.PASSWORD));
        definition.put(new PropertyDefinitionSimple("property6", "", true, PropertySimpleType.LONG));
        definition.put(new PropertyDefinitionSimple("property7", "", true, PropertySimpleType.DOUBLE));

        definition.put(new PropertyDefinitionMap("property20", "", true, null));

        definition.put(new PropertyDefinitionList("property30", "", true, null));

        return definition;
    }

}
