/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.ui.test;

import java.util.List;

import org.rhq.core.domain.operation.OperationDefinition;
import org.rhq.core.domain.operation.OperationHistory;
import org.rhq.core.domain.resource.Resource;
import org.testng.annotations.Test;

import org.jboss.on.embedded.ResourceTypes;
import org.jboss.on.embedded.util.EmbeddedSeamHelper;

/**
 * @author Mark Spritzler
 */
public class SingleResourceOperationTest extends EmbeddedSeamHelper
{
    @Override
    protected String getScenario()
    {
        return "scenario2.xml";
    }

    private List<OperationDefinition> operations;

    @Test
    public void testGetOperations() throws Exception
    {
        final Resource resource = getResourceInstance(ResourceTypes.RESOURCE_TYPE_XA_DATASOURCE, "datasourceThree");
        assert resource != null : "DataSource Three was not found and it should have been.";

        // It would have been nice to do:
        //new NonFacesRequest("/secure/resourceInstanceOperation.xhtml")
        // but need a way to setup the #{navigationAction.currentPath}
        new FacesRequest()
        {
            @Override
            protected void updateModelValues() throws Exception
            {
                setValue("#{navigationAction.currentPath}", String.valueOf(resource.getId()));
            }

            @Override
            protected void invokeApplication() throws Exception
            {
                invokeMethod("#{operationAction.initOperationsAndHistory}");
            }

            @Override
            protected void renderResponse() throws Exception
            {

                operations = (List<OperationDefinition>)getInstance("operations");
                assert operations != null : "Should return non-null operations";
                assert operations.size() > 0 : "Should return non-empty operations";
            }
        }.run();
    }

    // tests don't pass with dependsOnMethods present, JBSEAM-2519
    // even without dependsOnMethods this test is failing with
    // java.lang.IllegalStateException: Could not commit transaction
    //@Test //(dependsOnMethods={"testGetOperations"})
    public void testInvokeOperation() throws Exception
    {
        final Resource resource = getResourceInstance(ResourceTypes.RESOURCE_TYPE_XA_DATASOURCE, "datasourceThree");
        assert resource != null : "DataSource Three was not found and it should have been.";

        new FacesRequest()
        {

            @Override
            protected void updateModelValues() throws Exception
            {
                setValue("#{navigationAction.currentPath}", String.valueOf(resource.getId()));
                if (operations != null && operations.size() > 0)
                {
                    OperationDefinition action = operations.get(0);
                    setParameter("operation", action.getName());
                }
                else
                {
                    assert false : "No Operations were found for this resource";
                }
            }

            @Override
            protected void invokeApplication() throws Exception
            {
                // to match whats in pages.xml this has (), not sure if its necessary here
                invokeMethod("#{operationAction.invokeOperation()}");
            }


            @Override
            protected void renderResponse() throws Exception
            {
                List<OperationHistory> operationHistory = (List<OperationHistory>)getInstance("operationHistory");
                assert operationHistory.size() > 0 : "There should be something in OperationHistory";
            }
        }.run();
    }

    // tests don't pass with dependsOnMethods present, JBSEAM-2519
    //@Test//(dependsOnMethods = {"testInvokeOperation"})
    public void testGetHistory() throws Exception
    {
        final Resource resource = getResourceInstance(ResourceTypes.RESOURCE_TYPE_XA_DATASOURCE, "datasourceThree");
        assert resource != null : "DataSource Three was not found and it should have been.";

        new FacesRequest()
        {

            @Override
            protected void updateModelValues() throws Exception
            {
                setValue("#{navigationAction.currentPath}", String.valueOf(resource.getId()));
            }

            @Override
            protected void invokeApplication() throws Exception
            {
                invokeMethod("#{operationAction.getOperationsAndHistory}");
            }


            @Override
            protected void renderResponse() throws Exception
            {
                List<OperationHistory> operationHistory = (List<OperationHistory>)getInstance("operationHistory");
                assert operationHistory.size() > 0 : "There should be something in OperationHistory";
            }
        }.run();
    }
}
