/*
 * Embedded Jopr Project
 * Copyright (C) 2006-2009 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.jboss.on.embedded.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.rhq.core.domain.resource.Resource;
import org.testng.annotations.AfterClass;
import org.testng.annotations.BeforeClass;

import org.jboss.seam.mock.SeamTest;

import org.jboss.on.embedded.manager.ResourceManager;

/**
 * @author mspritzler
 */
public class EmbeddedSeamHelper extends SeamTest
{
    private static final Log LOG = LogFactory.getLog(EmbeddedSeamHelper.class);

    // can't use inheritance here, so use composition instead
    private EmbeddedPluginContainerHelper pluginContainerHelper;

    public EmbeddedSeamHelper()
    {
        pluginContainerHelper = new EmbeddedPluginContainerHelper()
        {
            protected String getScenario()
            {
                return EmbeddedSeamHelper.this.getScenario();
            }
        };
    }

    @Override
    @BeforeClass
    public void setupClass() throws Exception
    {
        // this is required to get vfs URLs (used by JBoss embeddded) recognized
        // while executing the test
        //org.jboss.net.protocol.URLStreamHandlerFactory factory = new org.jboss.net.protocol.URLStreamHandlerFactory();
        //URL.setURLStreamHandlerFactory(factory);

        // set up which scenario gets loaded
        pluginContainerHelper.initializeScenario();

        // do all the SeamTest set up
        super.setupClass();
    }

    // TODO once JBMANCON-183 is resolved this shouldn't be needed, we'll just
    // rely on what TestBootstrapAction does
    @AfterClass
    public void shutdownPluginContainer()
    {
        pluginContainerHelper.shutdownPluginContainer();
    }

    public ResourceManager getManager()
    {
        return pluginContainerHelper.getManager();
    }

    protected Resource getResourceInstance(String resourceTypeName, String resourceInstanceName)
    {
        return pluginContainerHelper.getResourceInstance(resourceTypeName, resourceInstanceName);
    }

    protected String getScenario()
    {
        return pluginContainerHelper.getScenario();
    }
}
