/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit;

public interface AppConstants extends EmbJoprTestConstants {


	
		public enum ResourceTypes {
			
		}

		/**
		 * Information about deployable types -
		 * navigation tree labels, archive extensions, 
		 * XML descriptor suffixes, and MIME type.
		 */
		public enum DeployableTypes {

			// Nav tree link label, EmbJopr display name, testdata/$dir, extension, suffix, mime type.
			EAR(AppConstants.NAV_EAR,         "EARs",           "ear",   ".ear", null,  "application/ear"),
			WAR(AppConstants.NAV_WAR,         "WARs	",           "war",   ".war", null,  "application/war"),
			EJB2(AppConstants.NAV_EJB2,       "EJB2 JARs",      "ejb2",  ".jar", null,  "application/java-archive"),
			EJB3(AppConstants.NAV_EJB3,       "EJB3 JARs",      "ejb3",  ".jar", null,  "application/java-archive"),
			SAR(AppConstants.NAV_SAR,         "SARs",           "sar",   ".sar", "-service.xml", "application/sar"),
			RAR(AppConstants.NAV_RAR,         "RARs",           "rar",   ".rar", null,   "application/rar"),
			EMB_RAR(AppConstants.NAV_EMB_RAR, "Embedded RARs",  "",      ".rar", null, "application/rar"),
			MC_BEAN(AppConstants.NAV_MC,      "?",             "mc",    "",     null,   "application/java-archive"),
			EMB_WAR(AppConstants.NAV_EMB_WAR, "Embbedded WARs", "",      ".war", null, "application/war"),
      
			// TODO: Create a superclass for non-app resource types - ResourceTypes.
			JBCACHE(AppConstants.NAV_JBCACHE, "JBoss Caches",   "jbcache", null,   null,  null),
			HBN(AppConstants.NAV_HBN,         "Hibernate",     "hbn",     null,   null,  null);


			static{
				EJB2.jmxMBeanNameFormat = "jboss.j2ee:module=\"%s\",service=EjbModule,*";
				EJB3.jmxMBeanNameFormat = "jboss.j2ee:ear=%1$s,jar=%1$s,service=EJB3";
			}

			private String jmxMBeanNameFormat;
			private void setJmxMBeanNameFormat( String name ){ this.jmxMBeanNameFormat = name; }
			public String getJmxMBeanNameFormat(){ return this.jmxMBeanNameFormat; }
			public String formatJmxMBeanName( String deployableName ){
				if( null == this.jmxMBeanNameFormat )
					throw new UnsupportedOperationException("JMX name for "+this.toString()+" is not filled (yet).");
				return String.format(this.jmxMBeanNameFormat, deployableName);
			}



			// -- Fields --

			protected final String navTreeLabel;
			public String getNavTreeLabel() {			return navTreeLabel;		}

			protected final String displayName;
			public String getDisplayName() {			return displayName;		}

			protected final String dataDir;
			public String getDataDir() {			return dataDir;		}

			protected final String extension;
			public String getExtension() {			return extension;		}

			protected final String suffix;
			public String getSuffix() {				return suffix;			}

			protected final String mimeType;
			public String getMimeType() {			return mimeType;		}




			// -- Constructors --

			private DeployableTypes(
							String navTreeLabel,
							String displayName,
							String dataDir,
							String extension,
							String suffix, String mimeType)
			{
				this.navTreeLabel = navTreeLabel;
				this.displayName = displayName;
				this.dataDir = dataDir;
				this.extension = extension;
				this.suffix = suffix;
				this.mimeType = mimeType;
			}


		}// enum DeployableTypes
		

		/** Several ways how to deploy / undeploy. */
		public enum DeploymentMeans {
			VIA_EMBJOPR,
			VIA_FILESYSTEM,
			VIA_JMX,
			VIA_MANAGEMENTVIEW
		}



    // Navigation
		public static final String NAV_APPLICATIONS = "Applications";
    //public static final String NAV_EJB = "EJB Application (EJB JAR)s";
    public static final String NAV_EJB2 = "EJB2 JARs";
    public static final String NAV_EJB3 = "EJB3 JARs";
    public static final String NAV_EAR = "Enterprise Application (EAR)s";
    public static final String NAV_SAR = "SARs";
    public static final String NAV_MC = "MC Beans";
    public static final String NAV_PAR = "PARs";
    public static final String NAV_RAR = "Resource Adapter Archive (RAR)s";
    public static final String NAV_WAR = "Web Application (WAR)s";
    public static final String NAV_EMB_WAR = "Embedded Web Application (WAR)s";
    public static final String NAV_EMB_RAR = "Embedded Resource Adapter Archive (RAR)s";

		// Non-app resource types.
    public static final String NAV_SBM = "Service Binding Manager";
    public static final String NAV_SBM_SETS = "Service Binding Sets";
    public static final String NAV_JBCACHE = "JBoss Cache";
    public static final String NAV_JBCACHES = "JBoss Caches";
    public static final String NAV_HBN = "Hibernate";


		public static final String EAR_MALFORMED_APP_FILENAME = "malformed-application-xml.ear";
		public static final String EAR_MALFORMED_APP_FILENAME_2 = "malformed-application-xml-2.ear";
		public static final String EAR_NOT_MALFORMED_APP_FILENAME = "not-malformed-application-xml.ear";
		public static final String WAR_FILENAME_MALFORMED_APP = "malformed-web-xml.war";




    // Test Archives


		// EJB 2.x
    public static final String BASIC_EJB2 = "deployment-ejb.jar";
    public static final String BAD_EJB2_JAR = "ejbredeploy-bad.jar";
    public static final String GOOD_EJB2_JAR = "ejbredeploy-good.jar";
    public static final String REDEPLOY_EJB2_JAR = "ejbredeploy.jar";


		// EJB 3.0
    public static final String BASIC_EJB3 = "BasicEJB3.jar";
    public static final String BAD_EJB3_JAR = "BasicEJB3-bad.jar";
    public static final String GOOD_EJB3_JAR = "BasicEJB3-good.jar";
    public static final String REDEPLOY_EJB3_JAR = "BasicEJB3.jar";

		public static final String EJB3_MDB = "jboss-ejb3-tutorial-mdb.jar";
		public static final String EJB3_SLB = "stateless-test.jar";
		public static final String EJB3_SFB = "EjbStatefulCounter.jar";
		public static final String EJB3_ENTITY = "entity-test.jar";


		// EAR
    public static final String BASIC_EAR = "eardeployment.ear";
		public static final String EAR_UNPACKED = "unpacked-ear1.ear";
		public static final String EAR_UNPACKED_ZIP = "unpacked-ear1.ear.zip";
		public static final String EAR_WITH_WAR = "EarWithWarNB.ear";
		public static final String EAR_WITH_WAR__WAR_NAME = "EarWithWarNB-war.war";
		public static final String EAR_WITH_WAR_COUNTER = "EarWithWar-Counter.ear";
		public static final String EAR_WITH_WAR_COUNTER_20 = "EarWithWar-Counter-20.ear";
		public static final String EAR_EXT_DEPENDENCIES = "EarExtDependencies.ear";
		public static final String EAR_WITH_RAR = "testscopedrar.ear";
		public static final String EAR_WITH_WAR_EMB = "EarWithWarNB-embwar.ear";
		public static final String EAR_WITH_WAR_EMB__WAR_NAME = "EarWithWarNB-emb_war.war";

		// WAR
    public static final String BASIC_WAR = "hellothere.war";
    public static final String BASIC_WAR_02 = "hellothere02.war";
    public static final String BASIC_WAR_03 = "hellothere03.war";
    public static final String BASIC_WAR_04 = "hellothere04.war";
    public static final String BASIC_WAR_05 = "hellothere05.war";
    public static final String BASIC_WAR_06 = "hellothere06.war";
    public static final String BASIC_WAR_07 = "hellothere07.war";
    public static final String BASIC_WAR_08 = "hellothere08.war";
    public static final String BASIC_WAR_09 = "hellothere09.war";
    public static final String BASIC_WAR_10 = "hellothere10.war";
    public static final String BASIC_WAR_DUD = "hellothere-DUD.war";
    public static final String BASIC_WAR_DUD_RETRY = "hellothere-DUD_RETRY.war";
    public static final String BASIC_WAR_STOPSTART = "hellothere-STOP_START.war";
    public static final String BASIC_WAR_RESTART = "hellothere-RESTART.war";
    public static final String BASIC_WAR_MULTI_HOTDEPLOY = "hellothere-multi-hotdeploy.war";

		public static final String WAR_UNPACKED = "unpacked-web1.war";
		public static final String WAR_UNPACKED_ZIP = "unpacked-web1.war.zip";

		public static final String WAR_VIRTUAL_HOSTS = "vhosts-test.war";

		// RAR
    public static final String BASIC_RAR = "jbosstestadapter.rar";



		// Resource constants
		public static final String NAV_SCRIPTS = "Scripts";
		public static final String NAV_JBOSS_WEB = "JBoss Web";
  		public static final String NAV_CONNECTORS = "Connectors";
  		public static final String NAV_VIRTUAL_HOSTS = "Virtual Hosts";




    // Defaults
    public static final String[] WAR_DEFAULTS = new String[]{
			"ROOT.war", "invoker.war",
			"jbas5-admin-console.war", "jbossws-management.war",
      "jmx-console.war", "web-console.war"
		};

    public static final String[] SAR_DEFAULTS = new String[]{
			"cache-invalidation-service.xml",
			"connection-factories-service.xml", "console-mgr.sar",
			"destinations-service.xml", "ejb2-timer-service.xml",
			"hsqldb-ds.xml", "hsqldb-persistence-service.xml",
			"http-invoker.sar", "jboss-service.xml", "jms-ds.xml",
			"jmx-invoker-service.xml", "jmx-remoting.sar", "jsr88-service.xml",
			"legacy-invokers-service.xml", "legacy-service.xml",
			"mail-service.xml", "messaging-service.xml", "monitoring-service.xml",
			"properties-service.xml", "remoting-bisocket-service.xml",
			"schedule-manager-service.xml", "scheduler-service.xml",
			"sqlexception-service.xml", "transaction-service.xml", "uuid-key-generator.sar"
		};
		
		// Status Messages
		//public static final String EJB2_DEL_MSG = "Successfully deleted EJB2 JAR";
		//public static final String EJB3_DEL_MSG = "Successfully deleted EJB3 JAR";
		public static final String EJB_DEL_MSG_FORMAT = "Successfully deleted %s JAR '%s'.";
		public static final String DELETE_MSG_FORMAT = "Successfully deleted %s '%s'.";

		public static final String MSG_FORMAT_DEPLOY_SUCCESS = "Resource %s created successfully!";

}
