/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.jopr.jsfunit;

import java.io.IOException;
import java.util.*;
import javax.management.*;
import com.gargoylesoftware.htmlunit.html.*;
import org.jboss.mx.util.MBeanServerLocator;



/**
 * This is the base test class for Embedded Jopr Application tests.
 * 
 * @author Shelly McGowan
 *
 */
public abstract class ApplicationTestBaseAS5 extends EmbjoprTestCase
				implements AppConstants {


	public ApplicationTestBaseAS5() {
		super();
	}

	public ApplicationTestBaseAS5(String theName) {
		//super(theName);
	}


	public HtmlButtonInput getAppDeleteButton(String resourceName)
	{
		return getDeleteButton("resourceSummaryForm", resourceName);
	}


	/**
	 * This method should query the JMX server and decide whether the given
	 * MBean displays deployed resource.
	 *
	 * TODO: Move to EmbJoprTestCase?
	 * TODO: Consider using MBean jboss.management.local:j2eeType=J2EEServer,name=Local,
	 *       which has attribute String[] deployedObjects
	 *
	 * @param deploymentMBean  Name of the MBean to examine. Differs between AS4 and 5.
	 * @return true  if the MBean indicates that the resource is deployed, false otherwise.
	 * @throws javax.management.JMException  upon JMX related error, including invalid
	 *                 or non-existent MBean / attribute name.
	 * @throws java.io.IOException  upon I/O error.
	 */
	protected boolean isMBeanStateDeployedImpl(ObjectName mBeanName)  throws JMException, IOException
	{
		JMXUtils jmxUtils = JMXUtils.getInstanceForLocalJBoss();
		Object state = jmxUtils.getMBeanAttribute(mBeanName, "State");
		return ("DEPLOYED".equals( state.toString() ));
	}

	protected boolean isMBeanStateDeployedImpl(String mBeanName)  throws JMException, IOException {
		return isMBeanStateDeployedImpl( new ObjectName(mBeanName) );
	}


	/**
	 * Queries JBoss via JMX for given query and checks whether the
	 * StateString property of the first result is "Started".
	 *
	 * @param mBeanName
	 * @return true if the StateString property of the first MBean found is "Started".
	 * @throws javax.management.JMException
	 * @throws java.io.IOException
	 */
	protected boolean isMBeanStateDeployedByQuery(String mBeanName)  throws JMException, IOException
	{
		log.info("Querying JMX: "+mBeanName);

		ObjectName objName =	new ObjectName(mBeanName);

		MBeanServer jmxServer = MBeanServerLocator.locateJBoss();
		Set mBeans = jmxServer.queryNames(objName, null);
		if (mBeans.size() != 1) {
			log.info(mBeanName+" not found.");
			return false;
		}
		ObjectName deploymentMBean = (ObjectName) mBeans.iterator().next();
		// Returns org.jboss.deployers.spi.DeploymentState.
		Object state = jmxServer.getAttribute(deploymentMBean, "State");
		log.info("State is '"+state.toString()+"'.");
		return 
					 "DEPLOYED".equals(state.toString()) // AS 4
					 || "3".equals(state.toString())     // AS 5; StateString = "Deployed"
					 || "8".equals(state.toString())     // EJB: "Registered"
					 ;
	}



	/* EJB */
	public boolean isEJBDeployed(String ejbJarName) throws RuntimeException
	{
		try {
			//String mBeanName = "jboss.j2ee:module=\""+ejbJarName+"\",service=EjbModule";

			// jboss.j2ee:module="sessiona.jar",service=EjbModule
			String mBeanName = "jboss.j2ee:module=\""+ejbJarName+"\",*";
			// id="jboss.j2ee:module="sessiona.jar",service=EjbModule",type=Component
			//String mBeanName = "jboss.deployment:id="jboss.j2ee:module="sessiona.jar",service=EjbModule",type=Component";
			//
			// rajesh: ozizka, seems to have changed between AS 5.1.Beta1 and CR1. so the test cases should be updated i guess.
			// ear=BasicEJB3.jar,jar=BasicEJB3.jar,service=EJB3
			//String mBeanName = "jboss.j2ee:ear="+ejbJarName+",jar="+ejbJarName+",service=EJB3";
			
			return isMBeanStateDeployedByQuery(mBeanName);
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/* WAR */
	public boolean isWarDeployed(String warName) throws RuntimeException
	{
		if (warName.endsWith(".war")) {
			warName = warName.substring(0, warName.lastIndexOf(".war"));
		} else {
			warName = warName.substring(0, warName.lastIndexOf(".WAR"));
		}

		try {
			String query = "jboss.deployment:id=\"jboss.web.deployment:war=/"+warName+"\",*";
			return isMBeanStateDeployedByQuery(query);
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/* EAR */
	public boolean isEarDeployed(String earName) throws RuntimeException
	{
    //jboss.deployment:id="vfszip:/home/brq/.../jboss-5.0.0.Beta/server/all/deploy/JBossCacheTest-by-fdrabek.ear/",type=Deployment
		try {
			//String query = "jboss.deployment:id=\"vfszip:" +
			//				System.getProperty("jsfunit.deploy.dir") + "/" + earName + "/\",*";
			String query = String.format("jboss.deployment:id=\"vfszip:%s/%s/\",*",
							System.getProperty("jsfunit.deploy.dir"), earName );

			return isMBeanStateDeployedByQuery(query);
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	/* RAR */
	public boolean isRarDeployed(String rarName) throws RuntimeException
	{
		try {
			String query = "jboss.deployment:id=\"vfszip:" +
					System.getProperty("jsfunit.deploy.dir") + "/" + rarName + "/\",*";
			//String query = String.format("jboss.deployment:id=\"vfszip:%s/%s/\",*",
			//                               System.getProperty("jsfunit.deploy.dir"), rarName );

			return isMBeanStateDeployedByQuery(query);
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}


	/* RAR in EAR */
	public boolean isRarInEarDeployed(String rarName, String earName) throws RuntimeException
	{
		try {
			 String query = "jboss.deployment:id=\"vfszip:" +
                                                System.getProperty("jsfunit.deploy.dir") + "/" + earName +
                                                "/" + rarName + "/\",*";

			return isMBeanStateDeployedByQuery(query);
		}
		catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

}// ApplicationTestBaseAS5 




