package org.jboss.jopr.jsfunit;

import java.io.IOException;
import java.util.*;
import javax.management.*;
import javax.naming.*;
import org.jboss.mx.util.MBeanServerLocator;

/**
 * JMX utilities.
 *
 * TODO: Write set(), invoke() if needed.
 * Possibly also remote JMX support (will we ever need it?).
 * 
 * @author ondra
 */
public final class JMXUtils{

	private final MBeanServerConnection jmxServer;
	private static final String RMI_ADAPTOR_JNDI_NAME = "jmx/invoker/RMIAdaptor";

	private JMXUtils(MBeanServer jmxServer) {
		this.jmxServer = jmxServer;
	}

	public static JMXUtils getInstanceForLocalJBoss(){
		return new JMXUtils( MBeanServerLocator.locateJBoss() );
	}

	/**
	 * This can come handy later...
	 * @param host  JBoss host.
	 * @param port  RMI connector port.
	 * @return      An instance of callable MBeanServerConnection.
	 * TODO: Needs dependency: jmx-adaptor-plugin - which version?
	 */
	public static JMXUtils getInstanceForRemoteJBoss( String host, Integer port ) throws NamingException{

		// Setup context environment.
		Hashtable env = new Hashtable();
		env.put("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
		String providerURL = "jnp://" + host + ":"+ port.toString() +"/";
		env.put("java.naming.provider.url", providerURL);
		env.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");

		// Do not let another partition member to respond.
		env.put("jnp.disableDiscovery", "true");

		//log.info("Looking for InitialContext for "+providerURL + " ...");

		InitialContext context = new InitialContext(env);

		//log.info("Found InitialContext: "+ context.toString() );

		// lookup JMX mbean RMI adaptor
		//RMIAdaptor server = (RMIAdaptor) ic.lookup(RMI_ADAPTOR_JNDI_NAME);
		//return new JMXUtils( context );

		throw new UnsupportedOperationException(
						"This uses RMIAdaptor and thus introduces unnecesarry dependency.");

	}// getInstanceForRemoteJBoss()

	public static JMXUtils getInstanceForRemoteJBoss( String host ) throws NamingException {
		return getInstanceForRemoteJBoss(host, 1099);
	}


	public Object getMBeanAttribute( String mBeanName, String attribute ) throws JMException, IOException {
		return this.getMBeanAttribute( new ObjectName(mBeanName), attribute );
	}

	public Object getMBeanAttribute( ObjectName mBeanName, String attribute ) throws JMException, IOException {
		return jmxServer.getAttribute( mBeanName, attribute );
	}

	public void dumpInfo(){

		try{
			System.out.println("Domains: ");
			for( String domainName : this.jmxServer.getDomains() ){
				System.out.println("  "+domainName);
			}

			System.out.println("MBeans count: " + this.jmxServer.getMBeanCount() );

			System.out.println("MBeans: ");
			Set<ObjectName> queryNames = this.jmxServer.queryNames(null, null);
			SortedSet<ObjectName> sortedQueryNames = new TreeSet<ObjectName>(queryNames);
			
			for( ObjectName objectName : sortedQueryNames ){
				System.out.println("  "+objectName.getCanonicalName());
			}
		}
		catch( IOException ex ){
			System.out.println("Exception occured: "+ ex.getMessage());
		}

	}

}// class JMXUtils
