/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as4;

import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import javax.management.ObjectName;
import javax.management.JMException;




/**
 * When complete, this class will contain tests for creating, 
 * configuring, and deleting various types of datasources. 
 * 
 * @author Ondrej Zizka
 * 
 */
public class DatasourceTest extends org.jboss.jopr.jsfunit.DatasourceTestBase {


    /**
     * Create a new datasource using the given template and properties.
		 * 
		 * Scenario:
		 *	 NavTree -> Datasources;
		 *   Template select;
		 *   Fill the form and save.
     * 
     * @param datasourceType is not used.
		 * @param datasourceTemplate is the value of the HTML select option.
     */
    protected void createDatasource(DatasourceType datasourceType, // unused here, left for API compat.
                                  String datasourceTemplate,
                                  Map<String, String> propertiesMap) throws IOException {
        
        // Expand the "Datasources" tree node
        HtmlAnchor warLink = getNavTreeLink("Datasources");
				warLink.click();
        
				// Click on the "Add new resource" button
				client.click("actionHeaderForm:addNewNotContent");  // 404 if setThrowExceptionOnFailingStatusCode(true) above

				// Select the datasource (options: default, Oracle XA, Oracle Local TX)
				HtmlSelect menu = (HtmlSelect)client.getElement("resourceCreateForm:selectedTemplate");
				menu.setSelectedAttribute(datasourceTemplate, true);

				// Submit the form.    [Continue]
        client.click("resourceCreateForm:addButton");

        // Configure the properties associated with this datasource
				propertiesMap.put("type", DatasourceType.LOCAL_TX_DATASOURCE.getHtmlRadioValue() );
        fillOutForm(propertiesMap, RESOURCE_CONFIGURATION_FORM);  
				// Remove the type property not to be treated later as regular properties.
				propertiesMap.remove("type");
    }

		
		/**
		 * Properties for the datasource for AS4.
		 *
		 * AS 4 has Type selection as radio buttons ("Type")
		 *
		 * @returns the default properties from the base class overriden by properties for AS4.
		 */
		@Override
		protected Map<String, String> createDatasourceProperties()
		{
				Map<String, String> propertiesMap = super.createDatasourceProperties();

				// Additional or different properties for AS 4

				// AS 4 has type set through radio button.
				//propertiesMap.put("type", DatasourceType.LOCAL_TX_DATASOURCE.getHtmlRadioValue() );
				// Inappropriate - type is then represented other way than other properties.


				return propertiesMap;

		}




		
    /**
     * Delete the datasource given by datasourceName.
     */
    protected void deleteDatasource(String datasourceName) throws IOException, EmbJoprTestException {
        HtmlAnchor datasourceLink = getNavTreeLink("Datasources");
        datasourceLink.click();
			//HtmlButtonInput deleteButton = getDeleteButton("resourceSummaryForm", datasourceName); /*categorySummaryForm*/
			//deleteButton.click();
			try {
				((HtmlInput) getFirstElementByXPath(getRowByName(datasourceName),
								".//input[ @value = 'Delete' ]")).click();
			} catch (EmbJoprTestException ex) {
				Logger.getLogger(DatasourceTest.class.getName()).log(Level.SEVERE, null, ex);
				throw ex;
			}
    }
    



		
		/**
		 * AS 4 lowercases the file name.
		 * @return
		 */
		@Override
		protected String getDatasourceConfigFile( String jndiName ) {
			//String jndiName = this.getDatasourceProperties().get("jndi-name");
			return System.getProperty("jsfunit.deploy.dir") + "/" + jndiName.toLowerCase() + "-ds.xml";
		}


		/**
		 * AS 4 has it simplier than AS 5.
		 * @param jndiName
		 * @param serviceName
		 * @return
		 */
		@Override
		protected String getMBeanName( String jndiName, String serviceName ){
			return "jboss.jca:name="+jndiName+",service="+serviceName;
		}

    
    
    
    /*
     * Some preliminary creation tests
     */



    /**
     * Create a new datasource. Leave some property values that aren't 
     * required unset.
     */
    public void testCreateDatasource() throws IOException, EmbJoprTestException {


				/* // Works
				 JMXUtils jmxu = JMXUtils.getInstanceForLocalJBoss();
				try {
					Object mBeanAttribute = jmxu.getMBeanAttribute("jboss.j2ee:service=ClientDeployer", "State");
					log.info("State: "+mBeanAttribute.toString());
				} catch (JMException ex) {
					log.fatal(ex.getMessage());
					ex.printStackTrace();
				} /**/

				log.info("Printing MBeans info");
				JMXUtils.getInstanceForLocalJBoss().dumpInfo();///


				Map<String, String> propertiesMap = getDatasourceProperties();

				/*  AS 4: 
					<option value="Oracle Local TX__Datasource">Oracle Local TX (Datasource)</option>
					<option value="Oracle XA__Datasource">Oracle XA (Datasource)</option>
					<option value="default__Datasource">default (Datasource)</option>
        createDatasource(DatasourceType.LOCAL_TX_DATASOURCE, // unsued for AS 4
												 // TODO: Differs! Split DatasourceType accordingly...
                         DatasourceTemplate.AS4_DEFAULT.getTemplateHtmlSelectValue(),
                         propertiesMap);
				/**/

				/*
				// Option - "No TX datasource" / value = no-tx-datasource   TODO: XSLT
				//((HtmlRadioButtonInput)client.getElement("resourceConfigurationForm:rhq_prop-287459352_3575610:0")).setChecked(true);
				((HtmlRadioButtonInput)util.getFirstElementByXPath(tabMenuBox, ".//input[@value='no-tx-datasource']"))
				.setChecked(true);
				 */


				log.info("Clicking Save...");
        client.click("resourceConfigurationForm:saveButton");
				log.info("Clicked");


				DebugUtils.writeFile("/home/ondra/work/JOPRembedded/emb.html", client.getPageAsText());////

				log.info("Printing MBeans info"); ////
				JMXUtils.getInstanceForLocalJBoss().dumpInfo();////

        // Check for the appropriate success messages.
        String expectedMessage = "Successfully added new Datasource.";
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);

				// Check whether the dataource is deployed.
        assertTrue(isDatasourceDeployed(propertiesMap.get("jndi-name"), 
                                        DatasourceType.LOCAL_TX_DATASOURCE));
        checkProperties(propertiesMap.get("jndi-name"), 
                        DatasourceType.LOCAL_TX_DATASOURCE,
                        propertiesMap);

        // TODO: need to verify that appropriate default values were
        // set for properties that were not specified above
       
        // Clean up
        deleteDatasource(propertiesMap.get("jndi-name"));

        expectedMessage = "Successfully deleted Datasource '"
                          + propertiesMap.get("jndi-name") + " Datasource'.";
				
        checkClientAndServerMessages(expectedMessage, expectedMessage, false);
				
    }
    


/*
		@Override
    public void testCreateDatasourceMissingRequiredValues() throws IOException {

			// Leave jndi-name and connection-url unset
			Map<String, String> propertiesMap = new HashMap<String, String>();
			propertiesMap.put("user-name", "testUser");
			propertiesMap.put("max-pool-size", "10");
			propertiesMap.put("idle-timeout-minutes", "20");

			createDatasource(DatasourceType.NO_TX_DATASOURCE,
					//"default__No TX Datasource",
					isJBoss4 ?
						DatasourceTemplate.DEFAULT.getTemplateHtmlSelectValue() :
						DatasourceTemplate.DEFAULT_NO_TX.getTemplateHtmlSelectValue(),
					propertiesMap);
			client.click("resourceConfigurationForm:saveButton");

			// Check for the appropriate error messages
			checkClientAndServerMessages(
							"An invalid value was specified for one or more properties",
							"Value is required",  true);
    }
*/











		/**
		 * @return the suite of tests being tested
		 */
		public static Test suite()
		{
			 return new TestSuite(DatasourceTest.class);
		}




		/**
		 * AS 4 uses State and StateString attributes.
		 * @param deploymentMBean
		 * @return
		 * @throws javax.management.JMException
		 * @throws java.io.IOException
		 */
		@Override
		protected boolean isMBeanStateDeployedImpl(ObjectName deploymentMBean)  throws JMException, IOException {

			JMXUtils jmxUtils = JMXUtils.getInstanceForLocalJBoss();

			Object state = jmxUtils.getMBeanAttribute(deploymentMBean, "StateString");
			//if(!("3".equals(state.toString()))) return false; // started state
			return ( "Started".equals(state.toString()) );

		}

		
}// DatasourceTest.java for AS 4

