/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as4;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.IOException;
import java.util.*;
import junit.framework.Test;
import junit.framework.TestSuite;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import com.gargoylesoftware.htmlunit.WebClient;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException;
import org.jboss.mx.util.MBeanServerLocator;


/**
 * This class contains tests for testing the EmbJopr Application
 * Management Functions for WAR components with JBoss EAP 4.
 * 
 * @author Shelly McGowan
 *
 */
public class WarTest extends ApplicationTestBaseAS4 {

	/**
	  * @return the suite of tests being tested
	*/

	public static Test suite()
	{
		return new TestSuite(WarTest.class);
	}

	public void XtestBasicWarDeployment() throws IOException, EmbJoprTestException
	{
		// click the nave tree
		HtmlAnchor warLink = getNavTreeLink(NAV_WAR);
		warLink.click();

		// click on the "Add new resource" button
		client.click("actionHeaderForm:addNewContent");  // 404 if setThrowExceptionOnFailingStatusCode(true) above

		// upload hellothere.war
		HtmlFileInput fileInput = (HtmlFileInput)client.getElement("createContentForm:file");
		fileInput.setContentType("application/war");
		fileInput.setValueAttribute(System.getProperty("jsfunit.testdata") + "/war/hellothere.war");
		client.click("createContentForm:addButton");

		try {

		// assert that the success message appeared on the client side
		assertTrue(client.getPageAsText().contains("hellothere.war created successfully"));

		// assert text and sevrity level for FacesMessage on server side
		assertTrue(server.getFacesMessages().hasNext());
		FacesMessage successMessage = server.getFacesMessages().next();
		assertTrue(FacesMessage.SEVERITY_INFO.equals(successMessage.getSeverity()));
		assertTrue(successMessage.getDetail().contains("hellothere.war created successfully"));

		// Use JMX to assert that the WAR really did deploy successfully
		assertTrue(isWarDeployed("hellothere.war"));



		// use HtmlUnit to test the newly deployed war in a new WebClient session
		// note that I have full access to the FacesContext and the previous request
		HttpServletRequest request =
			 (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();
		int port = request.getLocalPort();
		WebClient webClient = new WebClient();
		HtmlPage page = (HtmlPage)webClient.getPage("http://localhost:" + port + "/hellothere/hello.jsp");
		assertTrue(page.asText().contains("HELLO WORLD"));

		}
		finally {
			// Undeploy the WAR
			ejtt.deployment.undeployViaEmbJopr(DeployableTypes.WAR, "hellothere.war");

			// This assert doesn't work.  jopr does remove the WAR, but for some
			// reason, JBoss doesn't undeploy the MBeans
			//assertFalse(isWarDeployed("hellothere.war"));
		}

	}


	// Helper Methods

	@Override
        protected boolean isWarDeployed (String warName)
        {
                try {
                        MBeanServer jmxServer = MBeanServerLocator.locateJBoss();
                        ObjectName objName = new ObjectName("jboss.web.deployment:war=" +
                                warName + ",*");
                        Set mBeans = jmxServer.queryNames(objName, null);

                        if (mBeans.size() != 1) return false;

                        ObjectName deploymentMBean = (ObjectName)mBeans.iterator().next();
                        String state = (String)jmxServer.getAttribute(deploymentMBean, "StateString");

                        return "Started".equals(state);

                } catch (Exception e) {
                        throw new RuntimeException(e);
                }
        }

}

