/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.as5.app.ear;

import com.gargoylesoftware.htmlunit.Page;
import com.gargoylesoftware.htmlunit.WebClient;
import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.*;
import java.util.*;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.apache.commons.lang.math.RandomUtils;
import org.jboss.jopr.jsfunit.AppConstants.DeployableTypes;
import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;
import org.w3c.dom.Node;



/**
 * This class contains tests for testing the EmbJopr Application
 * Management Functions for Enterprise Application archives with JBoss AS 5.
 * 
 * @author Shelly McGowan
 *
 */
public class EarTest extends ApplicationTestBaseAS5 {

	public static final DeployableTypes APP_TYPE = DeployableTypes.EAR;


	
	/**
	  * @return the suite of tests being tested
	*/
	public static Test suite()
	{
		String testToRun = System.getProperty("jsfunit.test.single");
		if( null == testToRun ){
			return new TestSuite(EarTest.class);
		}
		else{
			// TODO: Try running single test set in system property.
			// http://members.pingnet.ch/gamma/junit.htm
			TestSuite suite = new TestSuite();
			//suite.addTest(this.getClass().getConstructor(String.class).newInstance(testToRun));
			suite.addTest( new EarTest(testToRun) );
			return suite;
		}
	}

	public EarTest() {
	}
	private EarTest(String testToRun) {
		super(testToRun);
	}




	/*
	 * testName: testBasicEarDeployment
   * assertion:  verify basic deployment of Enterprise Archive
	 * test Strategy:  Navigate to Enterprise Applications.
	 *	Add a new resource.  Verify the resource was successfully
	 *	deployed.  Undeploy the archive for test clean up purposes.
	 *
	 * Was failing: JMX doesn't report EAR as deployed: eardeployment.ear
	 */ 
	public void testEarBasicDeployment() throws IOException, EmbJoprTestException
	{

		final String DEPLOYABLE_NAME = BASIC_EAR;

		// Deploy the EAR.
		String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		deployEar( earFilePath );
		String expectedMessage = DEPLOYABLE_NAME + " created successfully";
		checkClientAndServerMessages(expectedMessage, expectedMessage, false);
		
		try {


			// Use JMX to assert that the EAR components really did deploy successfully.

			// id="vfszip:/home/brq/ozizka/JoprEmbedded/jboss-as-5.x/build/output/jboss-5.1.0.Beta1/server/default/deploy/eardeployment.ear/",type=Deployment
			// id="vfszip:/home/ondra/work/JOPRembedded/embjopr-svn-trunk/jsfunit/target/jboss5x/deploy/eardeployment.ear",* not found.
			assertTrue("JMX doesn't report EAR as deployed: eardeployment.ear", isEarDeployed(DEPLOYABLE_NAME));
			//assertTrue("JMX doesn't report EAR as deployed: eardeployment.ear", ejtt.deployment.isDeployedAccordingToJMX(APP_TYPE, DEPLOYABLE_NAME) );

			// id="jboss.j2ee:module="sessiona.jar",service=EjbModule",type=Component
			assertTrue("JMX doesn't report EJB sessiona.jar as deployed.", isEJBDeployed("sessiona.jar"));
			assertTrue("JMX doesn't report EJB sessionb.jar as deployed.", isEJBDeployed("sessionb.jar"));
		}
		finally {
			// Undeploy the EAR.
			ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
		}

		// This assert doesn't work. The JXM view is not consistent with the mananaged view. 
		// JBAS-XXXX
		//assertFalse(isEarDeployed("eardeployment.ear"));
		//assertFalse(isEJBDeployed("sessiona.jar"));
		//assertFalse(isEJBDeployed("sessionb.jar"));

	}







	
	/**
	 *	assertion:
	   
	      Verify an .ear that contained a bad deployment descriptor
				fails deployment, fixed using console, and can then be
				successfully deployed.

	  	test Strategy:
	   
	      Deploy an .ear that is known to have a bad deployment
				descriptor.  Verify the console shows deployment failed.
				Edit the .ear resource to fix the achive.  Redeploy and
				verify the archive has been deployed successfully.
	 
	 */
	public void testEarMalformedDescriptorDeployment() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EAR_MALFORMED_APP_FILENAME;

		try {
			String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
			//deployEar(earFilePath);
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, earFilePath);

			checkClientAndServerMessages("Failed to create Resource", "Failed to create Resource", true);

			if( ejtt.deployment.isDeployedAccordingToEmbJopr(APP_TYPE, DEPLOYABLE_NAME)){
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
				throw new EmbJoprTestException("An EAR with malformed descriptor " +
								"stays deployed after the deployment failed.");
			}
		}
		finally {
			
		}
	}



	/**
	 * TODO:
	 * 1) Deploy a valid EAR with a war in it (to check it's functionality).
	 * 2) In the Content tab, update it with an EAR with malformed descriptor.
	 * 3) Check that the error message contains "***** ROLLED BACK TO ORIGINAL APPLICATION FILE. *****"
	 * 4) Check that the embedded WAR works to verify the rollback.
	 *
	 * Was failing because of  EMBJOPR-226.
	 */
	public void testEarDeployFineThenRedeployMalformedDescriptor() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EAR_NOT_MALFORMED_APP_FILENAME;
		final String DEPLOYABLE_2_NAME = EAR_MALFORMED_APP_FILENAME_2;

		try {
			// Deploy first deployable - the correct one.

			String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, earFilePath);

			String expectedMessage = DEPLOYABLE_NAME + " created successfully";
			checkClientAndServerMessages(expectedMessage, expectedMessage, false);


			// Redeploy new version with malformed application.xml in the Content tab.

			// Has to have the same name -> copy to a tmp dir.
			earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_2_NAME;
			String tmpFilePath = ejtt.getTempDir()+'/'+DEPLOYABLE_NAME;
			FileUtils.copyFile( new File(earFilePath), new File(tmpFilePath) );

			// Fill the upload input and click the update button.
			ejtt.getTabMenu().getTabContentBox().getFirstTable().getFirstLinkContaining( DEPLOYABLE_NAME ).click();
			ejtt.getTabMenu().clickContentTab();
			ejtt.getTabMenu().getTabContentBox().getFirstFileInput().setValueAttribute( earFilePath );
			ejtt.getTabMenu().getTabContentBox().getSubmitButtonByLabel("Update").click();

			// We should get an error message.
			final String expMsg = "The specified file must have the same name as the existing deployed EAR File";
			checkClientAndServerMessages(expMsg, expMsg, FacesMessage.SEVERITY_WARN);


			// The error message should say that the original EAR was rolled back.
			// Was removed.
			//String rolledBackMsg = "ROLLED BACK TO ORIGINAL APPLICATION FILE.";
			//checkClientAndServerMessages(rolledBackMsg, rolledBackMsg, true);

		}
		finally {
			// The original EAR should remain deployed.
			if( ejtt.deployment.isDeployedAccordingToEmbJopr(APP_TYPE, DEPLOYABLE_NAME)){
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			}
			else{
				throw new EmbJoprTestException("The original EAR should remain deployed " +
								"after unsuccessful deployment of bad version of the same EAR.");
			}
		}
	}








	/**
	 * assertion:
	  
			Verify the navigation sequence to Enterprise Applications.

		 test Strategy:
	 
	    From the root of the navigation tree:
			Click JBossAS Servers ==> JBoss App Server:${config}
	          ==> Applications ==> Enterprise Application

	 *  PASSED.
	 */
	public void testEarNavigation() throws IOException, EmbJoprTestException, InterruptedException
	{

		// JBossAS Servers node 
		NavTreeNode nodeServers = ejtt.getNavTree().getNodeByLabel("JBossAS Servers");
		nodeServers.click();

		{
			String headerText = "JBossAS Server";

			assertTrue("Page doesn't contain the header: "+headerText,
							client.getPageAsText().contains(headerText));

			assertTrue("EmbJopr should list at least one server (the one it is running on)" +
							" and that should be in the UP sate.",
							client.getPageAsText().contains("UP"));
			// Check whether the server is listed. If not, Exception is thrown.
			ContentTableRow row =
							ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("JBoss Application Server")
							.getFirstRowContainingLink( String.format(AS_NODE_NAME_FORMAT, ejtt.getJBossConfig() ) );
			// Click the server link
			//HtmlAnchor link row.getLinkByLabel("JBAS (default)");
			Node firstLink = row.getCellByColumnName("Name").getElementsByTagName("a").item(0);
			if( null == firstLink || !(firstLink instanceof HtmlAnchor) )
				throw new HtmlElementNotFoundException("Can't find the server link.");

			((HtmlAnchor)firstLink).click();
		}

		// JBoss App Server:${config} node
		{
			String pageText = client.getPageAsText();

			String headerText = String.format(AS_NODE_NAME_FORMAT, ejtt.getJBossConfig() );
			assertTrue("Page doesn't contain the header: "+headerText,
							pageText.contains(headerText));

			headerText = "General Properties";
			assertTrue("Page doesn't contain the header: "+headerText,
							pageText.contains(headerText));

			// TODO: This page reports "Version:5.0 CR1" - EMBJOPR-77

			ejtt.getNavTree().getNodeByLabel("Applications").click();

		}


		// Applications node 
		{
			// Whooo-hooo! So much to click through!

			ejtt.getTabMenu().clickTab("Summary");

			// TODO: Pagination options: EMBJOPR-78
			// resourceDataScroller.xhtml, TableManager.java, "pageSizes".


			// There's at least one Application with State == UP.
			//ContentTable table = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("Different types of Applications");
			HtmlTable tableElement = (HtmlTable)client.getElement("categorySummaryForm:dataTable");
			ContentTable table = ejtt.getTabMenu().getTabContentBox().getTable(tableElement);
			ContentTableRow row = table.getFirstRowContainingText("UP");

			// Go further - try to click on any Application that is up.
			HtmlAnchor link = row.getFirstLinkFromColumn("Name");
			link.click();
		}


	}



	/**
	 * Deploys an EAR and clicks a bit through the UI.
	 */
	public void testEarDeploymentNavigation() throws IOException, EmbJoprTestException {
		// Concrete appliction node.
		{

			final String DEPLOYABLE_NAME = BASIC_EAR;

			// Go back to applications Sumary screen.
			ejtt.getNavTree().getNodeByLabel(NAV_APPLICATIONS).click();

			try {
				// Deploy the EAR.
				String earFilePath = System.getProperty(SYSPROP_TESTDATA_DIR) + "/ear/"+DEPLOYABLE_NAME;
				//deployEar( earFilePath );
				ejtt.deployment.deployViaEmbJopr(APP_TYPE, earFilePath);

				ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();

				ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
				assertTrue("Page doesn't list "+DEPLOYABLE_NAME+" in Summary tab.", earRow != null );

				// Go to the summary through listed item.
				earRow.getLinkByLabel(DEPLOYABLE_NAME).click();
				// Check that we have the summary tab for the selected EAR.
				assertTrue( "EAR name '"+DEPLOYABLE_NAME+"' not found in the content box.",
					ejtt.getTabMenu().getTabContentBox().getElement().getTextContent().contains(DEPLOYABLE_NAME) );

				// Go to the summary through nav tree node.
				NavTreeNode earNode = ejtt.getNavTree().getNodeByLabel(NAV_EAR);
				if( !earNode.isExpanded() ){
					log.info("Expanding.");
					earNode.getArrowLink().click();
					ejtt.sleep(2000);
				}
				ejtt.getNavTree().getNodeByLabel(DEPLOYABLE_NAME).click();
				// Check that we have the summary tab for the selected EAR.
				ejtt.getTabMenu().getTabContentBox().getElement().getTextContent().contains(DEPLOYABLE_NAME);
			}
			finally {
				try {
					undeployEar(DEPLOYABLE_NAME);
				}catch(Exception ex) {
					log.error("Failed to undeploy "+DEPLOYABLE_NAME+": "+ex);
				}
			}

		}

	}//




	/**
	 *  Assertion:  Verify the content of the Enterprise Application Summary tab
	 *
	 *
	 * FAILS because clicking on the EAR name in list brings us to the root node Summary.
	 * https://jira.jboss.org/jira/browse/EMBJOPR-80
	 *
	 * junit.framework.ComparisonFailure: expected:<eardeployment.ear> but was:<ondra-redhat>
     at org.jboss.jopr.jsfunit.as5.EarTest.testEarSummaryTab(EarTest.java:323)
	 */
	public void testEarSummaryTab() throws EmbJoprTestException, IOException, Exception {

		final String DEPLOYABLE_NAME = BASIC_EAR;

		final int DEPLOY_TIMEOUT_SEC = 20;

		// Deploy the EAR
		String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		ejtt.deployment.deployViaEmbJopr(APP_TYPE, earFilePath);

		try {

			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 2000, 10);

			// FAILS because of EMBJOPR-80.
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			earRow.getLinkByLabel(DEPLOYABLE_NAME).click();


			// Check the values in info table(s)

			// General Properties
			HtmlTable genpropTable = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_GENERAL_PROPERTIES).getElement();
			ContentInfoTable infoTable = ejtt.getContentInfoTable( genpropTable );
			Properties props = infoTable.getProperties();

			assertEquals(DEPLOYABLE_NAME, props.getProperty("Name").trim());
			//assertEquals("?", props.getProperty("Version")); // TODO: Where does RHQ get the version from?
			// TODO: Fill JIRA? Description of EAR should be taken from application.xml <display-name>
			//assertEquals("JBossTest Ear Deployment Testsuite", props.getProperty("Description"));


			// Resource Traits
			infoTable = ejtt.getContentInfoTable(
							ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_TRAITS).getElement() );
			props = infoTable.getProperties();


			String path = ejtt.getDeployDir()+"/"+DEPLOYABLE_NAME;
			assertEquals(path, props.getProperty("Path").trim());

			assertEquals("no", props.getProperty("Exploded?").trim());

		}
		finally {
			try {
				undeployEar(DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}// testEarSummary()



	/**
	 * Changes EAR configuration, and checks whether the changes were saved.
	 *
	 * FAILS: Configuration tab is disabled.
	 */
	public void DISABLEDtestEarConfigurationTab() throws IOException, EmbJoprTestException {

		// Deploy the EAR.
		String earFilePath = ejtt.getTestDataDir() +"/ear/"+ BASIC_EAR;
		deployEar( earFilePath );

		try {

			ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, BASIC_EAR, 3000, 15);

			// Navigate to the Configuration tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(BASIC_EAR);
			earRow.getLinkByLabel(BASIC_EAR).click();

			Page prevPage = client.getContentPage();
			ejtt.tabMenu.clickConfigurationTab();
			// FAILS: Configuration tab is disabled.
			assertNotSame("We should move to other page after clicking the Config tab. Perhaps Config tab is disabled?",
							prevPage, client.getContentPage());

			// Read properties.
			Properties props = new Properties();
			String filePath = ejtt.getTestDataDir()+"/ear/"+"ear-conf-basic.properties";
			props.load(new FileInputStream( filePath ));


			// Set the configuration options and Save
			fillOutForm(props);
			//ejtt.getTabMenu().getTabContentBox().getButtonByLabel("Save");
			ejtt.getClickableByID("resourceConfigurationForm:saveButton").click();


			// Check whether the properties were saved.
			ejtt.tabMenu.clickConfigurationTab();

			// TODO: We don't see the success message! EMBJOPR-89
			//checkClientAndServerMessages("successfully", "successfully", false);

			// FAILS because some of the values are read-only.  EMBJOPR-96
			checkForm(props);

			// TODO: Finish when the properties are marked read-only.

		}
		finally {
			try {
				undeployEar(BASIC_EAR);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}



	/**
	 * Changes EAR configuration, and verifies that the changes were NOT saved.
	 *
	 * FAILS: Configuration tab is disabled.
	 */
	public void DISABLEDtestEarConfigurationTabCancel() throws IOException, EmbJoprTestException {

		// Deploy the EAR.
		String earFilePath = ejtt.getTestDataDir() +"/ear/"+ BASIC_EAR;
		deployEar( earFilePath );

		try {

			ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, BASIC_EAR, 3000, 15);

			// Navigate to the Configuration tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(BASIC_EAR);
			earRow.getLinkByLabel(BASIC_EAR).click();

			Page prevPage = client.getContentPage();
			ejtt.tabMenu.clickConfigurationTab();
			assertNotSame("We should move to other page after clicking the Config tab.",
							prevPage, client.getContentPage());


			// Load properties (we will use their names).
			Properties propsToSet = new Properties();
			String propsFilePath = ejtt.getTestDataDir()+"/ear/"+"ear-conf-basic.properties";
			propsToSet.load(new FileInputStream( propsFilePath ));

			// Get the current properties.
			Properties propsOriginal = getFormPropertiesValues( propsToSet );


			// Set the configuration options and CANCEL
			fillOutForm( propsToSet );
			ejtt.getClickableByID("resourceConfigurationForm:cancelButton").click();


			// TODO: We should get back to Conf tab automatically: EMBJOPR-100
			ejtt.tabMenu.clickConfigurationTab();

			// Now check whether the values are the same as before editing.
			checkForm( propsOriginal );

		}
		finally {
			try {
				undeployEar(BASIC_EAR);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}



	/**
	 * Redeploys EAR.
	 *
	 * Fails because of EMBJOPR-109 - HTTP Status 500 - Error in Seam/JSF
	 * Fails because of EMBJOPR-42 - javax.el.ELException: org.jboss.seam.RequiredException:
	 *     @Out attribute requires non-null value: updateBackingContentAction.packageDetails
	 */
	public void testEarRedeployment() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME  = BASIC_EAR;

		// Deploy the EAR.
		String appFilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME;
		deployEar( appFilePath );

		try {

			ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);

			// Navigate to the Configuration tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			earRow.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.tabMenu.clickContentTab();

			String xPath = ".//table//input[@type='file']";
			HtmlFileInput fileInput = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			fileInput.setValueAttribute(appFilePath);
			xPath = ".//table//input[@type='submit']";
			HtmlSubmitInput submit = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			submit.click();



		}
		finally {
			try {
				undeployEar(DEPLOYABLE_NAME); // EMBJOPR-109 here.
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}



	/**
	 * Deploys exploded EAR and checks whether it is reported as exploded.
	 *
	 * DISABLED - Badly written test, to be fixed. TODO: Hot-deploy
	 */
	public void DISABLEDtestUnzippedEarReportedAsExploded() throws IOException, EmbJoprTestException {

		// Deploy the EAR.
		String earFilePath = ejtt.getTestDataDir() + "/ear/"+BASIC_EAR;
		deployEar( earFilePath );

		try {
			// Wait for EAR to be deployed and UP
			ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, BASIC_EAR, 3000, 15);

			ContentTable table = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("Resource Traits");
			ContentInfoTable infoTable = ejtt.getTabMenu().getTabContentBox().getContentInfoTable(table.getElement());
			Properties properties = infoTable.getProperties();
			assertEquals("'Exploded?' should be 'yes'", "yes", properties.getProperty("Exploded?").toLowerCase() );

		}
		finally {
			try {
				undeployEar(BASIC_EAR);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}




	/**
	 * Checks EAR metrics tab.
	 */
	public void testEarMetricsTab() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = BASIC_EAR;

		// Deploy the EAR.
		String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		deployEar( earFilePath );

		try {

			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);

			// Navigate to the Metrics tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			earRow.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.tabMenu.clickMetricsTab();

			// Resource Traits

			// Check the Path.
			HtmlTable tbl = ejtt.getTabMenu().getTabContentBox().
							getTableUnderHeader("Traits").getElement();
			ContentInfoTable infoTable = ejtt.getContentInfoTable( tbl );
			Properties props = infoTable.getProperties();
			log.debug("EAR Metrics tab - Traits: "+ props.toString());

			String path = ejtt.getDeployDir()+"/"+DEPLOYABLE_NAME;
			assertEquals(path, props.getProperty("Path"));

		}
		finally {
			try {
				undeployEar(DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}


	/**
	 * Changes EAR configuration, and checks whether the changes were saved.
	 *
	 * FAILS because some of the values are read-only.  EMBJOPR-96
	 */
	public void testEarMetricsTabRefreshButton() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = BASIC_EAR;

		// Deploy the EAR.
		String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		deployEar( earFilePath );

		try {

			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);

			// Navigate to the Metrics tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			earRow.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.tabMenu.clickMetricsTab();

			// Click the Refresh button.
			// Can this change to submit? Perhaps set an ID and use ejtt.getClickableById().
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Refresh").click();

			// Check that we are still on the right tab.
			assertTrue("Metrics tab is active", ejtt.tabMenu.isTabActive("Metrics") );
			assertTrue("Page should contain EAR name", client.getPageAsText().contains(DEPLOYABLE_NAME) );
			assertTrue("Page should contain 'Refresh'", client.getPageAsText().contains("Refresh") );

		}
		finally {
			try {
				undeployEar(DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}


	/**
	 * Deploys an exploded EAR application.
	 *
	 * PASSED.
	 * Now fails, because EAR does not appear.
	 */
	public void testEarDeployUnpacked() throws IOException, EmbJoprTestException {


		// Deploy the unpacked EAR.
		// We have to use hotdeploy - can't upload a directory.
		log.info("Unzipping ear/"+EAR_UNPACKED_ZIP);
		ejtt.deployment.unzipToDeployDir("ear/"+EAR_UNPACKED_ZIP, "");
		ejtt.sleep(5000); // 5 sec is default deployment scanner interval.

		try {
				// Loop, wait for the app to appear.
				log.info("Waiting for EAR to appear.");
				ejtt.deployment.waitActivelyForDeployment( APP_TYPE, EAR_UNPACKED, 5000, 35, this);


				ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();

				ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(EAR_UNPACKED);

				// TODO: Finish

				// TODO: Check whether Exploded?:  yes.  EMBJOPR-95

				// TODO: DeploymentUtils, DeployableTypes, isAvailable enum inner classes

				// TODO: Preliminary test - check system properties, their validity, java version, etc.
		}
		finally {
			// Delete the EAR dir.
			ejtt.deployment.deleteFromDeployDir(EAR_UNPACKED);
		}
	}






	/**
	 * Deploys an EAR with WAR as a module, and checks whether the web app runs.
	 */
	public void testEarWithWar() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EAR_WITH_WAR;

		// Deploy the EAR.
		//String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		String earFilePath = ejtt.getTestFilePath(APP_TYPE, DEPLOYABLE_NAME);
		//deployEar( earFilePath );
		ejtt.deployment.deployViaEmbJopr(APP_TYPE, earFilePath);

		// Get the HTTP port and the URL of the test page.
		HttpServletRequest tmpRequest = (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();
		int port = tmpRequest.getLocalPort();
		// Eventually remove ".war" suffix and add "-war" (convention)
		String context = StringUtils.chomp( DEPLOYABLE_NAME, ".ear" ) +"-war";
		String testPageUrl = "http://localhost:"+port+"/"+context+"/index.jsp";

		// Create web client we will use to check the web app.
		WebClient webClient = new WebClient();
		webClient.setThrowExceptionOnFailingStatusCode(false);
		HtmlPage page;

		try {

			// Wait until the EAR is UP.
			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 30, this);
			
			// Assert that the app is running.
      page = (HtmlPage)webClient.getPage(testPageUrl);
			assertEquals( "HTTP status of "+testPageUrl+" should be 200", 200, page.getWebResponse().getStatusCode() );
      assertTrue("Page should contain 'Hello World'", page.asText().contains("Hello World"));

		}
		finally {
			try {
				//undeployEar(DEPLOYABLE_NAME);
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught an exception when undeploying: "+ex, ex);
			}
		}

	}









	/**
	 * Stops and Starts an EAR with WAR in the Control tab.
	 *
	 * Was failing because of EMBJOPR-133.
	 * Now FAILS because of EMBJOPR-172.
	 */
	public void testEarStopAndStart() throws IOException, EmbJoprTestException
	{
		final String DEPLOYABLE_NAME = EAR_WITH_WAR_COUNTER;

		// Get the HTTP port and the URL of the test page.
		HttpServletRequest tmpRequest = (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();
		int port = tmpRequest.getLocalPort();
		String context = StringUtils.chomp( DEPLOYABLE_NAME, ".ear" )+"-war"; // Eventually remove ".ear" suffix.
		String testPageUrl = "http://localhost:"+port+"/"+context+"/hello.jsp";

		// Create web client we will use to check the web app.
		WebClient webClient = new WebClient();
		// We'll get 404 after stopping the app.
		webClient.setThrowExceptionOnFailingStatusCode(false);
		HtmlPage page;


		// Deploy the EAR.
		String appfilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		deployEar( appfilePath );

		try {

			// Wait until app is UP
			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME,
							3000, 15, this);

			// Assert that the app is running.
      page = (HtmlPage)webClient.getPage(testPageUrl);
			assertEquals( "HTTP status of "+testPageUrl+" should be 200", 200, page.getWebResponse().getStatusCode() );
      assertTrue("Page should contain 'HELLO WORLD'", page.asText().contains("HELLO WORLD"));

			// Refresh the page several times to increase the counter.
			page = (HtmlPage) page.refresh();
			page = (HtmlPage) page.refresh();

			// Get the counter number.
			Integer count1 = NumberUtils.createInteger(( (HtmlElement)page.getBody().getHtmlElementById("count") ).getTextContent());



			// Navigate to the web app, then to it's Control tab.
			ContentTableRow row = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			row.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.getTabMenu().clickControlTab();

			// Click the Stop button.
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Stop").click();

			// Wait for the op to finish and check the final status.
			ejtt.operations.waitActivelyForOperationToFinish("Stop", 1500, 5);
			assertTrue( ejtt.getTabMenu().getTabContentBox().getOperationsHistoryTable().wasLastOperationSuccesful() );



			// Assert that the app is stopped.
      			page = (HtmlPage)webClient.getPage(testPageUrl);
			assertTrue( 200 != page.getWebResponse().getStatusCode() );



			// Click the Start button.
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Start").click();
			// Check the latest message.
			ejtt.operations.waitActivelyForOperationToFinish("Start", 1500, 5);



			// Assert that the app is running.
      page = (HtmlPage)webClient.getPage(testPageUrl);
			assertTrue( 200 == page.getWebResponse().getStatusCode() );
      assertTrue(page.asText().contains("HELLO WORLD"));

			// Get the counter number.
			Integer count2 = NumberUtils.createInteger(( (HtmlElement)page.getBody().getHtmlElementById("count") ).getTextContent());

			assertTrue( "Counter is reset upon app restart, thus should be lower after.", count1 > count2 );


		}
		finally {
			try {
				undeployEar(DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}
	}


	/**
	 * Restarts the EAR in Control tab and checks whether it was really restarted.
	 *
	 * FAILS because of EMBJOPR-133.
	 */
	public void testEarRestart() throws IOException, EmbJoprTestException
	{
		final String DEPLOYABLE_NAME = EAR_WITH_WAR_COUNTER;

		// Get the HTTP port and the URL of the test page.
		HttpServletRequest tmpRequest = (HttpServletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest();
		int port = tmpRequest.getLocalPort();
		String context = StringUtils.chomp( DEPLOYABLE_NAME, ".ear" )+"-war";
		String testPageUrl = "http://localhost:"+port+"/"+context+"/hello.jsp";

		// Create web client we will use to check the web app.
		WebClient webClient = new WebClient();
		webClient.setThrowExceptionOnFailingStatusCode(false);
		HtmlPage page;


		// Deploy the app.
		String appFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		deployEar( appFilePath );

		try {

			// Wait until app is UP
			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME,
							3000, 15, this);

			// Assert that the app is running.
      page = (HtmlPage)webClient.getPage(testPageUrl);
			assertEquals( "HTTP status of "+testPageUrl+" should be 200", 200, page.getWebResponse().getStatusCode() );
      assertTrue("Page should contain 'HELLO WORLD'", page.asText().contains("HELLO WORLD"));

			// Refresh the page several times to increase the counter.
			page = (HtmlPage) page.refresh();
			page = (HtmlPage) page.refresh();

			// Get the counter number.
			Integer count1 = NumberUtils.createInteger(( (HtmlElement) page.getBody().getHtmlElementById("count") ).getTextContent());



			// Navigate to the web app, then to it's Control tab.
			ContentTableRow row = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			row.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.getTabMenu().clickControlTab();

			// Click the Restart button.
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Restart").click();

			// Wait for the op to finish and check the final status.
			ejtt.operations.waitActivelyForOperationToFinish("Restart", 1500, 5);
			assertTrue( ejtt.getTabMenu().getTabContentBox().getOperationsHistoryTable().wasLastOperationSuccesful() );



			// Assert that the app is running.
      page = (HtmlPage)webClient.getPage(testPageUrl);
			assertTrue( 200 == page.getWebResponse().getStatusCode() );
      assertTrue(page.asText().contains("HELLO WORLD"));

			// Get the counter number.
			Integer count2 = NumberUtils.createInteger(( (HtmlElement) page.getBody().getHtmlElementById("count") ).getTextContent());

			assertTrue( "Counter is reset upon app restart, thus should be lower after.", count1 > count2 );

		}
		finally {
			try {
				undeployEar(DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}
	}



	/**
	 * Deploys an EAR with dependencies which are not in default config,
	 * checks for the deployment error message,
	 * and checks that the EAR is not listed.
	 *
	 * FAILS because of EMBJOPR-137 - EAR with unsatisfied dependencies
	 * show error upon deploy, but then shows as UP.
	 */
	public void testEarWithUnsatisfiedDeps() throws IOException, EmbJoprTestException
	{

		final String DEPLOYABLE_NAME = EAR_EXT_DEPENDENCIES;

		try {
			// Deploy the EAR.
			String appFilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr( APP_TYPE, appFilePath );

			// Check that we get an error message.
			assertTrue( "We should get an error message.", server.getFacesMessages().hasNext() );
			assertTrue( "We should get an error message.",
							server.getFacesMessages().next().getSeverity() != FacesMessage.SEVERITY_INFO );

			// Check that the EAR is not listed as deployed.
			assertFalse( "The EAR should not be listed as deployed.",
		    ejtt.deployment.isDeployedAccordingToEmbJopr(APP_TYPE, DEPLOYABLE_NAME, false) );

		}
		finally {
			try {
				//undeployEar(DEPLOYABLE_NAME);
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			}catch( EmbJoprTestException ex ){
				log.info("Caught expected exception when undeploying: "+ex);
			}
		}

	}






	public void testEarRedeployDifferentName() throws IOException, EmbJoprTestException
	{

		final String DEPLOYABLE_NAME  = EAR_WITH_WAR_COUNTER;
		final String DEPLOYABLE_NAME2 = EAR_WITH_WAR_COUNTER_20;

		// Deploy the EAR.
		String appFilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME;
		ejtt.deployment.deployViaEmbJopr( APP_TYPE, appFilePath );

		String appFilePath2 = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME2;

		try {

			ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);

			// Navigate to the Configuration tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			earRow.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.tabMenu.clickContentTab();

			String xPath = ".//table//input[@type='file']";
			HtmlFileInput fileInput = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			fileInput.setValueAttribute(appFilePath2);
			xPath = ".//table//input[@type='submit']";
			HtmlSubmitInput submit = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			submit.click();

			// We should get:
			// The specified file must have the same name as the existing
			// deployed EAR File (EarWithWar-Counter.ear).
			String msg = "The specified file must have the same name " +
				"as the existing deployed EAR File";
			checkClientAndServerMessages( msg, msg, FacesMessage.SEVERITY_WARN );

			// Check that the old app is deployed.
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);

			// The deployable with different name shouldn't remain deployed.
			ejtt.sleep(2000); // Give AS some time...
			assertFalse( ejtt.deployment.isDeployedAccordingToEmbJopr(APP_TYPE, DEPLOYABLE_NAME2, false) );

		}
		finally {
			try {
				// undeployEar( DEPLOYABLE_NAME );
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			}catch( EmbJoprTestException ex ){
				log.error("Caught exception when undeploying '"+DEPLOYABLE_NAME+"': "+ex);
			}
			try {
				//undeployEar( DEPLOYABLE_NAME2 );
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME2);
			}catch( EmbJoprTestException ex ){
				log.error("Caught expected exception when undeploying '"+DEPLOYABLE_NAME2+"': "+ex);
			}
		}
	
	}




	/**
	 *  Deploys an EAR, then re-deploys (Content tab) EAR with the same name, but different content.
	 */
	public void testEarRedeployDifferentVersion() throws IOException, EmbJoprTestException
	{

		final String DEPLOYABLE_NAME  = EAR_WITH_WAR_COUNTER;
		final String DEPLOYABLE_NAME_2 = EAR_WITH_WAR_COUNTER_20;


		try {

			// Deploy the first EAR.
			String appFilePath = ejtt.getTestDataDir() +"/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr( APP_TYPE, appFilePath );

			ejtt.getNavTree().getNodeByLabel(NAV_EAR).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);


			// Copy new version to tmp dir under old ver's filename.
			String srcFilePath =  ejtt.getTestDataDir() +"/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME_2;
			String destFilePath = ejtt.getTempDir() +"/"+DEPLOYABLE_NAME;
			FileUtils.copyFile(new File(srcFilePath), new File(destFilePath));
			String appFilePath2 = destFilePath;

			// Navigate to the Configuration tab
			ContentTableRow earRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			earRow.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.tabMenu.clickContentTab();

			// Deploy the second version
			String xPath = ".//table//input[@type='file']";
			HtmlFileInput fileInput = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			fileInput.setValueAttribute(appFilePath2);
			xPath = ".//table//input[@type='submit']";
			HtmlSubmitInput submit = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			submit.click();

			// We should get:
			// The specified file must have the same name as the existing
			// deployed EAR File (EarWithWar-Counter.ear).
			String msg = "EAR File "+DEPLOYABLE_NAME+" updated.";
			checkClientAndServerMessages( msg, msg, FacesMessage.SEVERITY_INFO );

			// Check that the old app is deployed.
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME, 3000, 15);

		}
		finally {
			try {
				undeployEar( DEPLOYABLE_NAME );
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}

	}




	/*
	 * testName: testRarInEarDeployment
         * assertion:  verify the deployment of Enterprise Archive which contains a resource adapter jar
	 * test Strategy:  Navigate to Enterprise Applications.
	 *	Add a new resource.  Verify the resource was successfully
	 *	deployed.  Undeploy the archive for test clean up purposes.
	 *
	 */ 
	public void testEarRarInEarDeployment() throws IOException, EmbJoprTestException
	{

		final String DEPLOYABLE_NAME =  EAR_WITH_RAR;

		// Deploy the EAR.

		String earFilePath = ejtt.getTestDataDir() + "/ear/"+DEPLOYABLE_NAME;
		deployEar( earFilePath );

		String expectedMessage = DEPLOYABLE_NAME + " created successfully";
		checkClientAndServerMessages(expectedMessage, expectedMessage, false);


		assertTrue("JMX doesn't report EAR as deployed: testscopedrar.ear", isEarDeployed(DEPLOYABLE_NAME));
		assertTrue("JMX doesn't report RAR as deployed. jbosstestadapter.rar", isRarInEarDeployed(BASIC_RAR, EAR_WITH_RAR));

		// Check that the Embedded Rar is listed 
		log.info("Navigate to Embedded RAR");
		ejtt.navTree.getNodeByLabel(DeployableTypes.EMB_RAR.getNavTreeLabel()).getLabelLink().click();
		log.info("Assert that the Embedded Rar is Available and UP");
		assertTrue( ejtt.deployment.isDeployedAccordingToEmbJopr(DeployableTypes.EMB_RAR, BASIC_RAR, true) );

		// Undeploy the EAR.
		log.info("Undeploy the EAR");
		ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
		log.info("Check that the ear Deployment returns false");
		assertFalse(isEarDeployed(DEPLOYABLE_NAME));
		log.info("Check that the rar Deployment returns false");
		assertFalse(isRarInEarDeployed(BASIC_RAR, EAR_WITH_RAR));


	}


	private void deployEar( String earFilePath ) throws IOException, HtmlElementNotFoundException
	{
		if( !(new File(earFilePath)).exists())
			throw new FileNotFoundException(earFilePath);

		log.info("Deploying: "+earFilePath);

		// Navigate to Enterprise Archives
		ejtt.getNavTree().getNodeLink(NAV_EAR).click();

		// click on the "Add new resource" button
		client.click("actionHeaderForm:addNewContent");  // 404 if setThrowExceptionOnFailingStatusCode(true) above

		// Upload the file
		HtmlFileInput fileInput = (HtmlFileInput)client.getElement("createContentForm:file");
		//fileInput.setContentType("application/ear");
		//fileInput.setContentType("application/octet-stream");
		fileInput.setValueAttribute(earFilePath);
		fileInput.setData( FileUtils.readFileToByteArray( new File(earFilePath) ) );
		client.click("createContentForm:addButton");
		ejtt.sleep( 500 );

		// Log the message
		ejtt.logServerMessage("Something went wrong with deploy: ");
	}

	private void undeployEar( String earFileName ) throws IOException, HtmlElementNotFoundException, ActionNotAvailableException
	{

		// Navigate to Enterprise Archives
		ejtt.getNavTree().getNodeLink(NAV_EAR).click();

		ejtt.getTabMenu().clickSummaryTab();

		/// Debug log - we're getting HTTP 500 here :/
		String dumpFile = "target/"+this.getName()+"-undeployEar-"+(1000+RandomUtils.nextInt(1000))+".html";
		log.debug("Dumping page to "+dumpFile);
		DebugUtils.writeFile( dumpFile, client.getPageAsText());
		int httpStatusCode = client.getContentPage().getWebResponse().getStatusCode();
		if( 200 != httpStatusCode )
			log.warn("Status code before undeployment is: "+httpStatusCode);

		HtmlButtonInput deleteButton = getAppDeleteButton( earFileName );
		deleteButton.click();

		// Log the message (with prefix of potential warning)
		ejtt.logServerMessage("Something went wrong with undeploy: ");

		// Sleep for 3 sec.  TODO: write some waitForUndeployed()
		ejtt.sleep( 3000 );

	}



}// class EarTest
