/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.jopr.jsfunit.as5.app.ejb;

import java.io.IOException;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;

import org.jboss.jopr.jsfunit.AppConstants.DeployableTypes;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;

/**
 * This class contains tests for testing the EmbJopr Application
 * Management Functions for Enterprise JavaBean archives with JBoss AS 5.
 * 
 * @author Shelly McGowan
 *
 */
public class Ejb30Test extends EjbTestBase {

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		return new TestSuite(Ejb30Test.class);
	}


	public DeployableTypes getAppType() {
		return DeployableTypes.EJB3;
	}

	@Override
	public String getArchiveNameBasic() {
		return BASIC_EJB3;
	}

	@Override
	public String getArchiveNameGood() {
		return GOOD_EJB3_JAR;
	}

	@Override
	public String getArchiveNameBad() {
		return BAD_EJB3_JAR;
	}

	@Override
	public String getArchiveNameRedeploy() {
		return REDEPLOY_EJB3_JAR;
	}


	// --- Additional tests --- //

	private final String EJBS_SUB_DIR = "/"+APP_TYPE.getDataDir()+"/";

	public void testEjb30DeployMDB() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EJB3_MDB;

		try {
			String appPath = ejtt.getTestDataDir() + EJBS_SUB_DIR + DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, appPath);
			String expectedMsg = "Resource " + DEPLOYABLE_NAME + " created successfully!";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
			ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			expectedMsg = "Successfully deleted EJB3 JAR '"+DEPLOYABLE_NAME+"'.";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
		}
		finally {
		}

		
	}

	public void testEjb30DeploySLB() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EJB3_SLB;

		try {
			String appPath = ejtt.getTestDataDir() + EJBS_SUB_DIR + DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, appPath);
			String expectedMsg = "Resource " + DEPLOYABLE_NAME + " created successfully!";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
			ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			expectedMsg = "Successfully deleted EJB3 JAR '"+DEPLOYABLE_NAME+"'.";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
		}
		finally {
		}
	}


	public void testEjb30DeploySFB() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EJB3_SFB;

		try {
			String appPath = ejtt.getTestDataDir() + EJBS_SUB_DIR + DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, appPath);
			String expectedMsg = "Resource " + DEPLOYABLE_NAME + " created successfully!";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
			ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			expectedMsg = "Successfully deleted EJB3 JAR '"+DEPLOYABLE_NAME+"'.";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
		}
		finally {
		}
	}


	public void testEjb30DeployJPAEntity() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = EJB3_ENTITY;

		try {
			String appPath = ejtt.getTestDataDir() + EJBS_SUB_DIR + DEPLOYABLE_NAME;

			ejtt.deployment.deployViaEmbJopr(APP_TYPE, appPath);
			String expectedMsg = "Resource " + DEPLOYABLE_NAME + " created successfully!";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);

			ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			expectedMsg = "Successfully deleted EJB3 JAR '"+DEPLOYABLE_NAME+"'.";
			checkClientAndServerMessages(expectedMsg, expectedMsg, false);
		}
		finally {
		}
	}


}// class



