/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.jopr.jsfunit.as5.app.ejb;

import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;

import org.jboss.jopr.jsfunit.*;
import com.gargoylesoftware.htmlunit.html.*;
import java.io.*;
import java.util.*;
import javax.faces.application.FacesMessage;
import junit.framework.Test;
import org.apache.commons.io.FileUtils;
import org.jboss.jopr.jsfunit.AppConstants.DeployableTypes;
import org.jboss.jopr.jsfunit.exceptions.*;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.*;

/**
 * This class contains tests for testing the EmbJopr Application
 * Management Functions for Enterprise JavaBean archives with JBoss AS 5.
 * 
 * @author Ondrej Zizka
 *
 */
public abstract class EjbTestBase extends ApplicationTestBaseAS5 {

	public abstract DeployableTypes getAppType();
	public abstract String getArchiveNameBasic();
	public abstract String getArchiveNameGood();
	public abstract String getArchiveNameBad();
	public abstract String getArchiveNameRedeploy();


	public final DeployableTypes APP_TYPE = this.getAppType();

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		throw new UnsupportedOperationException("suite() method must be overriden.");
		//return new TestSuite(EjbTestBase.class);
	}





	/*
	 * testName: testBasicEjbDeployment
	 * assertion:  verify basic deployment of an EJB JAR.
	 * test Strategy:  Navigate to EJB Applications.
	 *      Add a new resource.  Verify the resource was successfully
	 *      deployed.  Undeploy the archive.
	 *
	 */
	public void testEjbBasicDeployment() throws IOException, EmbJoprTestException {

		final String DEPLOYABLE_NAME = getArchiveNameBasic();

		DebugUtils.writeFile("target/noForm.html", client.getPageAsText());///

		try {
			// Navigate to EJB Applications
			//HtmlAnchor ejbLink = getNavTreeLink(APP_TYPE.getNavTreeLabel());
			//ejbLink.click();
			ejtt.navTree.getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();

			// click on the "Add new resource" button
			client.click("actionHeaderForm:addNewContent");  // 404 if setThrowExceptionOnFailingStatusCode(true) above

			// TODO: "/ejb/BASIC_JAR" causes exceptions in seam:
			// http://wwwapps.rdu.redhat.com/w3xpastebin/pastebin.php?show=9842

			String filePath = ejtt.getTestDataDir() + "/" + APP_TYPE.getDataDir() + "/" + DEPLOYABLE_NAME;
			log.info("Uploading EJB archive: " + filePath);
			File uploadFile = new File(filePath);
			if (!uploadFile.exists()) {
				throw new EmbJoprTestException("Can't find EJB file to upload: '" + filePath + "'");
			}

			// upload ejb
			HtmlFileInput fileInput = (HtmlFileInput) client.getElement("createContentForm:file");
			fileInput.setContentType("application/ejb");
			fileInput.setValueAttribute(filePath);
			client.click("createContentForm:addButton");

			log.info("HTTP status after EJB upload: " + client.getContentPage().getWebResponse().getStatusCode());
			log.info("renderResponse() called: " + server.getFacesContext().getRenderResponse());
			log.info("responseComplete() called: " + server.getFacesContext().getResponseComplete());

			String errorMessage =
							(server.getFacesMessages().hasNext() && server.getFacesMessages().next().getSeverity() != FacesMessage.SEVERITY_INFO)
							? " Faces message: " + server.getFacesMessages().next()
							: ""; // Awful code

			// FAILS because of NPE and HTTP 500.
			// assert that the success message appeared on the client side
			assertTrue("Success message not found." + errorMessage, client.getPageAsText().contains("Resource " + DEPLOYABLE_NAME + " created successfully!"));

			// assert text and severity level for FacesMessage on server side
			assertTrue(server.getFacesMessages().hasNext());
			FacesMessage successMessage = server.getFacesMessages().next();
			assertTrue(FacesMessage.SEVERITY_INFO.equals(successMessage.getSeverity()));
			assertTrue(successMessage.getDetail().contains("Resource " + DEPLOYABLE_NAME + " created successfully!"));

			assertFalse("Page contains 'Total: 0', EJBs probably not listed.", client.getPageAsText().contains("Total: 0"));


			// Use JMX to assert that the EJB components really did deploy successfully
			//assertTrue("JMX doesn't report '"+DEPLOYABLE_NAME+"' as deployed.", isEJBDeployed(DEPLOYABLE_NAME));
			assertTrue("JMX doesn't report '"+DEPLOYABLE_NAME+"' as deployed.", ejtt.deployment.isDeployedAccordingToJMX(APP_TYPE, DEPLOYABLE_NAME));

			// Undeploy the EJB JAR
			HtmlButtonInput deleteButton = getAppDeleteButton(DEPLOYABLE_NAME);
			deleteButton.click();

			String expectedMessage = String.format(EJB_DEL_MSG_FORMAT, APP_TYPE, DEPLOYABLE_NAME);
			assertTrue("Page doesn't contain success message.", client.getPageAsText().contains(expectedMessage));
			assertFalse("Deployable should be undeployed: " + DEPLOYABLE_NAME, isEJBDeployed(DEPLOYABLE_NAME));
		}
		finally {
			// Undeploy the EJB JAR
			try {
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			} catch (EmbJoprTestException ex) {
				log.debug("Expected exception during undeployment - " + ex.getMessage());
			}
		}

	}//






	
	/**
		test Strategy:

		Deploy an ejb-jar that is known to have a bad deployment
		descriptor.  Verify the console shows deployment failed.
		Upload the fixed the achive, redeploy and
		verify the archive has been successfully deployed.
	 */
	public void testEjbDeployBadAndRedeploy() throws IOException, HtmlElementNotFoundException, ActionNotAvailableException, EmbJoprTestException {

		final String REDEPLOY_JAR_NAME = getArchiveNameRedeploy();

		try {
			String badFileSrcPath = ejtt.getTestDataDir() + "/" + APP_TYPE.getDataDir() + "/" + getArchiveNameBad();
			String goodFileSrcPath = ejtt.getTestDataDir() + "/" + APP_TYPE.getDataDir() + "/" + getArchiveNameGood();
			String ejbFilePath = ejtt.getTestDataDir() + "/" + APP_TYPE.getDataDir() + "/" + REDEPLOY_JAR_NAME;

			// Copy bad file under good file's name (we need to keep the filename the same for redeployment).
			FileUtils.copyFile(new File(badFileSrcPath), new File(ejbFilePath));
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, ejbFilePath);
			assertFalse("JMX shouldn't report " + REDEPLOY_JAR_NAME + " as deployed - it's renamed malformed jar '"+getArchiveNameBad()+"'.", isEJBDeployed(REDEPLOY_JAR_NAME));

			// Copy fixed archive to same name, and deploy.
			FileUtils.copyFile(new File(goodFileSrcPath), new File(ejbFilePath));
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, ejbFilePath);

			// Assert text and severity level for FacesMessage on server side.
			assertTrue(server.getFacesMessages().hasNext());
			FacesMessage successMessage = server.getFacesMessages().next();
			assertTrue(FacesMessage.SEVERITY_INFO.equals(successMessage.getSeverity()));
			assertTrue(successMessage.getDetail().contains("Resource " + REDEPLOY_JAR_NAME + " created successfully!"));
			// Use JMX to assert that the EJB components really did deploy successfully.
			assertTrue("JMX should report " + REDEPLOY_JAR_NAME + " as deployed.", ejtt.deployment.isDeployedAccordingToJMX(APP_TYPE, REDEPLOY_JAR_NAME));
			
		}
		finally {
			// Undeploy the EJB JAR
			try {
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, REDEPLOY_JAR_NAME);
			} catch (EmbJoprTestException ex) {
				log.debug("Exception during undeployment - " + ex.getMessage());
			}
		}
	}




	/**
	 *
	 */
	public void testEjbSummaryTab() throws EmbJoprTestException, IOException {

		final String DEPLOYABLE_NAME = getArchiveNameBasic();

		try {

			// Deploy the APP
			String appFilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/" + DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, appFilePath);

			// Click on the app link.
			ContentTableRow appRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			appRow.getLinkByLabel(DEPLOYABLE_NAME).click();


			// Check the values in info table(s)

			// General Properties
			HtmlTable genpropTable = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("General Properties").getElement();
			ContentInfoTable infoTable = ejtt.getContentInfoTable(genpropTable);
			Properties props = infoTable.getProperties();

			// Name
			assertEquals(DEPLOYABLE_NAME, props.getProperty("Name").trim());
			// Version: "--"
			// Description: "a standalone EJB 2.x application"


			// Traits
			infoTable = ejtt.getContentInfoTable(
							ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("Traits").getElement());
			props = infoTable.getProperties();

			// Path
			String path = ejtt.getDeployDir() + "/" + DEPLOYABLE_NAME;
			assertEquals(path, props.getProperty("Path").trim());

			// Exploded
			assertEquals("no", props.getProperty("Exploded?").trim());

			/* Currently hidden.
			// Metrics Summary
			HtmlTable summaryTable = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("Numeric Metrics").getElement();
			infoTable = ejtt.getContentInfoTable(summaryTable);
			/**/

		} finally {
			try {
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			} catch (Exception ex) {
				log.error("Caught exception when undeploying: " + ex, ex);
			}
		}

	}



	
	/**
	 *
	 */
	public void testEjbMetricsTab() throws EmbJoprTestException, IOException {

		final String DEPLOYABLE_NAME = getArchiveNameBasic();

		try {

			// Deploy the APP
			String appFilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/" + DEPLOYABLE_NAME;
			ejtt.deployment.deployViaEmbJopr(APP_TYPE, appFilePath);

			// Click on the app link.
			ContentTableRow appRow = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			appRow.getLinkByLabel(DEPLOYABLE_NAME).click();

			// Switch to the Metrics tab.
			ejtt.tabMenu.getTabByID(METRICS_TAB).click();


			// Check the values in info table(s)

			/* Was removed.
			// General Properties
			HtmlTable genpropTable = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("General Properties").getElement();
			ContentInfoTable infoTable = ejtt.getContentInfoTable(genpropTable);
			Properties props = infoTable.getProperties();
			*/

			// Traits
			ContentInfoTable infoTable = ejtt.getContentInfoTable(
							ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("Traits").getElement());
			Properties props = infoTable.getProperties();

			// Path
			String path = ejtt.getDeployDir() + "/" + DEPLOYABLE_NAME;
			assertEquals(path, props.getProperty("Path").trim());


			/* Currently no num metrics..
			// Metrics Summary
			HtmlTable summaryTable = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader("Numeric Metrics").getElement();
			infoTable = ejtt.getContentInfoTable(summaryTable);
			/**/


			// Finally, click the Refresh button.
			String xPath = ".//input[normalize-space(@value)='Refresh']";
			HtmlInput button = ejtt.tabMenu.getTabContentBox().getElement().getFirstByXPath(xPath);
			if( null == button )
				throw new HtmlElementNotFoundException("Refresh button not found using XPath: "+xPath);

			button.click();

			// Check that we are still on Metrics tab.
			assertTrue( ejtt.tabMenu.isTabActiveByID(METRICS_TAB) );


		} finally {
			try {
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			} catch (Exception ex) {
				log.error("Caught exception when undeploying: " + ex, ex);
			}
		}

	}





	/**
	 * Stops and Starts an EJB.
	 *
	 * Was failing because of EMBJOPR-133.
	 * Now FAILS because of EMBJOPR-172.
	 */
	public void testEjbStopAndStart() throws IOException, EmbJoprTestException
	{
		final String DEPLOYABLE_NAME = getArchiveNameBasic();


		// Deploy the APP.
		String appfilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME;
		ejtt.deployment.deployViaEmbJopr(APP_TYPE, appfilePath);


		try {

			// Wait until app is UP
			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME,
							3000, 15, this);

			// Navigate to the web app, then to it's Control tab.
			ContentTableRow row = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			row.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.getTabMenu().clickControlTab();

			// Click the Stop button.
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Stop").click();

			// Wait for the op to finish and check the final status.
			ejtt.operations.waitActivelyForOperationToFinish("Stop", 1500, 5);
			assertTrue( ejtt.getTabMenu().getTabContentBox().getOperationsHistoryTable().wasLastOperationSuccesful() );



			// Click the Start button.
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Start").click();
			ejtt.operations.waitActivelyForOperationToFinish("Start", 1500, 5);

		}
		finally {
			try {
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}
	}



	
	/**
	 * Restarts the EJB in Control tab.
	 */
	public void testEjbRestart() throws IOException, EmbJoprTestException
	{
		final String DEPLOYABLE_NAME = getArchiveNameBasic();


		// Deploy the app.
		String appFilePath = ejtt.getTestDataDir() + "/"+APP_TYPE.getDataDir()+"/"+DEPLOYABLE_NAME;
		ejtt.deployment.deployViaEmbJopr(APP_TYPE, appFilePath);

		try {

			// Wait until app is UP
			ejtt.getNavTree().getNodeByLabel(APP_TYPE.getNavTreeLabel()).click();
			ejtt.deployment.waitActivelyForDeployment(APP_TYPE, DEPLOYABLE_NAME,
							3000, 15, this);


			// Navigate to the app, then to it's Control tab.
			ContentTableRow row = ejtt.getDefaultContentTable().getFirstRowContainingLink(DEPLOYABLE_NAME);
			row.getLinkByLabel(DEPLOYABLE_NAME).click();
			ejtt.getTabMenu().clickControlTab();

			// Click the Restart button.
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Restart").click();

			// Wait for the op to finish and check the final status.
			ejtt.operations.waitActivelyForOperationToFinish("Restart", 1500, 5);
			assertTrue( ejtt.getTabMenu().getTabContentBox().getOperationsHistoryTable().wasLastOperationSuccesful() );

		}
		finally {
			try {
				ejtt.deployment.undeployViaEmbJopr(APP_TYPE, DEPLOYABLE_NAME);
			}catch( Exception ex ){
				log.error("Caught exception when undeploying: "+ex, ex);
			}
		}
	}





}// class



