package org.jboss.jopr.jsfunit.as5.jbcache;

import java.io.IOException;
import java.util.Properties;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.SingleProperties;
import org.jboss.jopr.jsfunit.util.ValuesValidator;

/**
 *
 * @author Ondrej Zizka
 *
 * JBoss Cache is only present in 'all' JBoss AS profile.
 */
public class JBossCacheDataContainerTest extends JBossCacheTestBase {



	/**
	 * Interceptor Summary tab.
	 */
	public void testJBossCacheDataContainerSummary() throws IOException, EmbJoprTestException {
		if( isJBossConfigWithoutCache() ) return;

		try {

			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_DATA_CONTAINER).click();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "Data Container");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);
			
		}
		finally {
		}

	}



	// Operations
	private static final String LABEL_NUMBER_OF_NODES = "Get the number of nodes.";
	private static final String LABEL_NUMBER_OF_ATTRIBUTES = "Get the number of attributes.";
	private static final String LABEL_DETAILS_TEXT = "Print details as plain text.";
	private static final String LABEL_DETAILS_HTML = "Print details as HTML.";



	/** Helper method - performs an operation, collects the results and checks whether the values are as expected. */
	public void checkOperationResults( String opName, Properties expected, ValuesValidator validator ) throws EmbJoprTestException, IOException
	{
           	// For all calling test.
                if( isJBossConfigWithoutCache() ) return;

		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_DATA_CONTAINER).click();
		ejtt.getTabMenu().clickControlTab();

		// Perform the operation, get the values, and check them.
		Properties actual = ejtt.operations.performOperation( opName );
		ejtt.checkValues(expected, actual, validator);
	}

	

	/** Number of Nodes. */
	public void testJBossCacheDataContainerNumNodes() throws EmbJoprTestException, IOException
	{
		checkOperationResults(LABEL_NUMBER_OF_NODES, new SingleProperties("Output", "1"), ValuesValidator.EQUAL );
	}


	/** Number of Attributes. */
	public void testJBossCacheDataContainerNumAttributes() throws EmbJoprTestException, IOException
	{
		checkOperationResults(LABEL_NUMBER_OF_ATTRIBUTES, new SingleProperties("Output", "0"), ValuesValidator.EQUAL );
	}


	/** Print details as plain text.  Just a check for currently returned value, can change in future. */
	public void testJBossCacheDataContainerDetailsAsText() throws EmbJoprTestException, IOException
	{
		checkOperationResults(LABEL_DETAILS_TEXT, new SingleProperties("Output", "/ {} /__HA_JNDI__ {}"), ValuesValidator.CONTAINS );
	}


	/** Print details as HTML.  Just a check for currently returned value, can change in future. */
	public void testJBossCacheDataContainerDetailsAsHtml() throws EmbJoprTestException, IOException
	{
		checkOperationResults(LABEL_DETAILS_HTML, new SingleProperties("Output", "/&nbsp;&nbsp;{}<br/>&nbsp;&nbsp;/__HA_JNDI__&nbsp;&nbsp;{}"), ValuesValidator.CONTAINS );
	}








}// class
