package org.jboss.jopr.jsfunit.as5.jbcache;

import java.io.IOException;
import java.util.Properties;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException;
import org.jboss.qa.jbcache.testear.client.*;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.ValuesValidator;

/**
 *
 * @author Ondrej Zizka
 *
 * JBoss Cache is only present in 'all' JBoss AS profile.
 *
 *
 * These tests are failing because of JOPR-334 .
 *
 */
public class JBossCacheInterceptorTest extends JBossCacheTestBase {



	/**
	 * Interceptor Summary tab.
	 */
	public void testJBossCacheInterceptorSummary() throws IOException, EmbJoprTestException {
		if( isJBossConfigWithoutCache() ) return;


		ejtt.deployment.deployViaEmbJopr( DeployableTypes.EAR, ejtt.getTestFilePath(this.APP_TYPE, JBCACHE_TEST_EAR));


		try {
			resetStatistics();

			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "Interceptor");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);


			/*
				Numeric Metrics
					Hit/Miss Ratio	      0	     Hit/Miss Ratio
					Read/Write Ratio	    0	     Read/Write Ratio
					Number Of Attributes	0	     the number of cache attributes
					Average Write Time	  0.0ms	 the average amount of time per write operation
					Average Read Time	    0.0ms	 the average amount of time per read operation
			 */
			props = new Properties();
			props.setProperty("Hit/Miss Ratio", "0");
			props.setProperty("Read/Write Ratio", "0");
			props.setProperty("Number Of Attributes", "0");
			props.setProperty("Average Write Time", "0.0ms");
			//props.setProperty("Average Read Time", "0.0ms");
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValuesEqual(props);

		}
		finally {
			ejtt.deployment.undeployViaEmbJopr( DeployableTypes.EAR, JBCACHE_TEST_EAR );
		}

	}






	/**
	 * Interceptor Summary tab after querying Hibernate.
	 */
	public void testJBossCacheInterceptorSummary_AfterQuery() throws IOException, EmbJoprTestException, NamingException {
		if( isJBossConfigWithoutCache() ) return;

		resetStatistics();

		// Go to the Interceptor's Summary tab and store the properties before query.
		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();
		Properties propsFirst = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).getProperties();



		// Deploy an EAR.
		ejtt.deployment.deployViaEmbJopr( DeployableTypes.EAR, ejtt.getTestFilePath(this.APP_TYPE, JBCACHE_TEST_EAR));

		try {


			// Run the test() method of the bean - performs few Hibernate operations.
			performBeanRequest();


			// Go back to the Interceptor's Summary tab.
			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "Interceptor");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);



			// Check that the latter values are diffrent from the former.
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(propsFirst, ValuesValidator.NOT_EQUAL);


			// Check that these metrics didn't stay at their init values.
			/*
				Numeric Metrics
					Hit/Miss Ratio	      0	     Hit/Miss Ratio
					Read/Write Ratio	    0	     Read/Write Ratio
					Number Of Attributes	0	     the number of cache attributes
					Average Write Time	  0.0ms	 the average amount of time per write operation
					Average Read Time	    0.0ms	 the average amount of time per read operation
			 */
			props = new Properties();
			props.setProperty("Hit/Miss Ratio", "0");
			props.setProperty("Read/Write Ratio", "0");
			props.setProperty("Number Of Attributes", "0.0ms");
			props.setProperty("Average Write Time", "0.0ms");
			props.setProperty("Average Read Time", "0.0ms");
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.NOT_EQUAL);

		}
		finally {
			ejtt.deployment.undeployViaEmbJopr( DeployableTypes.EAR, JBCACHE_TEST_EAR );
		}

	}








	/**
	 * Interceptor Metrics tab.
	 */
	public void testJBossCacheInterceptorMetrics() throws IOException, EmbJoprTestException {
		if( isJBossConfigWithoutCache() ) return;

		resetStatistics();

		ejtt.deployment.deployViaEmbJopr( DeployableTypes.EAR, ejtt.getTestFilePath(this.APP_TYPE, JBCACHE_TEST_EAR));
		try {

			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();
			ejtt.tabMenu.clickMetricsTab();

			// Traits.
			Properties props = new Properties();
			props.setProperty("Statistics Enabled", "true");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_TRAITS).checkValuesEqual(props);

			// Numeric Metrics
			props = getInitialMetricsProperties();
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValuesEqual(props);

		}
		finally {
			ejtt.deployment.undeployViaEmbJopr( DeployableTypes.EAR, JBCACHE_TEST_EAR );
		}

	}




	/**
	 * Interceptor Metrics tab after querying Hibernate.
	 */
	public void testJBossCacheInterceptorMetrics_AfterQuery() throws IOException, EmbJoprTestException, NamingException {
		if( isJBossConfigWithoutCache() ) return;

		resetStatistics();

		// Go to the Interceptor's Metrics tab and store the properties before query.
		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();
		ejtt.tabMenu.clickMetricsTab();
		Properties propsFirst = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).getProperties();



		// Deploy an EAR.
		ejtt.deployment.deployViaEmbJopr( DeployableTypes.EAR, ejtt.getTestFilePath(this.APP_TYPE, JBCACHE_TEST_EAR));

		try {


			// Run the test() method of the bean - performs few Hibernate operations.
			performBeanRequest();


			// Go back to the Interceptor's Metrics tab.
			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "Interceptor");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);


			// Check that the latter values are diffrent from the former.
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(propsFirst, ValuesValidator.NOT_EQUAL);


			// Check that these metrics didn't stay at their init values.
			props = getInitialMetricsProperties();
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.NOT_EQUAL);

		}
		finally {
			ejtt.deployment.undeployViaEmbJopr( DeployableTypes.EAR, JBCACHE_TEST_EAR );
		}

	}








	/**
	 * Interceptor Control tab after querying Hibernate.
	 */
	public void testJBossCacheInterceptorControl_AfterQuery() throws IOException, EmbJoprTestException, NamingException {
		if( isJBossConfigWithoutCache() ) return;

		resetStatistics();

		// Go to the Interceptor's Control tab and store the properties before query.
		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();
		ejtt.tabMenu.clickMetricsTab();
		Properties propsFirst = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).getProperties();


		// Deploy an EAR.
		ejtt.deployment.deployViaEmbJopr( DeployableTypes.EAR, ejtt.getTestFilePath(this.APP_TYPE, JBCACHE_TEST_EAR));

		try {


			// Run the test() method of the bean - performs few Hibernate operations.
			performBeanRequest();


			// Go back to the Interceptor's Control tab and reset the statistics.
			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();
			ejtt.tabMenu.clickMetricsTab();
			Properties propsSecond = ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).getProperties();

			ejtt.tabMenu.clickControlTab();
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Reset Statistics");

			ejtt.tabMenu.clickMetricsTab();

			// Check that these metrics didn't stay at their previous values.
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(propsSecond, ValuesValidator.NOT_EQUAL);

			// Check that these metrics are back at their initial values.
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues( this.getInitialMetricsProperties(), ValuesValidator.EQUAL);

		}
		finally {
			ejtt.deployment.undeployViaEmbJopr( DeployableTypes.EAR, JBCACHE_TEST_EAR );
		}

	}





	// ------ Helper methods ------ //



	/** Initial metrics values  - should be there after start or reset. */
	private Properties getInitialMetricsProperties(){
			/*
				Numeric Metrics
						Name	Value	Description
					Category: performance
						Evictions         0	the number of cache eviction operations
						Hit/Miss Ratio    0	Hit/Miss Ratio
						Read/Write Ratio  0	Read/Write Ratio
						NumberOfNodes     1	the number of nodes in the cache
						Hits              0	the number of cache attribute hits
						Number Of Attributes	0	the number of cache attributes
						Misses                1	the number of cache attribute misses
						Average Write Time    0.0ms	the average amount of time per write operation
						Stores                0	the number of cache attribute put operations
						Average Read Time	    18.0ms	the average amount of time per read operation
					Category: utilization
						Elapsed Time          9.1m	the time elapsed since the cache was started
						Time Since Reset      9.1m	the time elapsed since the cache statistics were last reset
			 */
			Properties props = new Properties();
			props.setProperty("Evictions", "0");
			props.setProperty("NumberOfNodes", "1");
			props.setProperty("Hits", "0");
			props.setProperty("Misses", "1");
			props.setProperty("Stores", "0");
			props.setProperty("Hit/Miss Ratio", "0");
			props.setProperty("Read/Write Ratio", "0");
			props.setProperty("Number Of Attributes", "0");
			props.setProperty("Average Write Time", "0.0ms");
			//props.setProperty("Average Read Time", "18.0"); // Not zero after startup.

			props.setProperty("Elapsed Time", "0.0");
			props.setProperty("Time Since Reset", "0.0m");

			return props;
	}







	


	/**
	 * Calls the test bean's methods which do some JPA entities manipulation.
	 * @throws javax.naming.NamingException
	 */
	private void performBeanRequest() throws NamingException{
			final String BEAN_JNDI = "TestOperationBean/remote";

			// Query the bean.
			log.info("Looking up "+BEAN_JNDI+"...");
			InitialContext ctx = new InitialContext();
			TestOperation bean = (TestOperation) ctx.lookup(BEAN_JNDI);

			log.info("Sending request...");
			bean.test();
	}



	private void resetStatistics() throws HtmlElementNotFoundException, EmbJoprTestException, IOException {
			// Go back to the Interceptor's Control tab and reset the statistics.
			ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
			ejtt.navTree.getNodeByLabel(SUBNODE_INTERCEPTOR).click();
			ejtt.tabMenu.clickControlTab();
			ejtt.tabMenu.getTabContentBox().getButtonInputByLabel("Reset Statistics");
	}






}// class
