package org.jboss.jopr.jsfunit.as5.jbcache;

import java.io.IOException;
import java.util.Properties;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.SingleProperties;
import org.jboss.jopr.jsfunit.util.ValuesValidator;

/**
 *
 * @author Ondrej Zizka
 *
 * JBoss Cache is only present in 'all' JBoss AS profile.
 */
public class JBossCacheRegionManagerTest extends JBossCacheTestBase {


	private static final String LABEL_DUMP_REGIONS = "Dump Regions";


	/** Navigates to the node being tested. */
	private void navigateToNode() throws EmbJoprTestException, IOException {
		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_REGION_MANAGER).click();
	}

	private void dumpRegions() throws EmbJoprTestException, IOException{
		navigateToNode();
		ejtt.getTabMenu().clickControlTab();
		ejtt.operations.performOperation(LABEL_DUMP_REGIONS);
	}


	/**
	 * Region Manager Summary tab.
	 */
	public void testJBossCacheRegionManagerSummary() throws IOException, EmbJoprTestException {
		if( isJBossConfigWithoutCache() ) return;

		try {

			navigateToNode();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "RegionManager");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);

		}
		finally {
		}

	}


	/**
	 * 
	 */
	public void testJBossCacheRegionManagerMetrics() throws IOException, EmbJoprTestException {

		if( isJBossConfigWithoutCache() ) return;

		navigateToNode();
		ejtt.getTabMenu().clickMetricsTab();


		// Metrics.
		Properties props = new SingleProperties("Number of Regions", "0");
		ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.EQUAL);

	}


	/**
	 *
	 */
	public void testJBossCacheRegionManagerControl() throws IOException, EmbJoprTestException {

		if( isJBossConfigWithoutCache() ) return;

		navigateToNode();
		ejtt.getTabMenu().clickControlTab();
		ejtt.operations.performOperation(LABEL_DUMP_REGIONS);

	}








}// class
