package org.jboss.jopr.jsfunit.as5.jbcache;

import java.io.IOException;
import java.util.Properties;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.SingleProperties;
import org.jboss.jopr.jsfunit.util.ValuesValidator;

/**
 *
 * @author Ondrej Zizka
 *
 * JBoss Cache is only present in 'all' JBoss AS profile.
 */
public class JBossCacheRpcManagerTest extends JBossCacheTestBase {


	/** Navigates to the node being tested. */
	private void navigateToNode() throws EmbJoprTestException, IOException {
		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_RPC_MANAGER).click();
	}

	private void resetStatistics() throws EmbJoprTestException, IOException{
		navigateToNode();
		ejtt.getTabMenu().clickControlTab();
		ejtt.operations.performOperation("Reset Statistics");
	}


	/**
	 * RPC Manager Summary tab.
	 */
	public void testJBossCacheRpcManagerSummary() throws IOException, EmbJoprTestException {
		if( isJBossConfigWithoutCache() ) return;

		try {

			resetStatistics();
			navigateToNode();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "RPC Manager");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);

			// Metrics.
			props = new Properties();
			props.setProperty("Failed Replications", "0");
			props.setProperty("Successful Replications", "0");
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.EQUAL);

		}
		finally {
		}

	}


	/**
	 * 
	 */
	public void testJBossCacheRpcManagerMetrics() throws IOException, EmbJoprTestException {

                if( isJBossConfigWithoutCache() ) return;

		navigateToNode();
		ejtt.getTabMenu().clickMetricsTab();

		/*
			Traits
				RPC Enabled?: true
				Local Address: 127.0.1.1:60357
				Statistics Enabled: false
				Success Ratio: N/A
				Members: [127.0.1.1:60357]
		*/
		Properties props = new Properties();
		props.setProperty("RPC Enabled?", "true");
		props.setProperty("Statistics Enabled", "false");
		props.setProperty("Success Ratio", "N/A");
		ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_TRAITS).checkValues(props, ValuesValidator.EQUAL);

		//props = new SingleProperties("Members", "[127.0.\\d+.\\d+:\\d+]");
		//ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_TRAITS).checkValues(props, ValuesValidator.MATCHES);
		props = new Properties();
		props.setProperty("Members", "[127.0.");
		props.setProperty("Local Address", "127.0.");
		ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_TRAITS).checkValues(props, ValuesValidator.CONTAINS);


		// Metrics.
		props = new Properties();
		props.setProperty("Failed Replications", "0");
		props.setProperty("Successful Replications", "0");
		ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.EQUAL);

	}








}// class
