package org.jboss.jopr.jsfunit.as5.jbcache;

import java.io.IOException;
import java.util.Properties;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.SingleProperties;
import org.jboss.jopr.jsfunit.util.ValuesValidator;

/**
 *
 * @author Ondrej Zizka
 *
 * JBoss Cache is only present in 'all' JBoss AS profile.
 */
public class JBossCacheTransactionTableTest extends JBossCacheTestBase {


	private static final String LABEL_RESET_STATISTICS = "Reset Statistics";


	/** Navigates to the node being tested. */
	private void navigateToNode() throws EmbJoprTestException, IOException {
		ejtt.navTree.getNodeByLabel(JBCACHE_HA_PARTITION).expand();
		ejtt.navTree.getNodeByLabel(SUBNODE_TRANSACTION_TABLE).click();
	}

	private void resetStatistics() throws EmbJoprTestException, IOException{
		navigateToNode();
		ejtt.getTabMenu().clickControlTab();
		ejtt.operations.performOperation(LABEL_RESET_STATISTICS);
	}


	/**
	 * Transaction Table Summary tab.
	 */
	public void testJBossCacheTransactionTableSummary() throws IOException, EmbJoprTestException {
		if( isJBossConfigWithoutCache() ) return;

		try {

			navigateToNode();

			// Gen Props - nothing useful.
			Properties props = new Properties();
			props.setProperty("Name", "Transaction Table");
			ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual(props);

			// Metrics.
			props = new SingleProperties("Number Of Registered Transactions", "0");
			ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.EQUAL);

		}
		finally {
		}

	}


	/**
	 * 
	 */
	public void testJBossCacheTransactionTableMetrics() throws IOException, EmbJoprTestException {

                if( isJBossConfigWithoutCache() ) return;

		navigateToNode();
		ejtt.getTabMenu().clickMetricsTab();


		// Traits
		Properties props = new SingleProperties("Transaction Map", "{}"); // null
		ejtt.getTabMenu().getTabContentBox().getTableUnformatted(LABEL_TRAITS).checkValues(props, ValuesValidator.EQUAL);

		// Metrics.
		props = new SingleProperties("Number Of Registered Transactions", "0");
		ejtt.getTabMenu().getTabContentBox().getTableUnderHeader(LABEL_NUMERIC_METRICS).checkValues(props, ValuesValidator.EQUAL);

	}


	/**
	 *
	 */
	public void testJBossCacheTransactionTableControl() throws IOException, EmbJoprTestException {

                if( isJBossConfigWithoutCache() ) return;

		navigateToNode();
		ejtt.getTabMenu().clickControlTab();
		ejtt.operations.performOperation(LABEL_RESET_STATISTICS);

	}








}// class
