package org.jboss.jopr.jsfunit.as5.sbm;

import java.io.IOException;
import java.util.Properties;
import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.jopr.jsfunit.ApplicationTestBaseAS5;
import org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.ContentInfoTable;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.ContentTable;
import org.jboss.jopr.jsfunit.util.EmbJoprTestToolkit.ContentTableRow;
import org.jboss.jopr.jsfunit.util.SingleProperties;

/**
 * See  <JBOSS_HOME>/server/default/conf/bindingservice.beans/META-INF/bindings-jboss-beans.xml
 * and  http://www.jboss.org/community/wiki/AS5ServiceBindingManager
 *
 * @author Ondrej Zizka
 */
public class ServiceBindingManagerTest extends ApplicationTestBaseAS5 {

	public static Test suite(){
		return new TestSuite(ServiceBindingManagerTest.class);
	}


	/**
	 * Checks that SBM is listed in the tree, and verifies the Summary tab values.
	 */
	public void testServiceBindingManagerSummaryTest() throws IOException, EmbJoprTestException	{

		// Go to Service Binding Managers node.
		ejtt.navTree.getNodeByLabel(NAV_SBM).click();

		ejtt.tabMenu.getTabContentBox().getTableUnformatted(LABEL_GENERAL_PROPERTIES).checkValuesEqual( new SingleProperties("Name", NAV_SBM) );

	}




	/**
	 * Checks that SBM is listed in the tree, and verifies the Summary tab values.
	 *
	 * DISABLED - to be reimplemented.
	 */
	public void DISABLEDtestServiceBindingManagerConfigTest() throws IOException, EmbJoprTestException	{

		// Go to Service Binding Managers node.
		ejtt.navTree.getNodeByLabel(NAV_SBM).click();
		ejtt.getTabMenu().clickConfigurationTab();

		

		// Check that SBM is UP.
		ContentTableRow row = ejtt.tabMenu.getTabContentBox().getFirstTable().getFirstRowContainingLink(NAV_SBM_SETS);
		assertEquals(NAV_SBM_SETS+" should be UP", "UP", row.getCellTextByColumnName("Status") );

		// Go to SBM.
		row.getLinkByLabel(NAV_SBM_SETS).click();


		// Summary tab
		ContentTable table = ejtt.tabMenu.getTabContentBox().getTableUnderHeader(LABEL_GENERAL_PROPERTIES);
		ContentInfoTable infoTable = ejtt.getContentInfoTable(table.getElement());
		Properties props = infoTable.getProperties();

		// Name
		assertEquals("Name should be "+NAV_SBM_SETS, NAV_SBM_SETS, props.getProperty("Name"));

	}





  /**
	 * Changes "Active Binding Set Name" property in the Conf tab, saves,
	 * and then changes back to original.
	 * TODO: "Server restart needed" - see JOPR-230.
	 */
	public void testServiceBindingManagerConfigurationTest() throws IOException, EmbJoprTestException	{

		final String PROPERTY_activeBindingSetName = "activeBindingSetName";


		// Go to Service Binding Manager node.
		ejtt.navTree.getNodeByLabel(NAV_SBM).click();
		ejtt.getTabMenu().clickConfigurationTab();



		// -- Update -- //

		// Go to Configuration tab.
		ejtt.tabMenu.clickConfigurationTab();

		//ejtt.tabMenu.getTabContentBox().getFirstTable().getFirstRowContainingText("")
		Properties props = new Properties();
		// Carefully - setting this to non-existent leads to disfunctional AS! See
		props.setProperty(PROPERTY_activeBindingSetName, "ports-01");
		fillOutForm(props);

		ejtt.tabMenu.getTabContentBox().getSubmitButtonByLabel("Save").click();

		//String expMsg = "Successfully updated Service Binding Manager '"+NAV_SBM+"'";
		String expMsg = "Successfully updated Service Binding Manager"; // Partial match - matching is done by .contains().
		checkClientAndServerMessages(expMsg, expMsg, false);


		// -- Set back to original -- //

		// Go to Configuration tab.
		ejtt.tabMenu.clickConfigurationTab();

		//ejtt.tabMenu.getTabContentBox().getFirstTable().getFirstRowContainingText("")
		props = new Properties();
		// Carefully - setting this to non-existent leads to disfunctional AS! See
		props.setProperty(PROPERTY_activeBindingSetName, "ports-default");
		fillOutForm(props);

		ejtt.tabMenu.getTabContentBox().getSubmitButtonByLabel("Save").click();

		expMsg = "Successfully updated Service Binding Manager"; // Partial match - matching is done by .contains().Who
		checkClientAndServerMessages(expMsg, expMsg, false);


	}



}// class



