/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2009, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.jopr.jsfunit.exceptions;

import org.jboss.jopr.jsfunit.DebugUtils;
import org.jboss.jopr.jsfunit.EmbjoprTestCase;


/**
 * Root Exception for Exceptions thrown in Embedded Jopr tests.
 * Used to distinguish "expected" exceptions from those unhandled / unexpected.
 *
 * @author ondra
 */
public class EmbJoprTestException extends Exception {

	public EmbJoprTestException(Throwable cause) {
		super(cause);
	}

	/** Writes the current page to a file named "<testName>-ElNotFound.html". */
	public EmbJoprTestException(String message, EmbjoprTestCase test)
					/*throws FileNotFoundException, IOException*/
	{
		super(message);

		if( null == test ) return;

		String path = "target/"+test.getName() + getDumpSuffix()+".html";
		try {
			DebugUtils.writeFile(path, test.getClient().getPageAsText());
		} catch (Throwable ex) {
			System.out.println("ERROR dumping to "+path);
		}
	}

	public EmbJoprTestException(String message, Throwable cause) {
		super(message, cause);
	}

	public EmbJoprTestException(String message) {
		super(message);
	}

	public EmbJoprTestException() {
	}

	private String getDumpSuffix() {
		//return "-ElmNotFoundEx";
		return "-EmbJoprTestEx";
	}
	
}
