
package org.jboss.jopr.jsfunit.util;


import com.gargoylesoftware.htmlunit.ElementNotFoundException;
import com.gargoylesoftware.htmlunit.Page;
import java.net.UnknownHostException;
import java.util.*;

import javax.management.MalformedObjectNameException;
import javax.naming.NamingException;
import org.jboss.jopr.jsfunit.exceptions.*;
import com.gargoylesoftware.htmlunit.html.*;
import com.gargoylesoftware.htmlunit.javascript.JavaScriptEngine;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.InetAddress;
import java.net.URL;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.management.JMException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.math.NumberUtils;
import org.jboss.jopr.jsfunit.AppConstants;
import org.jboss.jopr.jsfunit.AppConstants.DeployableTypes;
import org.jboss.jopr.jsfunit.DebugUtils;
import org.jboss.jopr.jsfunit.EmbJoprTestConstants;
import org.jboss.jopr.jsfunit.EmbjoprTestCase;
import org.jboss.jsfunit.jsfsession.*;
import org.jboss.logging.Logger;
import org.jboss.mx.util.MBeanServerLocator;
import org.mozilla.javascript.NativeFunction;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;



/**
 * Embedded Jopr Test Toolkit - helps with testing EmbJopr's web UI
 * by creating object model of the UI.
 *
 * @author Ondrej Zizka
 */
public class EmbJoprTestToolkit {

	private static Logger log = Logger.getLogger(EmbJoprTestToolkit.class.getName());

	protected JSFClientSession client;
	protected JSFServerSession server;
	private EmbJoprTestToolkit selfEjtt; // For closures.
	public JSFClientSession getClient() { return client; }
	public JSFServerSession getServer() { return server; }

	
	public EmbJoprTestToolkit(JSFClientSession client, JSFServerSession server) throws EmbJoprTestException {
		this.client = client;
		this.server = server;
		this.selfEjtt = this;

		try {
			profileService = new ProfileServiceUtils(true); // TODO: Make "secured" configurable in POM.
		} catch (NamingException ex) {
			String message = "Error when initializing ProfileService: "+ex;
			log.fatal( message );
			throw new EmbJoprTestException(message, ex);
		}
	}


	/** Used in inner classes (they can't use this). */
	public EmbJoprTestToolkit getEjtt(){ return this; }

	/** Returns information about current test being run. Filled before each test method (in setUp()). */
	public EmbjoprTestCase getCurrentTest() {		return currentTest;	}
	public void setCurrentTest(EmbjoprTestCase currentTest) {		this.currentTest = currentTest; }
	private EmbjoprTestCase currentTest = null;






	
	/**
	 * ProfileService references.
	 */
	public final ProfileServiceUtils profileService;


	/**
	 * Single nav tree instance.
	 *
	 * Using instance instead of static methods is necessary
	 * if we want to subclass this tools for AS 4 tests in the future.
	 */
	public final NavTree navTree = new NavTree();

	/**
	 * Single tab menu instance.
	 *
	 * Using instance instead of static methods is necessary
	 * if we want to subclass this tools for AS 4 tests in the future.
	 */
	public final TabMenu tabMenu = new TabMenu();


	
	public NavTree getNavTree() {		return navTree;	}

	public TabMenu getTabMenu() {		return tabMenu;	}





	public String getTestDataDir() {
		return System.getProperty(AppConstants.SYSPROP_TESTDATA_DIR);
	}

	public String getTestFilePath( DeployableTypes location, String fileName ){
		return this.getTestDataDir()+"/"+location.getDataDir()+"/"+fileName;
	}

	public String getTempDir() {
		return System.getProperty(AppConstants.SYSPROP_TEMP_DIR);
	}

	public String getDeployDir() {
		return System.getProperty(AppConstants.SYSPROP_DEPLOY_DIR);
	}

	public String getJBossConfig() {
		return System.getProperty(AppConstants.SYSPROP_JBOSS_CONFIG);
	}

	public boolean isJBossSecured() {
		return System.getProperty(AppConstants.SYSPROP_JBOSS_IS_SECURED,"false").equals("true");
	}

	/**
	 * @returns  the directory in wich the AS instance for cargo is located.
	 *           THIS IS NOT WHERE THE AS CONFIG DIR (with deploy/ etc.) ARE.
	 */
	public String getJBossHomeDir() {
		return System.getProperty(AppConstants.SYSPROP_CARGO_JBOSS_HOME_DIR);
	}

	public JavaScriptEngine getJavaScriptEngine(){
		return client.getContentPage().getEnclosingWindow().getWebClient().getJavaScriptEngine();
	}

	public int getEmbJoprPort(){
		return ( (HttpServletRequest)FacesContext.getCurrentInstance()
						.getExternalContext().getRequest() ).getLocalPort();
	}

	public InetAddress getEmbJoprIpAddress(){
		String localAddr = ( (HttpServletRequest)FacesContext.getCurrentInstance()
						.getExternalContext().getRequest() ).getLocalAddr();
		try {
			InetAddress ia = InetAddress.getByName(localAddr);
			return ia;
		} catch (UnknownHostException ex) {
			throw new IllegalStateException("Error when creating EmbJopr's IP adress: "+ex.toString());
		}
	}

	/**
	 * @returns JavaScript object, as returned by JavaScriptEngine#execute().
	 */
	public Object getJavaScriptObject( String jsExpression ){
		return this.getJavaScriptEngine().execute(
						(HtmlPage) client.getContentPage(),
						jsExpression,
						"testTreeItemLoaded", 0);
	}
	

	/**
	 * Reloads the current page.
	 */
	public void refreshPage() throws IOException {
		((HtmlPage)client.getContentPage()).refresh();
	}


	/** Dumps the page to target directory as a HTML file. */
	public void dumpPage( String fileName ) throws IOException {
		DebugUtils.writeFile("target/"+fileName, this.client.getPageAsText());
	}





	/**
	 * Inner class to encapsulate navigation tree operations.
	 *
	 * Note that this class does not hold any element reference,
	 * because the reference becomes invalid with every navigation action.
	 *
	 * Instead, all these methods call client methods.
	 */
	public class NavTree {

		public static final String ID_NAV_TREE_FORM = "navTreeForm";

		private void clickRootNode() throws IOException, EmbJoprTestException {
			DomElement element = (DomElement)client.getElement(ID_NAV_TREE_FORM);
			if( null == element )
				throw new HtmlElementNotFoundException("Can't find #"+ID_NAV_TREE_FORM+".", currentTest);

			// ID changes upon core build?
			//HtmlAnchor rootNodeLink = element.getFirstByXPath(".//a[@id='navTreeForm:navTree:2::homeLink']");

			// No XSLT 2.0 support :(
			// javax.xml.transform.TransformerException: Can't find function: matches
			//HtmlAnchor rootNodeLink = element.getFirstByXPath(
			//				".//a[matches(@id,'^navTreeForm:navTree:.+::homeLink$')]");

			String xPath = ".//a[ starts-with( @id, '"+ID_NAV_TREE_FORM+":navTree:' ) " +
							           " and contains( @id, '::homeLink' ) ]"; // XPath 2.0: ends-with( @id, '::homeLink' )
			HtmlAnchor rootNodeLink = element.getFirstByXPath(xPath);

			if( null == rootNodeLink )
				throw new HtmlElementNotFoundException("Root node not found using XPath: "+xPath, currentTest);

			rootNodeLink.click();
		}




		/**
		 * Waits and refreshes the page until ProfileService plugin 
		 * has loaded the data for the nav tree.
		 * 
		 * The difference from #waitUntilReady() is that this method
		 * waits for JOPR backend data to be ready (server side),
		 * while waitUntilReady() waits until the Tree.Item JavaScript object
		 * is initialized (client side).
		 *
		 * @param msInterval  Milliseconds to wait between tries.
		 * @param retries     Number of retries. Total time = (retries - 1) * msInterval.
		 * @throws java.lang.Exception  Whatever exception is thrown from the isTrue() method.
		 * @throws org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException
		 *         when max number of retries was reached.
		 * 
		 * @see EMBJOPR-92.
		 */
		public void waitUntilLoaded( int msInterval, int retries ) throws Exception {
			final NavTree self = this;
			//final EmbJoprTestToolkit ejtt = client;
			new ActiveConditionChecker( new DescribedCondition("Nav tree contains Applications node.") {
				int callsMade = 0;
				public boolean isTrue() throws HtmlElementNotFoundException, IOException {
					// Refresh the page if this is not the first call.
					if( callsMade++ > 0 )
						selfEjtt.refreshPage();
					// If found, return true.
					return null != self.findNodeByLabel("Applications");
				}
			}).throwOnTimeout().waitWithTimeout(msInterval, retries);
		}



		/**
		 * Waits until the node with given label exists in the nav tree.
		 */
		public void waitUntilNodeLoadedByAjax( final String nodeLabel, int msInterval, int retries )
						throws EmbJoprTestException
		{
			final NavTree self = this;

			try {
				new ActiveConditionChecker( new DescribedCondition("Nav tree contains "+nodeLabel+" node.") {
					int callsMade = 0;
					public boolean isTrue() throws HtmlElementNotFoundException {
						// If found, return true.
						return null != self.findNodeByLabel(nodeLabel);
					}
				}).throwOnTimeout().waitWithTimeout(msInterval, retries);
			}
			catch( Exception ex ){
				throw new EmbJoprTestException(
								"Exception when finding node '"+nodeLabel+"': "+ex.getMessage(), ex);
			}
		}




		/**
		 * Finds the nav tree node by it's link label.
		 *
		 * @param label
		 * @return
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 *         when there's no node with given label.
		 */
		public NavTreeNode getNodeByLabel( String label ) throws HtmlElementNotFoundException{
			return getNodeByLabel(label, true);
		}
		public NavTreeNode findNodeByLabel( String label ) throws HtmlElementNotFoundException{
			return getNodeByLabel(label, false);
		}
		public NavTreeNode getNodeByLabel( String label, boolean throwIfNotFound ) throws HtmlElementNotFoundException{

			DomElement navTreeForm = (DomElement)client.getElement(ID_NAV_TREE_FORM);
			if( null == navTreeForm ){
				try {
					DebugUtils.writeFile("target/getNodeByLabel-navTreeIDnotFound.html", client.getPageAsText());
				}catch(Exception ex){
					log.error("Caught when writing file: "+ex);
				}
				throw new HtmlElementNotFoundException("Nav tree form not found using ID: "+ID_NAV_TREE_FORM, currentTest);
			}

			// A table which has an anchor containing given text.
			// Note: Different node types have different id endings - typeSummaryLink, categorySummaryLink, ...
			//String xPath = ".//table[.//td[contains(@id,':text')]//a[contains(@id,'SummaryLink') and normalize-space() = '"+label+"']]";
			// Node has @id = navTreeForm:navTree:136:137:138:139:143:195::instanceNodeOrLeafLink
			// Let's rely on the ":text" id part only and suppose there will be just one link in that TD.
			String xPath = ".//table[.//td[contains(@id,':text')]//a[normalize-space() = '"+label+"']]";

			HtmlTable nodeTable = navTreeForm.getFirstByXPath( xPath );

			if( null == nodeTable ){
				if( throwIfNotFound )
					throw new HtmlElementNotFoundException(
									"Node '"+label+"' not found using XPath: "+xPath, currentTest);
				else
					return null;
			}				

			return new NavTreeNode(nodeTable);

		}


    /**
     * Need a standard JSFUnit API to replace this code.
		 * @see getNodeByLabel().getLabelLink()
     */
    public HtmlAnchor getNodeLink(String linkLabel) throws HtmlElementNotFoundException
    {
			//return getLinkInsideForm(ID_NAV_TREE_FORM, linkLabel);
			/*HtmlElement navTreeForm = (HtmlElement) client.getElement(ID_NAV_TREE_FORM);
			HtmlAnchor link = navTreeForm.getFirstByXPath(".//a[normalize-space() = '" + linkLabel + "']");
			if( null == link ){
				throw new HtmlElementNotFoundException(linkLabel)
			}*/
			return getNodeByLabel(linkLabel).getLabelLink();
    }


    /**
     * Finds the arrow icon in the nav tree that corresponds to the resource
     * given by resourceName. This method is used to expand tree nodes
     * (eg. "Web Applications (WAR)", "Datasources", etc.) in the
     * navigation tree.
		 * @see getNodeByLabel().getArrowLink()
     */
    public ClickableElement getNodeArrow(String resourceName) throws HtmlElementNotFoundException
		{
        HtmlAnchor link = getNodeLink(resourceName);
        String id = link.getIdAttribute();

        // An example id is: "navTreeForm:navTree:81:82:83:84::typeSummaryLink"
        // The icon's id would be: "81:82:83:84::typeSummary:handle:img:collapsed"
        int index = id.lastIndexOf("Link");
        id = id.substring(0, index) + ":handle";
        return (ClickableElement)client.getElement(id);
    }


		/**
		 * Returns true if the JS function object 'Tree.Item' is initialized.
		 */
		public boolean isReady(){
			Object obj = getJavaScriptObject("Tree.Item");
			if( null == obj ) return false;
			//if (	org.mozilla.javascript.NOT_FOUND ) return false;
			if( ! (obj instanceof NativeFunction ) ) return false;
			return true;
		}


		/**
		 * Waits until the JS function object 'Tree.Item' is initialized.
		 * @param msInterval  Millisecons between tries.
		 * @param retries     Number of tries.
		 * @throws org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException
		 *		wrapping whatever exception happens when checking the JS object's value.
		 *
		 * @see https://jira.jboss.org/jira/browse/EMBJOPR-101
		 */
		public void waitUntilReady( int msInterval, int retries ) throws EmbJoprTestException {
			final NavTree self = this;
			try {
				new ActiveConditionChecker( new DescribedCondition("Tree.Item JS object is initialized.") {
					public boolean isTrue() throws Exception {
						return self.isReady();
					}
				}).throwOnTimeout().waitWithTimeout(msInterval, retries);
			}
			catch( Exception ex ){
				throw new EmbJoprTestException(
								"Exception when checking Tree.Item JS object: "+ex.getMessage(), ex);
			}
		}


	}// class NavTree





	/**
	 * Represents nav tree node.
	 * Contains convenience methods to work with the node.
	 */
	public class NavTreeNode {

		/** Keeps the table element of this node. */
		private HtmlElement elem;
		public HtmlElement getElement() {			return this.elem;		}

		private NavTreeNode( HtmlTable nodeTable ){
			this.elem = nodeTable;
		}

		/** Returns the display name of the node. */
		public String getName(){
			return this.elem.getTextContent().trim();
		}

		/** Returns true if this node is expanded. */
		public boolean isExpanded() throws EmbJoprTestException
		{
			HtmlAnchor arrowLink = this.getArrowLink();
			HtmlElement img = (HtmlElement)arrowLink.getFirstByXPath("img[contains(@style,'display: none')]");

			if( img.getId().endsWith("expanded") )
				return false;
			else if( img.getId().endsWith("collapsed") )
				return true;
			else
				throw new EmbJoprTestException("Can't determine whether nav tree node is expanded.");

		}

		/**
		 * If the node is collapsed, expand it.
		 * Calls #expand( NodeFinder ) using a finder which looks for a node with the same name as this one.
		 *
		 * @returns true if the node already was expanded.
		 */
		public boolean expand() throws EmbJoprTestException, IOException {
			final String nodeName = this.getName();
			return expand( new NodeFinder() {
				public HtmlTable getNodeElem(EmbJoprTestToolkit ejtt) throws HtmlElementNotFoundException {
					return (HtmlTable) ejtt.navTree.getNodeByLabel( nodeName ).getElement();
				}
			});
		}

		/**
		 * If the node is collapsed, expand it.
		 * Expanding a node causes AJAX call which renders old HTML element stale.
		 * Thus, upon expanding, element must be re-bound. That's what the NodeFinder is for.
		 * 
		 * @returns true if the node already was expanded.
		 */
		public boolean expand( NodeFinder finder ) throws EmbJoprTestException, IOException{

			// null finder -> use default finder.
			if( null == finder)
				return expand();

			boolean isExpanded = isExpanded();
			if( !isExpanded ){
				this.getArrowLink().click();
				this.elem = finder.getNodeElem(selfEjtt);
			}
			return isExpanded;
		}

		/** Returns the text link of this node. */
		public HtmlAnchor getLabelLink(){
			// Until I come up with something smarter, let it be so:
			String xPath = ".//td[contains(@id,':text')]/a";
			return (HtmlAnchor) this.elem.getFirstByXPath( xPath );
		}


		private final String XPATH_ARROW_LINK = ".//td[contains(@id,':handles')]//a[contains(@id,':handle')]";

		/** Returns the arrow's link. */
		public HtmlAnchor findArrowLink(){
			HtmlAnchor arrowLink = (HtmlAnchor) this.elem.getFirstByXPath( XPATH_ARROW_LINK );
			return arrowLink;
		}
		public HtmlAnchor getArrowLink() throws HtmlElementNotFoundException{
			HtmlAnchor arrowLink = this.findArrowLink();
			if( null == arrowLink ){
				throw new HtmlElementNotFoundException("Arrow link not found using XPath: "+XPATH_ARROW_LINK, currentTest);
			}
			return arrowLink;
		}


		/** Clicks the link of the this node. */
		public void click() throws IOException {
			this.getLabelLink().click();
		}

		
		public boolean isExpandable() {
			if( null == this.findArrowLink() )
				return false;
			else
				return true;
		}

		// Note: The div is present even for empty nodes.
		private final String XPATH_CHILDREN_DIV = "./following-sibling::div[contains(@id, ':childs')]";

		public boolean hasChildren() throws EmbJoprTestException, IOException {
			// Let's rely on the arrow, which's presence is a sign of having children.
			return this.isExpandable();
		}


		private final String XPATH_CHILD_NODES = "./following-sibling::div[contains(@id, ':childs') and position()=1]/table";

		public List<NavTreeNode> getChildren() throws EmbJoprTestException, IOException{
			return getChildren(null);
		}

		public List<NavTreeNode> getChildren( NodeFinder finder ) throws EmbJoprTestException, IOException{
			if( ! this.isExpandable() )
				return Collections.EMPTY_LIST;
			this.expand( finder );

			List<HtmlTable> tables = (List<HtmlTable>) this.elem.getByXPath(XPATH_CHILD_NODES);
			
			/*List<NavTreeNode> childNodes = (List<NavTreeNode>) CollectionUtils.collect(tables, new Transformer() {
				public Object transform(Object input) {
					return new NavTreeNode( (HtmlTable)input );
				}
			});/**/

			List<NavTreeNode> childNodes = new ArrayList<NavTreeNode>();
			for (HtmlTable htmlTable : tables) {
				childNodes.add( new NavTreeNode(htmlTable) );
			}

			return childNodes;
		}


	}// class NavTreeNode()


	/** Interface to reload a nav tree node after reloading operation (including AJAX calls). */
	public interface NodeFinder {
		public HtmlTable getNodeElem( EmbJoprTestToolkit ejtt ) throws HtmlElementNotFoundException;
	}





	/**
	 * Inner class to encapsulate tab menu operations.
	 */
	public class TabMenu {

		public TabContentBox getTabContentBox() throws HtmlElementNotFoundException
		{
			HtmlElement contentElement = (HtmlElement) client.getElement("content");
			if( null == contentElement )
				throw new HtmlElementNotFoundException("#content element not found.", currentTest);

			String xPath = "div[@class='tabmenubox' or @class='notabmenubox']";
			HtmlElement tabContentBox = (HtmlElement) contentElement.getFirstByXPath(xPath);
			if( null == tabContentBox )
				throw new HtmlElementNotFoundException("Tab content box not found using XPath: "+xPath, currentTest);

			return new TabContentBox(tabContentBox);
		}

		public ClickableElement getTabByLabel( String label ) throws HtmlElementNotFoundException
		{
			DomElement element = (DomElement)client.getElement("tabmenu");
			String xPath = "ul/li/span[normalize-space(string())='"+label+"'] | ul/li/a[normalize-space(string())='"+label+"']";
			ClickableElement tabContent = element.getFirstByXPath(xPath);


			if( null == tabContent )
				throw new HtmlElementNotFoundException("Tab '"+label+"' not found using XPath '"+xPath+"'", currentTest);

			return tabContent;
		}

		public ClickableElement getTabByID( String tabID ) throws HtmlElementNotFoundException
		{
			ClickableElement element = (ClickableElement)client.getElement(tabID);
			if( null == element )
				throw new HtmlElementNotFoundException("Tab with id '"+tabID+"' not found, perhaps disabled?", currentTest);

			return element;
		}

		public void clickSummaryTab() throws HtmlElementNotFoundException, IOException{ getTabByID("summaryTab").click(); }
		public void clickConfigurationTab() throws HtmlElementNotFoundException, IOException{ getTabByID("configurationTab").click(); }
		public void clickMetricsTab() throws HtmlElementNotFoundException, IOException{ getTabByID("metricsTab").click(); }
		public void clickControlTab() throws HtmlElementNotFoundException, IOException{ getTabByID("controlTab").click(); }
		public void clickContentTab() throws HtmlElementNotFoundException, IOException{ getTabByID("contentTab").click(); }

		/** Returns true if the tab with given label is active (it's content is shown). */
		public boolean isTabActive( String label ) throws IOException, HtmlElementNotFoundException {
			return isTabActive( getTabByLabel(label) );
		}

		/** Returns true if the tab with given ID is active (it's content is shown). */
		public boolean isTabActiveByID( String id ) throws IOException, HtmlElementNotFoundException {
			return isTabActive( getTabByID(id) );
		}

		/** Returns true if the tab with given label is disabled (grayed and can't be activated). */
		public boolean isTabDisabled( String label ) throws HtmlElementNotFoundException {
			return isTabDisabled( getTabByLabel(label) );
		}

		/** Returns true if the tab with given ID is disabled (grayed and can't be activated). */
		public boolean isTabDisabledByID( String id ) throws HtmlElementNotFoundException {
			return isTabDisabled( getTabByID(id) );
		}

		public boolean isTabActive( StyledElement tab ){
			return "span".equals( tab.getTagName() )
							&& tab.getClassAttribute().contains("active");
		}

		public boolean isTabDisabled( StyledElement tab ){
			return "span".equals( tab.getTagName() )
							&& tab.getClassAttribute().contains("disabled");
		}

		/** Shotcut -  getTab(label).click(); Not necesarilly clicks an anchor. */
		public void clickTab( String label ) throws IOException, ActionNotAvailableException, HtmlElementNotFoundException {

			StyledElement tabContent = getTabByLabel(label);
			if( !( tabContent instanceof ClickableElement ) )
				throw new ActionNotAvailableException("Tab element <"+tabContent.getTagName()+"> is not clickable: "+label);

			((ClickableElement)tabContent).click();

		}

	}


	/**
	 * Base class for parts that will check their validity before performing actions.
	 * Not necessary if we pay attention to validity, but why not check it -
	 * performance is not a question for us.
	 *
	 * TODO: Instead, use the Page returned from click() etc.
	 */
	public class PageContextAwareElement {
		private URL validForURL;
		private String validForDate;

		/** Store the context identifiing attributes upon creation. */
		public PageContextAwareElement() {
			// TODO: Shouldn't we store the elements's page instead of current?
			this.validForDate = client.getContentPage().getWebResponse().getResponseHeaderValue("Date");
			this.validForURL = client.getContentPage().getWebResponse().getUrl();
		}

		/**
		 * Checks whether this element is still valid in the current web page context,
		 * i.e. whether we are still on the page in which this element existed.
		 * @throws org.jboss.jopr.jsfunit.exceptions.ActionOutOfSyncException
		 */
		public void checkIfStillValid() throws ActionOutOfSyncException {

			String validForDate_ = client.getContentPage().getWebResponse().getResponseHeaderValue("Date");
			URL    validForURL_  = client.getContentPage().getWebResponse().getUrl();

			if( !validForDate_.equals(this.validForDate) ||
			    !validForURL_.equals(this.validForURL) )
			{
				throw new ActionOutOfSyncException(
								"This element wrapper was created from another page and is not valid now.");
			}
		}

	}
	

	/**
	 * Base class for parts that will check their validity before performing actions.
	 * Not necessary if we pay attention to validity, but why not check it.
	 */
	public class PageAware {

		private Page  validForPage;
		
		public PageAware(){
			this.validForPage = client.getContentPage();
		}
		
		/**
		 * Checks whether this element is still valid in the current web page context,
		 * i.e. whether we are still on the page in which this element existed.
		 * @throws org.jboss.jopr.jsfunit.exceptions.ActionOutOfSyncException
		 */
		public void checkIfStillValid() throws PageHasChangedException  {
			if( !client.getContentPage().equals( this.validForPage ) ){
				throw new PageHasChangedException(
						"This element wrapper was created from another page and is not valid now.",
						client.getContentPage(), this.validForPage );
			}
		}
	}


	/**
	 *  EJTT parts which are based on an HTML element.
	 */
	public interface HtmlElementBacked {
	 public HtmlElement getElement();
	 public HtmlElement getFirstByXPath( String xPath ) throws HtmlElementNotFoundException ;
	 public List<HtmlElement> getByXPath( String xPath ) throws HtmlElementNotFoundException;
	}


	/**
	 *  
	 */
	protected class HtmlElementBackedBase implements HtmlElementBacked {

		protected HtmlTableRow element;
		public HtmlTableRow getElement() {			return element;		}

		public HtmlElementBackedBase(HtmlTableRow element) {
			this.element = element;
		}


		public HtmlElement getFirstByXPath( String xPath ) throws HtmlElementNotFoundException {
			HtmlElement elm = this.element.getFirstByXPath(xPath);
			if( null == elm )
				throw new HtmlElementNotFoundException("No element found using xPath: "+xPath);
			return elm;
		}
		public List<HtmlElement> getByXPath( String xPath ) throws HtmlElementNotFoundException {
			List<HtmlElement> elms = (List<HtmlElement>) this.element.getByXPath(xPath);
			if( elms.size() == 0 )
				throw new HtmlElementNotFoundException("No elements found using xPath: "+xPath);
			return elms;
		}

	}




	/**
	 * Inner class for manipulation with tab content box.
	 */
	public class TabContentBox extends PageAware /*implements HtmlElementBacked*/ {

		private HtmlElement element;
		public HtmlElement getElement() {			return element;		}

		public TabContentBox(HtmlElement element) {
			super();
			this.element = element;
		}


		/**
		 * Returns pagination wrapper.
		 */
		public ContentBoxPagination getPagination(){
			return new ContentBoxPagination();
		}


		public static final String XPATH_TABLE_UNDER_HEADER_FORMAT = "*[contains(normalize-space(), '%s')]/following::table";
		// [@id='resourceSummaryForm:dataTable'] - is this reliable?


		/**
		 * Returns first table under given header.
		 * If the header is not found or there's no table following the header, returns null or throws HtmlElementNotFoundException,
		 * depending on throwIfNotFound.
		 *
		 * Unfortunately, headers are not H2 or similar, but DIV class="instructionalText".
		 * To be revised later.
		 */
		public ContentTable getTableUnderHeader( String headerText, boolean throwIfNotFound ) throws ActionOutOfSyncException, HtmlElementNotFoundException
		{
			checkIfStillValid();

			//String xPath = "*[contains(normalize-space(), '"+headerText+"')]/following::table";
			String xPath = String.format(XPATH_TABLE_UNDER_HEADER_FORMAT, headerText);
			HtmlTable tableElement = (HtmlTable) element.getFirstByXPath( xPath );
			if( null == tableElement ){
				if( throwIfNotFound ){
					throw new HtmlElementNotFoundException(
									"Table under header '"+headerText+"' not found using XPath: "+xPath, currentTest);
				}
				else {
					return null;
				}
			}
			
			return new ContentTable(tableElement, headerText);
		}

		/**
		 * Returns first table under given header.
		 * @throws HtmlElementNotFoundException if the header is not found or there's no table following the header.
		 */
		public ContentTable getTableUnderHeader( String headerText ) throws ActionOutOfSyncException, HtmlElementNotFoundException {
			return getTableUnderHeader(headerText, true );
		}

		public ContentTable findTableUnderHeader( String headerText ) throws ActionOutOfSyncException, HtmlElementNotFoundException {
			return getTableUnderHeader(headerText, false );
		}
		

		/**
		 * Returns first table in the content box.
		 */
		public ContentTable getFirstTable() throws ActionOutOfSyncException
		{
			checkIfStillValid();

			String xPath = ".//form//table[contains( normalize-space(@id), ':dataTable')]";
			// [@id='resourceSummaryForm:dataTable'] - is this reliable?
			HtmlTable tableElement = (HtmlTable) element.getFirstByXPath(xPath);
			return new ContentTable(tableElement);
		}


		/**
		 * Returns table backed by the given table element.
		 */
		public ContentTable getTable(HtmlTable table)
		{
			return new ContentTable(table);
		}


		/**
		 * Creates a data table wrapper for first found element with one of these IDs:
		 * ID_CATEGORY_DATA_TABLE, ID_RESOURCE_DATA_TABLE
		 *
		 * @see ContentTable#ContentTable() 
		 */
		public ContentTable getDefaultTable() throws HtmlElementNotFoundException{

			// Find the data table - try some known IDs.
			String[] elemIDs = { ContentTable.ID_CATEGORY_DATA_TABLE,
			                     ContentTable.ID_RESOURCE_DATA_TABLE };
			Element elem = null;
			for( String elemID : elemIDs ){
				elem = client.getElement(elemID);
				if( null != elem )
					break;
			}
			if( null == elem )
				throw new HtmlElementNotFoundException(
								"Can't find the content table element, searched IDs: "
								+ StringUtils.join(elemIDs, ", "), currentTest );

			return new ContentTable( (HtmlTable)elem );

		}

		/**
		 * Returns the operation history table from the Control tab.
		 */
		public OperationsHistoryTable getOperationsHistoryTable() throws HtmlElementNotFoundException {

			// Find the ops history table.
			//String elemID = "operationHistoryForm:dataTable";
			String elemID = "historiesForm:dataTable";
			Element elem = client.getElement(elemID);
			if( null == elem ){
				throw new HtmlElementNotFoundException(
								"Can't find the ops history table element, searched ID: " + elemID, currentTest);
			}
			return new OperationsHistoryTable( (HtmlTable)elem );

		}


		/**
		 * Returns the operation results table from the Control tab.
		 */
		public ContentTable getOperationResultsTable() throws HtmlElementNotFoundException {
			String elemID = "operationResults";
			HtmlElement elem = (HtmlElement) client.getElement(elemID);
			if( null == elem ){
				throw new HtmlElementNotFoundException(
								"Can't find the op results table element, searched ID: " + elemID, currentTest);
			}
			HtmlTable tableElm = elem.getFirstByXPath(".//table[@class='properties-table']");
			return new ContentTable( (HtmlTable)tableElm );
		}



		/**
		 * Finds first button with given label inside this tab box.
		 */
		public ClickableElement getButtonByLabel( String label )
						throws HtmlElementNotFoundException, ActionOutOfSyncException
		{
			checkIfStillValid();
			ClickableElement button = this.element.getFirstByXPath(".//input[normalize-space(@value)='"+label+"']");
			if( null == button )
				throw new HtmlElementNotFoundException("Button labelled '"+label+"' not found.", currentTest);

			if( ! (button instanceof HtmlButton) || ! (button instanceof HtmlButtonInput) || ! (button instanceof HtmlSubmitInput) )
				throw new HtmlElementNotFoundException("Input with value '"+label+"' is not a button input nor submit input.", currentTest);

			return button;
		}
		

		/**
		 * Finds first button with given label inside this tab box.
		 */
		public HtmlButtonInput getButtonInputByLabel( String label )
						throws HtmlElementNotFoundException, ActionOutOfSyncException
		{
			checkIfStillValid();
			HtmlInput button = this.element.getFirstByXPath(".//input[normalize-space(@value)='"+label+"']");
			if( null == button )
				throw new HtmlElementNotFoundException("Button labelled '"+label+"' not found.", currentTest);
			
			if( !(button instanceof HtmlButtonInput) )
				throw new HtmlElementNotFoundException("Input with value '"+label+"' is not a button input nor submit input.", currentTest);

			return (HtmlButtonInput) button;
		}

		
		/**
		 * Finds first button with given label inside this tab box.
		 */
		public HtmlSubmitInput getSubmitButtonByLabel( String label )
						throws HtmlElementNotFoundException, ActionOutOfSyncException
		{
			checkIfStillValid();
			HtmlSubmitInput button = this.element.getFirstByXPath(".//input[normalize-space(@value)='"+label+"']");
			if( null == button )
				throw new HtmlElementNotFoundException("Submit button labelled '"+label+"' not found.", currentTest);
			return button;
		}

		/**
		 * Returns content info table based on given table element.
		 */
		public ContentInfoTable getContentInfoTable(HtmlTable tableElement) {
			return new ContentInfoTable(tableElement);
		}


		/**
		 *  Tries to find a data table row with given deployable, using pagination if needed.
		 * @returns  The row wrapper if found, or null.
		 *
		 * TODO: Get to the first page at the beginning.
		 */
		public ContentTableRow findLinkRowInDataTableUsingPagination(String label)
						throws HtmlElementNotFoundException, IOException
		{

			ContentTableRow row = null;

			if( null == getTabMenu().getTabContentBox().getPagination().findPageContols() ){
				log.debug("No page controls present - that probably means that only single page is listed.");
			}
			else{
				log.debug("Number of pages: "+getTabMenu().getTabContentBox().getPagination().getPageCount());
			}


			int currentPage = 1;

			// For all pages...
			do {
				HtmlTable tableElm = (HtmlTable) client.getElement("resourceSummaryForm:dataTable");
				if( null == tableElm ){
					throw new HtmlElementNotFoundException(
									"Data table not found (looking for ID 'resourceSummaryForm:dataTable'", currentTest);
				}

				ContentTable table = new ContentTable(tableElm);
				row = table.findFirstRowContainingText(label);

				// Did we find?
				if( null != row )
					break;
				else
					log.debug("Row with '"+label+"' not present on page "+currentPage+".");

			} while (
					// Infinite loop prevention.
					currentPage++ < 20
					// Pagination is not rendered if not needed.
					&& null != getTabMenu().getTabContentBox().getPagination().findPageContols()
					// goNext() returns true if there's a next page to go.
					&& getTabMenu().getTabContentBox().getPagination().goNext()
			);

			return row;
		}



		/**
		 * Returns the unformatted table (eg. General Properties and Traits).
		 */
		public ContentInfoTable getTableUnformatted(String tableHeader) throws EmbJoprTestException {
			ContentTable table = getTabMenu().getTabContentBox().getTableUnderHeader(tableHeader);
			ContentInfoTable traitsTable = getTabMenu().getTabContentBox().getContentInfoTable( table.getElement() );
			return traitsTable;
		}



		/** Returns first file upload input. */
		public HtmlFileInput getFirstFileInput(){
			String xPath = ".//input[ contains( @type, 'file') and position() = 1 ]";
			return (HtmlFileInput) this.getElement().getFirstByXPath(xPath);
		}


	}// inner class TabContentBox






	/**
	 * Contains convenience methods for accessing tables in EmbJopr.
	 * Works with any HTML table.
	 */
	public class ContentTable {

		public static final String ID_CATEGORY_DATA_TABLE = "categorySummaryForm:dataTable";
		public static final String ID_RESOURCE_DATA_TABLE = "resourceSummaryForm:dataTable";

		/** HTML element of this table. */
		public HtmlTable getElement() {			return this.element;		}
		protected HtmlTable element;

		/** Name of the table. Usualy header above the table is used. */
		public String getTableName() {			return this.name;		}
		protected String name;


		/**
		 * Creates a data table wrapper for the given table element.
		 */
		public ContentTable( HtmlTable element ) { init(element, null); }

		/**
		 * Creates a data table wrapper for the given table element and stores it's name.
		 */
		public ContentTable( HtmlTable element, String tableName ) { init(element, tableName);	}

		private void init( HtmlTable element, String tableName ){
			if( null == element )
				throw new IllegalArgumentException("null given as element for ContentTable.");
			this.element = element;
			this.name = tableName;
		}


		/**
		 * Creates a data table wrapper for first found element with one of these IDs:
		 * ID_CATEGORY_DATA_TABLE, ID_RESOURCE_DATA_TABLE
		 *
		 * This method assumes there's only one "data table" per page.
		 * If not, it simply returns the first in the order of IDs searched.
		 * You can always specify the element using ContentTable( HtmlTable element ).
		 */
		/*
		public ContentTable() throws HtmlElementNotFoundException {

			// Find the data table - try some known IDs.
			String[] elemIDs = { ID_CATEGORY_DATA_TABLE, ID_RESOURCE_DATA_TABLE };
			Element elem = null;
			for( String elemID : elemIDs ){
				elem = client.getElement(elemID);
				if( null != elem )
					break;
			}
			if( null == elem )
				throw new HtmlElementNotFoundException(
								"Can't find the content table element, searched IDs: "
								+ StringUtils.join(elemIDs, ", ") );

			this.element = (HtmlTable)elem;
		}
		*/


		// Columns maps
		private List<String> colLabels = null;
		private Map<String, Integer> colIndexes = null;
		//private boolean analyzedButNotFound = false;


		
		/**
		 * Return the table cell given by the row and column.
		 */
		public String getTableCell(int rowIndex, int columnIndex) throws HtmlElementNotFoundException {
		    List<HtmlTableRow> rows = element.getRows();

		    HtmlTableRow tableRow = rows.get(rowIndex);
		    HtmlTableCell tableCell = tableRow.getCell(columnIndex);

		    return tableCell.asText();
		}



		/** Returns first link containing given text. */
		public HtmlAnchor getFirstLinkContaining( String label ){
			String xPath = ".//a[ contains( normalize-space(), '"+label+"') and position() = 1 ]";
			return (HtmlAnchor) this.getElement().getFirstByXPath(xPath);
		}



		/**
		 * Returns the first row that contains given text, or throws HtmlElementNotFoundException.
		 * @param text
		 * @return
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 *	       when no row contains specified text.
		 */
		public ContentTableRow getFirstRowContainingText( String text )
						throws HtmlElementNotFoundException
		{
			if( 0 == element.getRowCount() )
				throw new HtmlElementNotFoundException("Table has no rows.", currentTest);

			// TODO: Escape the single quotes. By doubling?
			// http://books.google.com/books?id=jzqFMlM0gb0C&pg=PA308&lpg=PA308&dq=xquery+escape+quote&source=bl&ots=DIKQ92AhHh&sig=A7adGlif6jfYKtJXGc4eZbXYeCQ&hl=cs&ei=LYCcSYKLO5ir-gazwfjtBA&sa=X&oi=book_result&resnum=8&ct=result
			//String xPath = ".//tr[contains(string(), '"+text+"')]";
			String xPath = ".//tr[.//*[contains(string(), '"+text+"')]]";
			HtmlTableRow elm = (HtmlTableRow) element.getFirstByXPath(xPath);
			if( null == elm )
				throw new HtmlElementNotFoundException(xPath, currentTest);
			return new ContentTableRow(elm, this);
		}

		/**  Same as getFirstRowContainingText(), only returns null if row is not found. */
		public ContentTableRow findFirstRowContainingText( String text )
						throws HtmlElementNotFoundException
		{
			if( 0 == element.getRowCount() )
				return null;

			String xPath = ".//tr[.//*[contains(string(), '"+text+"')]]";
			HtmlTableRow elm = (HtmlTableRow) element.getFirstByXPath(xPath);
			if( null == elm )
				return null;
			return new ContentTableRow(elm, this);
		}



		/**
		 * Returns wrapper of the table row which contains a link with given label,
		 * or null when the no such row is found.
		 */
		public ContentTableRow findFirstRowContainingLink( String linkLabel )
		{
			if( 0 == element.getRowCount() )
				return null;

			String xPath = ".//tr[.//a[normalize-space()='"+linkLabel+"']]";
			HtmlTableRow elm = (HtmlTableRow) element.getFirstByXPath(xPath);
			if( null == elm )
				return null;
			return new ContentTableRow(elm, this);
		}

		
		/**
		 * Returns a collection of wrapped rows of this table.
		 */
		public List<ContentTableRow> getRows(){
			if( 0 == element.getRowCount() )
				return Collections.EMPTY_LIST;

			/// debug
			// This was caused by EMBJOPR-228
			/*
			String xPath_ = "./tbody/tr";
			List<HtmlTableRow> trList_ = (List) element.getByXPath(xPath_);
			xPath_ = ".//tr";
			trList_ = (List) element.getByXPath(xPath_);

			// Other way:
			{
				List<HtmlTableRow> rowElems = element.getRows();
				List<ContentTableRow> rows = new ArrayList(rowElems.size());
				for (HtmlTableRow tr : rowElems) {
					rows.add( new ContentTableRow(tr, this));
				}
			}*/

			//String xPath = "./tbody/tr[not( contains( @class , 'categoryRow') )]";
			// TODO: Put back the line above once EMBJOPR-228 is fixed.
			String xPath = "./tbody/tr[not( contains( @class , 'categoryRow') )] | ./tbody/div/tr[not( contains( @class , 'categoryRow') )]";
			List<HtmlTableRow> trList = (List) element.getByXPath(xPath);
			List<ContentTableRow> rows = new ArrayList(trList.size());
			for (HtmlTableRow tr : trList) {
				rows.add( new ContentTableRow(tr, this));
			}
			return rows;
		}

		/**
		 * Returns wrapper of the table row which contains a link with given label.
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 *				 when no row with such label found.
		 */
		public ContentTableRow getFirstRowContainingLink( String linkLabel )
						throws HtmlElementNotFoundException
		{
			if( 0 == element.getRowCount() )
				throw new HtmlElementNotFoundException("Table has no rows.", currentTest);

			String xPath = ".//tr[.//a[normalize-space()='"+linkLabel+"']]";
			HtmlTableRow elm = (HtmlTableRow) element.getFirstByXPath(xPath);
			if( null == elm )
				throw new HtmlElementNotFoundException(
								"Can't find row containing link '"+linkLabel+"' using XPath: "+xPath, currentTest);
			return new ContentTableRow(elm, this);
		}

		/**
		 * Creates a list of columns headers and a label => col index map.
		 * Works for tables which have column headers,
		 * ie. they have TH elements in the first row.
		 */
		public void analyzeColumns() throws HtmlElementNotFoundException
		{

			/*if( this.analyzedButNotFound )
				throw new HtmlElementNotFoundException("Table has no column headers.");
			/**/ // Give it another chance, JavaScript could change it.

			// Get all TH from the first THEAD row that contains TH.
			//String xPath = "./thead/tr[th and position()=1]/th";
			String xPath = ".//tr[th and position()=1]/th";
			List<HtmlTableHeaderCell> colHeaders = (List<HtmlTableHeaderCell>) this.element.getByXPath(xPath);

			if( 0 == colHeaders.size() ){
				//this.analyzedButNotFound = true;
				throw new HtmlElementNotFoundException("Table has no column headers.", currentTest);
			}

			List<String> colLabels_ = new ArrayList(colHeaders.size());
			Map<String, Integer> colIndexes_ = new HashMap(colHeaders.size());

			//for( HtmlTableHeaderCell th : colHeaders ){
			for( int i = 0; i < colHeaders.size(); i++ ) {
				HtmlTableHeaderCell th = colHeaders.get(i);
				String sHeader = th.getTextContent();
				colLabels_.add( sHeader );
				colIndexes_.put( sHeader, i );
			}

			this.colLabels = colLabels_;
			this.colIndexes = colIndexes_;

		}

		/**
		 * Returns an index of the column with given name in the TH header,
		 * or throws HtmlElementNotFound in two cases:
		 *  1) Table does not have column headers
		 *  2) No header with such name was found.
		 *
		 * Works for tables which have column headers,
		 * ie. they have TH elements in the first row.
     *
		 * @param colName
		 * @return
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 */
		public int getColumnIndexByName( String colName ) throws HtmlElementNotFoundException {
			if( null == this.colIndexes )
				this.analyzeColumns();

			Integer index = this.colIndexes.get(colName);
			if( null == index )
				throw new HtmlElementNotFoundException("No column named '"+colName+"'.", currentTest);

			return index;
		}


		/**
		 * Creates a properties using values from this table.
		 *
		 * Works for tables which have column headers,
		 * ie. they have TH elements in the first row.
		 *
		 * @param keyColName    Name of column containing the key (e.g. "Name").
		 * @param valueColName  Name of column containing the value (e.g. "Value").
		 */
		public Properties getProperties( String keyColName, String valueColName ) throws HtmlElementNotFoundException
		{
			int keyColIndex = this.getColumnIndexByName(keyColName);
			int valColIndex = this.getColumnIndexByName(valueColName);

			log.debug("keyColIndex: "+keyColIndex+", valColIndex: "+valColIndex);
			
			Properties props = new Properties();

			for( ContentTableRow row : this.getRows() ){
				log.debug("Row ("+row.getElement().getCells().size()+" cells): "+row.getElement().getTextContent());
				String key   = row.getCell( keyColIndex ).getTextContent().trim();

				// Get the cell.
				HtmlTableCell cell = row.getCell(valColIndex);
				String value;
				// If it contains one input, get the value of the input.
				List<HtmlTextInput> inputs = (List<HtmlTextInput>) cell.getByXPath("./input[@type='text']");

				log.debug("Value cell has "+inputs.size()+" <input> elements.");
				if( inputs.size() == 1 ){
					value = ((HtmlTextInput)inputs.get(0)).asText(); //getValueAttribute();
				}
				// Otherwise, get the text content of the cell.
				else{
					value = cell.getTextContent().trim();
				}

				props.setProperty(key, value);
			}

			return props;
		}

		/**
		 *  Calls getProperties( key, value ) with default column names (Name and Value).
		 */
		public Properties getProperties() throws HtmlElementNotFoundException {
			return this.getProperties(EmbJoprTestConstants.COL_PROP_NAME, EmbJoprTestConstants.COL_PROP_VALUE);
		}



		/**
		 * Checks that name/value pairs in this table mach the given values.
		 * @throws EmbJoprTestException  if any of given values does not match with the value in this table.
		 */
		public void checkValuesEqual( Properties propsExpected ) throws EmbJoprTestException {

			// Get this tables properties. Different table types can override this.
			Properties propsActual = this.getProperties();

			// List the actual properties.
			for( Object key : propsActual.keySet() ){
				String propName = (String)key;
				log.info("Actual property: '"+propName+"': '"+propsActual.getProperty(propName)+"'");
			}

			// List of non-equal properties.
			List<String> failures = new ArrayList<String>();

			// Check all the properties.
			for( Object key : propsExpected.keySet() ){
				String propName = (String)key;
				log.info("Checking '"+propName+"'...");
				String exp = propsExpected.getProperty(propName);
				String act = propsActual.getProperty(propName);

				exp = EmbjoprTestCase.normalizeIfNumber(exp);
				act = EmbjoprTestCase.normalizeIfNumber(act);

				if( !StringUtils.equals( exp, act ) ){
					// If this table has a name, use it in the exception message.
					String msgFormat = (null == this.getTableName() ? "%2$s: expected '%3$s', actual '%4$s'." : "%s - %s: expected '%s', actual '%s'.");
					failures.add( String.format(msgFormat, this.getTableName(), propName, exp, act) );
				}

				// If some did not match, throw an exception with their list.
				if( failures.size() != 0 ){
					throw new EmbJoprTestException( String.format("Non-matching properties: (%d of %d expected):\n", failures.size(), propsExpected.size()) + StringUtils.join(failures, "\n") );
				}
			}
		}

		/**
		 * Checks that name/value pairs in this table mach the given values.
		 * @throws EmbJoprTestException  if any of given values does not match with the value in this table.
		 */
		public void checkValues( Properties propsExpected, ValuesValidator validator ) throws EmbJoprTestException {

			// Get this tables properties. Different table types can override this.
			Properties propsActual = this.getProperties();

			selfEjtt.checkValues(propsExpected, propsActual, validator, this.getTableName() );

			/*
			// List the actual properties.
			for( Object key : propsActual.keySet() ){
				String propName = (String)key;
				log.info("Actual property: '"+propName+"': '"+propsActual.getProperty(propName)+"'");
			}

			// List of non-equal properties.
			List<String> failures = new ArrayList<String>();

			// Check all the properties.
			for( Object key : propsExpected.keySet() ){
				String propName = (String)key;
				log.info("Checking '"+propName+"'...");
				String exp = propsExpected.getProperty(propName);
				String act = propsActual.getProperty(propName);

				exp = EmbjoprTestCase.normalizeIfNumber(exp);
				act = EmbjoprTestCase.normalizeIfNumber(act);

				String validationError = validator.validate( exp, act );
				if( null != validationError ){
					// If this table has a name, use it in the exception message.
					String msgFormat = (null == this.getTableName() ? "%2$s: " : "%s - %s: ");
					failures.add( String.format(msgFormat, this.getTableName(), propName) + validationError );
				}

				// If some did not match, throw an exception with their list.
				if( failures.size() != 0 ){
					throw new EmbJoprTestException( String.format("Non-matching properties: (%d of %d expected):\n", failures.size(), propsExpected.size()) + StringUtils.join(failures, "\n") );
				}
			}
			*/
		}


	}// inner class ContentTable






	/**
	 * Creates a data table wrapper for first found element with one of these IDs:
	 * ID_CATEGORY_DATA_TABLE, ID_RESOURCE_DATA_TABLE
	 *
	 * This is a shortcut for ejtt.getTabMenu().getTabContentBox().getDefaultTable(),
	 * which we can do because it searches the element using ID.
	 */
	public ContentTable getDefaultContentTable() throws HtmlElementNotFoundException{
		return this.getTabMenu().getTabContentBox().getDefaultTable();
	}






	/**
	 * Provides extra method that parses text-like info table (with one column) -
	 * usualy Traits tables have this structure.
	 */
	public class ContentInfoTable extends ContentTable {

		public ContentInfoTable(HtmlTable element) {
			super(element);
		}

		/**
		 * Parses the content of the table.
		 * Works with text-like info table (with one column) -
		 * usualy Traits tables have this structure.
		 *
		 * @returns Set of properties extracted from rows
		 *          whose plain text has the 'Name: Value' format.
		 */
		public Properties getProperties()
		{
			Properties props = new Properties();

			// The template has label in span/strong and the value as text in td.
			String xPath = ".//tr/td[span/strong | strong | b]";
			//String xPath = ".//tr/td[span/strong] | .//tr/td[.//strong]";
			List<HtmlTableCell> cells  = (List<HtmlTableCell>) this.getElement().getByXPath(xPath);
			for( HtmlTableCell cell : cells ){
				String cellText = cell.getTextContent();
				String[] parts = cellText.split(":", 2);
				if( parts.length != 2 ){
					log.warn("Cell text not in format 'Name: Value': "+cellText);
					continue;
				}
				props.setProperty(parts[0].trim(), parts[1].trim());
			}
			return props;
		}


	}// inner class ContentInfoTable

	/**
	 * This is a shortcut for ejtt.getTabMenu().getTabContentBox().getContentInfoTable(tableElement).
	 */
	public ContentInfoTable getContentInfoTable(HtmlTable tableElement) {
		return new ContentInfoTable(tableElement);
	}






	
	/**
	 * Table which shows the result of an operation.
	 */
	public class OperationsHistoryTable extends ContentTable {

		public OperationsHistoryTable(HtmlTable element) throws HtmlElementNotFoundException {
			super(element);
			analyzeColumns();
		}

		/**
		 * Returns the status row of the last operation.
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 *         when the table has no rows or XPath fails to find the status row.
		 */
		public HtmlTableRow getLastOperationRow() throws HtmlElementNotFoundException{
			if( 0 == this.getRows().size() )
				throw new HtmlElementNotFoundException(
								"Operation table has no rows (no operations are listed).");

			String xPath = "tbody/tr[contains(@class, 'rich-table-firstrow')]";
			//String xPath = "tbody/tr[contains(@class, 'operationStatusColumn')]";

			HtmlTableRow tr = this.element.getFirstByXPath(xPath);
			if( null == tr ){
				throw new HtmlElementNotFoundException(
								"Operation status row not recognized; used XPath: "+xPath, currentTest);
			}
			return tr;
		}

		
		public OperationStatusType getLastOperationStatus() throws HtmlElementNotFoundException
		{
			// TODO:  Ugly - we should get the cell, not whole row.
			HtmlTableRow tr = getLastOperationRow();
			//OperationStatusType status = OperationStatusType.fromStatusText( tr.getTextContent() );

			int statusColIndex = this.getColumnIndexByName("Status");
			String statusText = this.getRows().get(0).getCell(statusColIndex).getTextContent().trim();
			OperationStatusType status = OperationStatusType.fromStatusText( statusText );

			if( null == status )
				throw new HtmlElementNotFoundException(
								"Operation status not recognized. Row text: "+tr.getTextContent(), currentTest);
			return status;
		}

		public boolean wasLastOperationSuccesful() throws HtmlElementNotFoundException
		{
			HtmlTableRow tr = getLastOperationRow();
			// Kind of simple hack.
			return tr.getTextContent().contains("Success");
		}

	}
	
	/**
	 * Generalization of status result types.
	 */
	public enum OperationStatusType {
		SUCCESS("Success"), FAILURE("Failed"), IN_PROGRESS("In progress");

		private String statusText;
		public String getStatusText() {			return statusText;		}

		OperationStatusType( String statusText ){
			this.statusText = statusText;
		}

		public static OperationStatusType fromStatusText( String text ){
			for( OperationStatusType type : OperationStatusType.values() ){
				if( text.toLowerCase().contains( type.getStatusText().toLowerCase() )  )
					return type;
			}
			//log.debug("Unrecognized operation status: "+text);
			return null;
		}
	}




	/**
	 * Row of a content table.
	 * Contains convenience methods for accessing content table rows in EmbJopr.
	 */
	public class ContentTableRow extends HtmlElementBackedBase {

		private ContentTable containingTable;

		private ContentTableRow(HtmlTableRow elm, ContentTable containingTable) {
			super(elm);
			this.containingTable = containingTable;
		}


		/**
		 *  Returns true if this row contains at least one TH element and no TD element.
		 */
		public boolean isHeader(){
			/* Does not work - returns:   headers.getLength(): 8;  cells.getLength(): 196;
			NodeList headers = this.getElement().getElementsByTagName("th");
			NodeList cells = this.getElement().getElementsByTagName("td");
			log.debug("headers.getLength(): "+headers.getLength()+";  cells.getLength(): "+cells.getLength()+";");
			return headers.getLength() != 0  &&  cells.getLength() == 0;
			/**/

			List<?> headers = this.getElement().getByXPath("./th");
			List<?> cells = this.getElement().getByXPath("./td");
			//log.debug("isHeader(): headers.size(): "+headers.size()+";  cells.size(): "+cells.size()+";");
			return headers.size() != 0  &&  cells.size() == 0;
		}


		/**
		 * Finds a <button> or <input type="button"> with the given label.
		 * @param label The label of the button. Compared to a trimmed label text of the button.
		 * @returns the button element - either HtmlButtonInput or HtmlButton.
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 */
		public ClickableElement getButtonByLabel( String label ) throws HtmlElementNotFoundException
		{
			String xPath = ".//input[@type='button' and normalize-space(@value) = '"+label+"']" +
							" | .//button[normalize-space() = '"+label+"']";
			HtmlElement elm = this.element.getFirstByXPath(xPath);
			if( null == elm )
				throw new HtmlElementNotFoundException("Can't find the button using xPath: "+xPath);
			if( !(elm instanceof HtmlButton ) && !(elm instanceof HtmlButtonInput) )
				throw new HtmlElementNotFoundException("Element is not a button, but: "+elm.getClass().getName());

			return (ClickableElement)elm;
		}


		/**
		 * Gives the (first) link from the cell of this row, from the column with given header.
		 * @param colName
		 * @return
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 *				 in two cases:
		 *             1) Table has no column of given name
		 *             2) There's no link in that cell.
		 */
		public HtmlAnchor getFirstLinkFromColumn(String colName) throws HtmlElementNotFoundException
		{
			HtmlTableCell cell = this.getCellByColumnName(colName);

			String xPath = ".//a";
			HtmlAnchor link = (HtmlAnchor) this.element.getFirstByXPath(xPath);

			if( null == link )
				throw new HtmlElementNotFoundException("No link found in column '"+colName+"'");

			return link;
		}


		/**
		 * Returns the link with given label.
		 * @param string
		 */
		public HtmlAnchor getLinkByLabel(String linkLabel) throws HtmlElementNotFoundException {
			String xPath = ".//a[normalize-space()='"+linkLabel+"']";
			HtmlAnchor link = (HtmlAnchor) this.element.getFirstByXPath(xPath);
			if( null == link ){
				StringBuilder sb = new StringBuilder();
				List<HtmlElement> linksFound = this.element.getHtmlElementsByTagName("a");
				for( HtmlElement linkFound : linksFound ){
					sb.append("'").append( ((HtmlAnchor)linkFound).getTextContent() ).append("', ");
				}
				String availLinks = StringUtils.removeEnd( sb.toString(), ", " );

				throw new HtmlElementNotFoundException("Can't find link using '"+xPath+"', available: "+availLinks);
			}
			return link;
		}

		/**
		 * Returns the cell of this table from given index,
		 * or throws IndexOutOfBoundsException.
		 *
		 * TODO: May throw IndexOutOfBoundsException - leave unchecked?
		 *
     * @param index the 0-based index
     * @return the cell at the given index
     * @throws IndexOutOfBoundsException if there is no cell at the given index
		 */
		public HtmlTableCell getCell( int index ){
			HtmlTableCell cell = this.element.getCell(index);
			return cell;
		}

		/**
		 * Returns  this row's cell element from the column with specified name.
		 * @param colName  Name of the column.
		 * @throws org.jboss.jopr.jsfunit.exceptions.HtmlElementNotFoundException
		 *         if no such column exists in the table.
		 */
		public HtmlTableCell getCellByColumnName( String colName ) throws HtmlElementNotFoundException
		{
			int index = this.containingTable.getColumnIndexByName(colName);
			return getCell(index);
		}

		/**  Shortcut - returns text content of this row's cell from given column. */
		public String getCellTextByColumnName( String colName ) throws HtmlElementNotFoundException {
			return this.getCellByColumnName(colName).getTextContent();
		}


	}// inner class ContentTableRow




	/**
	 *  Pagination control
	 * @returns the pagination DIV, or null if the pagination is not displayed
	 *    (DIV can exist, but contains no First, Prev, Next, Last etc.
	 */
	public class ContentBoxPagination {

		//private HtmlElement element;
		//public HtmlElement getElement() {			return element;		}

		private static final String ID_PAGE_CONTROLS = "categorySummaryForm:dataTableScroller";
		private static final String ID_PAGE_SIZE_SELECT = "categorySummaryForm:currentPageSize";
		private static final String ID_PAGINATION_TOTAL_ITEMS = "paginationTotalItems";

		/** Returns the HTML element of the bar with "First | Prev | 1 2 | Next Last" controls. */
		public HtmlDivision findPageContols() throws HtmlElementNotFoundException {
			//return (HtmlDivision) client.getElement(ID_PAGE_CONTROLS);
			HtmlDivision div = (HtmlDivision) getTabMenu().getTabContentBox().getElement()
							.getFirstByXPath(".//*[contains(@id, 'SummaryForm:dataTableScroller')]");
			if( null == div )
				return null;

			// When no pagination is needed, #dataTableScroller div contains only a script element.
			// So if their number is the same, return null, which means
			List allElems    = div.getByXPath("*");
			List scriptElems = div.getByXPath("script");
			if( allElems.size() == scriptElems.size() )
				return null;

			// Else if it contains something besides script, return.
			return div;
		}

		/** Calls #findPageControls(), and if that returns null (not found), throws an exception. */
		public HtmlDivision getPageContols() throws HtmlElementNotFoundException {
			HtmlDivision div = findPageContols();
			if( null == div)
				throw new HtmlElementNotFoundException("Page controls not found.", currentTest);
			return div;
		}

		/** Returns the HTML Select element of page size selection. */
		public HtmlSelect getPageSizeSelect() throws HtmlElementNotFoundException {
			String xPath = ".//select[contains(@id, 'SummaryForm:currentPageSize')]";
			HtmlSelect select = (HtmlSelect) getTabMenu().getTabContentBox().getElement().getFirstByXPath(xPath);
			if( null == select )
				throw new HtmlElementNotFoundException("Can't find page size <select> using: "+xPath, currentTest);
			return select;
		}

		/** Returns total items count, taken from the "Total: N" element. */
		public int getTotalItemsCount() throws ActionNotAvailableException, HtmlElementNotFoundException
		{
			Element e = client.getElement(ID_PAGINATION_TOTAL_ITEMS);
			String textContent_ = e.getTextContent();
			log.debug("Element #"+ID_PAGINATION_TOTAL_ITEMS+" content: "+textContent_);

			String xPath = ".//select[contains(@id, 'SummaryForm:paginationTotalItems')]";
			HtmlDivision totalDiv = (HtmlDivision) getTabMenu().getTabContentBox().getElement().getFirstByXPath(xPath);
			if( null == totalDiv )
				throw new HtmlElementNotFoundException("Can't find items count div (Total: ...) using: "+xPath, currentTest);
			String textContent = totalDiv.getTextContent();
			log.debug("Element #...SummaryForm:paginationTotalItems content: "+textContent);

			String[] parts = textContent.split(":");
			if( parts.length < 2 )
				throw new ActionNotAvailableException("Total pagination items count expected after 'Total:' or similar.", currentTest);

			String countStr = parts[1].trim();
			try{
				return NumberUtils.createInteger( countStr );
			}catch(NumberFormatException ex ){
				throw new ActionNotAvailableException("Can't parse pagination items count from '"+countStr+"'", ex);
			}
		}

		protected HtmlSpan getGoSpan( String spanLabel ) throws HtmlElementNotFoundException {
			String xPath = ".//span[normalize-space() = '"+spanLabel+"']";
			HtmlSpan span = getPageContols().getFirstByXPath(xPath);
			if( null == span )
				throw new HtmlElementNotFoundException(
								"Span with '"+spanLabel+"' not found using XPath '"+xPath+"'");
			return span;
		}

		protected HtmlTableCell getGoPage( int pageNumber ) throws HtmlElementNotFoundException{
			String xPath = ".//td[normalize-space() = '"+pageNumber+"']";
			HtmlElement td = getPageContols().getFirstByXPath(xPath);
			if( null == td )
				throw new HtmlElementNotFoundException(
								"Page number "+pageNumber+" not found. Try 1 - "+this.getPageCount());
			return (HtmlTableCell) td;
		}


		public boolean goFirst() throws IOException, HtmlElementNotFoundException{
			// TODO: Compare old and new page object reference?
			HtmlSpan goSpan = getGoSpan("First");
			goSpan.click();
			return !"".equals( ((HtmlTableCell)goSpan.getParentNode()).getOnClickAttribute() );
		}

		public boolean goPrev() throws IOException, HtmlElementNotFoundException{
			HtmlSpan goSpan = getGoSpan("Prev");
			goSpan.click();
			return !"".equals( ((HtmlTableCell)goSpan.getParentNode()).getOnClickAttribute() );
		}

		/** Clicks the "Next" link of the pagination controls.
		 * @returns true if the "Next" link will not bring us to a new page (has no onClick handler).
		 */
		public boolean goNext() throws IOException, HtmlElementNotFoundException{
			//Page originalPage = client.getContentPage();
			HtmlSpan goSpan = getGoSpan("Next");
			goSpan.click();
			//originalPage.equals( ejtt.getClient().getContentPage() )
			return !"".equals( ((HtmlTableCell)goSpan.getParentNode()).getOnClickAttribute() );
		}

		public boolean goLast() throws IOException, HtmlElementNotFoundException{
			HtmlSpan goSpan = getGoSpan("Last");
			goSpan.click();
			return !"".equals( ((HtmlTableCell)goSpan.getParentNode()).getOnClickAttribute() );
		}

		public void goPage( int pageNumber ) throws IOException, HtmlElementNotFoundException{
			getGoPage(pageNumber).click();
		}


		public boolean isPageActive( int pageNumber ) throws IOException, HtmlElementNotFoundException{
			return !getGoPage(pageNumber).getStyleAttribute().contains("-inact");
		}

		public int getPageCount() throws HtmlElementNotFoundException {
			String xPath = ".//td[ count(./*) = 0  and  normalize-space() > 0 ]";
			List<?> elements = getPageContols().getByXPath(xPath);
			return elements.size();
		}

		public int getItemsPerPage() throws HtmlElementNotFoundException, ActionNotAvailableException {
			String selectText = getPageSizeSelect().asText();
			try {
				return NumberUtils.createInteger( selectText );
			}
			catch( NumberFormatException ex ){
				throw new ActionNotAvailableException("Can't parse 'items per page' from '"+selectText+"'.", ex);
			}
		}
		public void setItemsPerPage( int num ) throws HtmlElementNotFoundException {
			String optionValue = ""+num;
			HtmlSelect pageSizeSelect = getPageSizeSelect();

			try{
				pageSizeSelect.getOptionByValue(optionValue);
			}
			catch( ElementNotFoundException ex ){
				// Print out available options.
				StringBuilder sb = new StringBuilder(pageSizeSelect.getOptionSize());
				for( HtmlOption opt : pageSizeSelect.getOptions() ){
					sb.append(" ").append(opt.getValueAttribute());
				}
				String possibleOptions = sb.toString().trim().replace(" ", ", ");

				throw new HtmlElementNotFoundException("Specified option is invalid. Try one of: "+possibleOptions, ex);
			}

			pageSizeSelect.setSelectedAttribute(optionValue, true);
		}

	}// ContentBoxPagination



	/**
	 * Some of the tables are outside tab menu content box.
	 */
	public ContentTable getTable(HtmlTable table)
	{
		return new ContentTable(table);
	}




	/**
	 * Finds a <a> tag inside a form that has a particular label.
	 *
	 * TODO: Create Form wrapper.
	 */
	public HtmlAnchor getLinkInsideForm(String formId, String linkLabel) throws HtmlElementNotFoundException
	{
		HtmlForm form = (HtmlForm)client.getElement(formId);
		if( null == form ){
			/// Dump the page
			String normID = StringUtils.replaceChars(formId, "\\/.: ", "-----");
			String dumpFileName = "target/formNotFound-"+ formId +".html";
			log.debug("Dumping page to file: "+dumpFileName);
			try { DebugUtils.writeFile(dumpFileName, client.getPageAsText()); }
			catch (Exception ex) { log.warn("getLinkInsideForm(): "+ ex.toString()); }

			throw new HtmlElementNotFoundException("Form element of given ID not found: "+formId, this.getCurrentTest());
		}

		String xPath = ".//a[contains(normalize-space(), '"+linkLabel+"')]";
		HtmlAnchor link = form.getFirstByXPath(xPath);
		if( null == link )
			throw new HtmlElementNotFoundException("Can't find row containing link '"+linkLabel+"' using XPath: "+xPath);
		return link;
	}




	/** Calls Thread.sleep(ms), ignores the InterruptedException. */
	@SuppressWarnings("empty-statement")
	public void sleep( int ms ) {
		try {
			Thread.sleep(ms);
		} catch (InterruptedException ex) {
			log.warn("EJTT.sleep() interrupted.");
			// We don't care more, that's the purpose of this method.
		}
	}


	/**
	 *  Finds element by ID and returns retyped as HTML button.
	 */
	public HtmlButtonInput getButtonByID( String id ) throws HtmlElementNotFoundException {
		HtmlButtonInput button = (HtmlButtonInput) client.getElement( id );
		if( null == button )
			throw new HtmlElementNotFoundException("Can't find button with ID: "+button);
		return button;
	}

	/**
	 *  Finds element by ID and returns retyped as HTML button.
	 */
	public ClickableElement getClickableByID( String id ) throws HtmlElementNotFoundException {
		ClickableElement clickable = (ClickableElement) client.getElement( id );
		if( null == clickable )
			throw new HtmlElementNotFoundException("Can't find element with ID: "+id);
		return clickable;
	}


	/** Converts java.util.Properties to java.util.Map. */
	public static Map<String,String> propertiesAsMap( Properties props )
	{
		Map<String,String> map = new HashMap<String,String>(props.size());

		// Get Properties names.
		//Set<String> propnames = props.stringPropertyNames();// JDK 1.6
		Set<String> propNames = stringPropertyNames(props);

		// Put all properties into the map.
		for( String propName : propNames ) {
			map.put(propName, props.getProperty(propName));
		}
		return map;
	}

	/**
	 * Converts properties to Set of Strings.
	 * Substitutes JDK 1.6's Properties::stringPropertyNames().
	 */
	public static Set<String> stringPropertyNames( Properties props )
	{
		Set<String> propNames = new HashSet<String>();
		Enumeration<?> propNamesIter = props.propertyNames();
		while( propNamesIter.hasMoreElements() )
			propNames.add( (String)propNamesIter.nextElement());
		return propNames;
	}





	public void logServerMessage(){ logServerMessage(""); }

	public void logServerMessage( String warnPrefix ){
		if( server.getFacesMessages().hasNext() ){
			FacesMessage msg = server.getFacesMessages().next();
			if( msg.getSeverity() == FacesMessage.SEVERITY_INFO ){
				log.info( msg.getSummary() +"\n"+ msg.getDetail() );
			}else{
				try {
					this.dumpPage(this.currentTest.getName() + "-logServerMsg.html");
				} catch (IOException ex) {
					log.error("Can't dump page: "+ex);
				}
				log.error( warnPrefix + msg.getSummary() +"\n"+ msg.getDetail() );
			}
		}
	}





	public final Operations operations = new Operations();

	/**
	 * Operations
	 */
	public class Operations {

		/**
		 * Checks whether the status of the last operation on current page (must be on operations page)
		 * is "In Progress". If true, waits for it to change. If false,
		 *
		 * @param opName    Operation name - only for log purposes.
		 * @param interval  Millisecons between retries (page refreshes).
		 * @param retries   Number of retries.
		 * @throws org.jboss.jopr.jsfunit.exceptions.EmbJoprTestException
		 */
		public void waitActivelyForOperationToFinish( String opName, int interval, int retries ) throws EmbJoprTestException
		{
			try {
				new ActiveConditionChecker( new DescribedCondition("'"+opName+"' operation status changes from '"+OperationStatusType.IN_PROGRESS.getStatusText()+"'.") {
					int calls = 0;
					public boolean isTrue() throws Exception {
						if( ++calls > 1 ) ((HtmlPage)client.getContentPage()).refresh();
						OperationStatusType status = selfEjtt.getTabMenu().getTabContentBox().getOperationsHistoryTable().getLastOperationStatus();
						return ! OperationStatusType.IN_PROGRESS.equals( status );
					}
				}).dumpPageOnTimeout(selfEjtt.currentTest).throwOnTimeout().waitWithTimeout(1500, 5);
			}
			catch(Exception ex){
				throw new EmbJoprTestException(ex.getMessage(), ex.getCause());
			}

		}


		/**
		 * Performs the operation, waits until it finishes, and checks whether the result was as expected.
		 * @param name
		 * @param expectedResult
		 */
		public Properties performOperation( String operationName ) throws EmbJoprTestException, IOException {
			getEjtt().tabMenu.getTabContentBox().getButtonInputByLabel(operationName).click();
			getEjtt().operations.waitActivelyForOperationToFinish(operationName, 500, 5);
			if( ! getEjtt().tabMenu.getTabContentBox().getOperationsHistoryTable().wasLastOperationSuccesful() ){
				OperationStatusType status = getEjtt().tabMenu.getTabContentBox().getOperationsHistoryTable().getLastOperationStatus();
				throw new EmbJoprTestException("Operation was not succesfull: "+operationName+" Status: "+status.getStatusText());
			}
			return getEjtt().operations.getLastOperationResults();
		}

		/**
		 * Performs the operation and checks whether the Output result property matches given expected output.
		 */
		public void testOperation( String operationName, String expectedOutput ) throws EmbJoprTestException, IOException {
			Properties results = performOperation(operationName);
			String output = results.getProperty(EmbJoprTestConstants.OPERATION_OUTPUT_PROPERTY);
			if( expectedOutput != null  &&  ! StringUtils.equals(expectedOutput, output) ){
				throw new EmbJoprTestException( String.format("Operation %s: Expected: '%s', actual: '%s'", operationName, expectedOutput, output) );
			}
		}


		/**
		 * Returns a Properties with the result of last performed operation.
		 * Assumes that we're on the Control tab.
		 */
		public Properties getLastOperationResults() throws HtmlElementNotFoundException{
			Properties props = new Properties();

			TabContentBox tabContentBox = getEjtt().tabMenu.getTabContentBox();

			// Operation has no result (table not displayed).
			if( tabContentBox.getElement().getTextContent().contains("The operation did not return any results."))
				return props;
			
			ContentTable resultsTable = tabContentBox.getOperationResultsTable();
			props = resultsTable.getProperties();

			return props;
		}

	}// class Operations


	



	public final Deployment deployment = new Deployment();

	/**
	 * Deployment
	 */
	public class Deployment {


		/**
		 * Makes an MBean name and checks via JMX whether it is deployed.
		 */
		public boolean isDeployedAccordingToJMX( AppConstants.DeployableTypes type, String deployableName )
						throws EmbJoprTestException, IOException
		{
			// Get the MBean name.
			String mBeanName = type.formatJmxMBeanName(deployableName);


			// Check whether the mBean is deployed.
			try {
				return isMBeanStateDeployedByQuery(mBeanName);
			} catch (MalformedObjectNameException ex) {
				throw new EmbJoprTestException("Malformed name: "+mBeanName, ex);
			} catch (JMException ex) {
				throw new EmbJoprTestException(ex);
			} catch (IOException ex) {
				throw ex;
			}
		}

		
		/**
		 * Queries JBoss via JMX for given query and checks whether the
		 * StateString property of the first result is "Started".
		 *
		 * @param mBeanName
		 * @return true if the StateString property of the first MBean found is "Started".
		 * @throws javax.management.JMException
		 * @throws java.io.IOException
		 */
		protected boolean isMBeanStateDeployedByQuery(String mBeanName)  throws JMException, IOException, MalformedObjectNameException
		{
			log.info("Querying JMX: "+mBeanName);
			ObjectName objName =	new ObjectName(mBeanName);
			MBeanServer jmxServer = MBeanServerLocator.locateJBoss();
			Set mBeans = jmxServer.queryNames(objName, null);
			if (mBeans.size() != 1) {
				log.info(mBeanName+" not found.");
				return false;
			}
			ObjectName deploymentMBean = (ObjectName) mBeans.iterator().next();
			// Returns org.jboss.deployers.spi.DeploymentState.
			Object state = jmxServer.getAttribute(deploymentMBean, "State");
			log.info("State is '"+state.toString()+"'.");
			return
						 "DEPLOYED".equals(state.toString()) // AS 4
						 || "3".equals(state.toString())     // AS 5; StateString = "Deployed"
						 || "8".equals(state.toString())     // EJB: "Registered"
						 ;
		}




		/**
		 * Deploys the deployable using EmbJopr's web UI.
		 *
		 * @param type  Type of the deployable - EAR, SAR, WAR, ...
		 */
		public void deployViaEmbJopr( AppConstants.DeployableTypes type, String filePath )
					throws IOException, HtmlElementNotFoundException
		{
			if( !(new File(filePath)).exists())
				throw new FileNotFoundException(filePath);

			log.info("Deploying: "+filePath);

			// Navigate to appropriate page
			getNavTree().getNodeLink(type.getNavTreeLabel()).click();

			// click on the "Add new resource" button
			client.click("actionHeaderForm:addNewContent");  // 404 if setThrowExceptionOnFailingStatusCode(true) above

			// Upload the file
			HtmlFileInput fileInput = (HtmlFileInput)client.getElement("createContentForm:file");
			fileInput.setContentType(type.getMimeType());
			fileInput.setValueAttribute( filePath );
			client.click("createContentForm:addButton");
			sleep( 500 );

		}// deployViaEmbJopr()




		public void deployViaEmbJoprRepeatedly( final AppConstants.DeployableTypes type, final String filePath ) throws IOException, HtmlElementNotFoundException, EmbJoprTestException {
			deployViaEmbJoprRepeatedly( type, filePath, 5000, 8 );
		}


		public void deployViaEmbJoprRepeatedly( final AppConstants.DeployableTypes type, final String filePath, int interval, int repeats )
					throws IOException, HtmlElementNotFoundException, EmbJoprTestException
		{

			if( !(new File(filePath)).exists())
				throw new FileNotFoundException(filePath);

			File appFile = new File(filePath);
			final String fileName = appFile.getName();

			try {

				// Try it 5x in 5 second interval.
				new ActiveConditionChecker( new DescribedCondition(type.name()+" '"+fileName+"' succesfuly deployed") {
					public boolean isTrue() throws Exception {
						deployViaEmbJopr( type, filePath );

						// Check whether deployment ended with success message.
						/* Now broken, EmbJopr reports an error even when deployed successfully:
						 * Failed to create Resource hellothere.war - cause:
								java.lang.RuntimeException:java.lang.NullPointerException
						 -> java.lang.NullPointerException:null
						 * See EMBJOPR-116
						 */
						/*
						if( server.getFacesMessages().hasNext() ){
							FacesMessage msg = server.getFacesMessages().next();
							if( msg.getSeverity() == FacesMessage.SEVERITY_INFO )
								return true;
							else
								ejtt.logServerMessage("Deployment error: ");
						}
						*/

						// Check whether the WAR is listed in EmbJopr.
						log.debug("Looking for listed "+type.name()+": "+fileName);
						boolean listed = deployment.isDeployedAccordingToEmbJopr(type, fileName, true);
						if( listed )
							return true;
						else
							return false;
					}
				}).dumpPageOnTimeout(currentTest).throwOnTimeout().waitWithTimeout(interval, repeats);

			}
			catch( Exception ex ){
				throw new EmbJoprTestException(ex.getClass().getName()+": "+ex.getMessage(), ex);
			}

		}





		/**
		 * Undeploys the deployable using EmbJopr's web UI.
		 *
		 * @param type  Type of the deployable - EAR, SAR, WAR, ...
		 */
		public void undeployViaEmbJopr( AppConstants.DeployableTypes type, String fileName )
						throws IOException, HtmlElementNotFoundException, ActionNotAvailableException, ActionOutOfSyncException, EmbJoprTestException
		{

			// Navigate to Enterprise Archives
			getNavTree().getNodeLink(type.getNavTreeLabel()).click();

			getTabMenu().clickSummaryTab();

			// Not tested 
			HtmlButtonInput deleteButton = (HtmlButtonInput) getTabMenu().getTabContentBox()
							.getFirstTable().getFirstRowContainingText(fileName).getButtonByLabel("Delete");
			deleteButton.click();

			// Log the message (with prefix of potential warning)
			//logServerMessage("Something went wrong with undeploy: ");
			// In case of error, throw an exception.
			if( server.getFacesMessages().hasNext() ){
				FacesMessage msg = server.getFacesMessages().next();
				if( msg.getSeverity() == FacesMessage.SEVERITY_INFO ){
					log.info( msg.getSummary() +"\n"+ msg.getDetail() );
				}else{
					throw new EmbJoprTestException( "Undeployment error: " + msg.getSummary() +"\n"+ msg.getDetail() );
				}
			}


			// Sleep for 3 sec. TODO: write some waitForUndeployed()
			sleep( 3000 );

		}// undeployViaEmbJopr()

		// TODO: The following was copied from EarTest.java; remove from there.



		/**
		 * Returns true if EmbJopr lists a deployable of given type and name,
		 * and it's State is UP; false otherwise.
		 */
		public boolean isDeployedAccordingToEmbJopr( DeployableTypes type, String deployableName )
				throws HtmlElementNotFoundException, IOException, ActionNotAvailableException{
			return isDeployedAccordingToEmbJopr( type, deployableName, false );
		}


		/**
		 * Returns true if EmbJopr lists a deployable of given type and name.
		 * @param  mustBeUP  Deployable's state must be UP; otherwise, returns false.
		 */
		public boolean isDeployedAccordingToEmbJopr( 
						DeployableTypes type, String deployableName, boolean mustBeUP )
				throws HtmlElementNotFoundException, IOException, ActionNotAvailableException
		{
			// Refresh / go to the appropriate page.
			getNavTree().getNodeByLabel( type.getNavTreeLabel() ).click();

			// Find the row containing the deployable. Use the pagination if needed.
			ContentTableRow appRow = getTabMenu().getTabContentBox().
							findLinkRowInDataTableUsingPagination(deployableName);

			if( null == appRow ){
				log.debug("Row with '"+deployableName+"' not found.");
				return false;
			}

			if( !mustBeUP ){
				// Not needed to be UP; we found it, thus return true.
				return true;
			}

			String statusText = appRow.getCellTextByColumnName("Status");
			if( "UP".equals(statusText) ){
				return true;
			}else{
				log.debug("Row with "+deployableName+" has Status == '"+statusText+"' != UP.");
				return false;
			}

		}// isDeployedAccordingToEmbJopr()




		/**
		 * Waits for the deployable to be in the UP State.
		 *
		 * @param type  Type of deployable - EAR, WAR, SAR, ...
		 * @param name  Name of the deployable, like 'hello.war'.
		 */
		public void waitActivelyForDeployment(
							final DeployableTypes type, final String name,
							int intervalMS, int retries
						)
						throws EmbJoprTestException, IOException
		{
			waitActivelyForDeployment(type, name, intervalMS, retries, currentTest);
		}

		/**
		 * Waits for the deployable to be in the UP State.
		 *
		 * @param type  Type of deployable - EAR, WAR, SAR, ...
		 * @param name  Name of the deployable, like 'hello.war'.
		 */
		public void waitActivelyForDeployment(
							final DeployableTypes type, final String name,
							int intervalMS, int retries,
							EmbjoprTestCase test // used for page dump
						)
						throws EmbJoprTestException, IOException
		{
			try {
				//final EmbJoprTestToolkit selfEjtt = getEjtt();

				String conditionDesc = type.name()+" "+name+" appears in embjopr as deployed";
				ActiveConditionChecker checker =
				new ActiveConditionChecker(new DescribedCondition(conditionDesc) {

					int callsMade = 0;

					public boolean isTrue() throws HtmlElementNotFoundException, IOException, ActionNotAvailableException {
						// Refresh the page if this is not the first call.
						if (callsMade++ > 0) {
							selfEjtt.refreshPage();
						}
						// If found, return true.
						return isDeployedAccordingToEmbJopr(type, name, true);
						// If found, return true.
					}
				}).throwOnTimeout();
				
				if( null != test )
					checker.dumpPageOnTimeout(test);
					
				boolean deployedSuccessfuly = checker.waitWithTimeout( intervalMS, retries );
			}
			catch( EmbJoprTestException ex ){ throw ex; }
			catch( IOException ex ){ throw ex; }
			// Wrap all other exceptions to EmbJoprTestException.
			catch( Exception ex ){
				throw new EmbJoprTestException("Exception thrown while actively waiting for condition.", ex);
			}
		}


		
		/**
		 * Returns true if there's a file of the given name in server's deploy dir;
		 * false otherwise.
		 */
		public boolean isDeployedAccordingToFileSystem( DeployableTypes type, String name )
		{
			String deployPath = getDeployDir();
			String fileName = deployPath+"/"+name;
			return (new File(fileName)).exists();
		}

		public void undeployViaFileSystem( String name ) throws EmbJoprTestException{
			String deployPath = getDeployDir();
			String fileName = deployPath+"/"+name;
			File fileToDelete = new File(fileName);

			if( !fileToDelete.exists() )
				throw new EmbJoprTestException("Deployable doesn't exist: "+fileName);

			if(fileToDelete.isFile()){
				if( !fileToDelete.delete() ){
					throw new EmbJoprTestException("Unable to delete: "+fileName);
				}
			}else if(fileToDelete.isDirectory()){
				deleteDirectory(fileToDelete);
			}

		}// isDeployedAccordingToFileSystem()



		/**
		 * Unzips given archive from testdata dir to server's deploy/ dir.
		 * @param relativeArchivePath  Path of the archive relative to testdata dir.
		 * @param relativeDestDir      Path of the destination dir relative to deploy dir.
		 */
		public void unzipToDeployDir( String relativeArchivePath, String relativeDestDir )
						throws FileNotFoundException, IOException
		{

			String destDir = getDeployDir()+"/"+relativeDestDir;
			String archivePath = getTestDataDir()+"/"+relativeArchivePath;

			log.info("Unzipping '"+archivePath+"' to '"+destDir+"'.");

			unzipArchive( archivePath, destDir );
		}



		/**
		 * Unzips archive from the given path to the given destination dir.
		 */
		public void unzipArchive(String archivePath, String destDir)
						throws FileNotFoundException, IOException
		{

			FileInputStream fis = new FileInputStream( archivePath );
			ZipInputStream zin = new ZipInputStream(new BufferedInputStream(fis));

			ZipEntry entry;
			while( null != (entry = zin.getNextEntry()) ){

				if(entry.isDirectory()) {
					// Assume directories are stored parents first then children.
					log.info("Extracting directory: " + entry.getName());
					// This is not robust, just for demonstration purposes.
					(new File(destDir +"/"+ entry.getName())).mkdir();
					continue;
				}

				log.info("Extracting file: " + entry.getName());

				final int BUFFER = 2048;
				FileOutputStream fos = new FileOutputStream( destDir +"/"+ entry.getName());
				BufferedOutputStream dest = new BufferedOutputStream(fos, BUFFER);

				int count;
				byte[] data = new byte[BUFFER];
				while ((count = zin.read(data, 0, BUFFER)) != -1) {
					 dest.write(data, 0, count);
				}
				dest.flush();
				dest.close();
				zin.closeEntry();

			}

			zin.close();

		}


		public void moveDir( String from, String to )
		{
			File dirFrom = new File(from);
			File dirTo = new File(to);
			dirFrom.renameTo(dirTo);
		}




		public void deleteFromDeployDir( String deployableName ) throws IOException, EmbJoprTestException
		{

			String deployDir = getDeployDir();

			File dirToDelete = new File( deployDir, deployableName);

			// Just not to delete something accidentally...
			// For tests:
			// /.../embjopr-svn-trunk/jsfunit/target/jboss5x/deploy/unpacked-web1.war.zip
			/*if( ! dirToDelete.getCanonicalPath().contains("/server/") )
				throw new EmbJoprTestException( "Path to deployable doesn't contain '/server/': "
				+dirToDelete.getCanonicalPath() );
			/**/

			// Check if not outside deploy dir.
			if( !dirToDelete.getCanonicalPath().startsWith( deployDir ) )
				throw new IllegalArgumentException(
								" The resulting path is outside deploy dir: "+dirToDelete.getCanonicalPath());

			log.info("Deleting '"+dirToDelete.getPath()+"'...");
			deleteDirectory( dirToDelete );
		}



		public /*static*/ void deleteDirectory(File path) throws EmbJoprTestException
		{
			List<String> undeletableFiles = new ArrayList();
			deleteDirectory(path, undeletableFiles);
			if( !undeletableFiles.isEmpty() ){
				throw new EmbJoprTestException("Couldn't delete following files: \n  "+
								StringUtils.join(undeletableFiles, "\n  ") + "\n");
			}
		}

		public /*static*/ void deleteDirectory(File path, List<String> undeletableFiles)
		{
			if( path.exists() ) {
				File[] files = path.listFiles();
				for(int i=0; i<files.length; i++) {
					 if(files[i].isDirectory()) {
						 deleteDirectory(files[i], undeletableFiles );
					 }
					 else {
						 if( !files[i].delete() ){
							 undeletableFiles.add(files[i].getAbsolutePath());
						 }
					 }
				}
			}
			if( ! path.delete() )
				undeletableFiles.add( path.getAbsolutePath() );
		}


	}// class Deployment




	public final void checkValues( Properties propsExpected, Properties propsActual, ValuesValidator validator ) throws EmbJoprTestException
	{
		checkValues(propsExpected, propsActual, validator, null);
	}

	public final void checkValues( Properties propsExpected, Properties propsActual, ValuesValidator validator, String name ) throws EmbJoprTestException
	{

			// List the actual properties.
			for( Object key : propsActual.keySet() ){
				String propName = (String)key;
				log.info("Actual property: '"+propName+"': '"+propsActual.getProperty(propName)+"'");
			}

			// List of non-equal properties.
			List<String> failures = new ArrayList<String>();

			// Check all the properties.
			for( Object key : propsExpected.keySet() ){
				String propName = (String)key;
				log.info("Checking '"+propName+"'...");
				String exp = propsExpected.getProperty(propName);
				String act = propsActual.getProperty(propName);

				exp = EmbjoprTestCase.normalizeIfNumber(exp);
				act = EmbjoprTestCase.normalizeIfNumber(act);

				String validationError = validator.validate( exp, act );
				if( null != validationError ){
					failures.add( propName+": " + validationError );
				}

				// If some did not match, throw an exception with their list.
				if( failures.size() != 0 ){
					String realm = StringUtils.isBlank(name) ? "" : name+": ";
					String exceptionMsg = String.format("%sNon-matching properties: (%d of %d expected):\n",
									realm, failures.size(), propsExpected.size()) + StringUtils.join(failures, "\n");
					throw new EmbJoprTestException( exceptionMsg, currentTest );
				}
			}

	}



}// class EmbJoprTestToolkit
