package org.jboss.jopr.jsfunit.util;

import javax.naming.NamingException;
import org.jboss.deployers.spi.management.ManagementView;
import org.jboss.deployers.spi.management.deploy.DeploymentManager;
import org.jboss.jopr.jsfunit.util.jaas.SecuredProfileServiceAccess;
import org.jboss.profileservice.spi.ProfileService;


/**
 * Immutable class holding references to ProfileService, ManagementView and DeploymentManager,
 * either secured or unsecured, depending on constructor parameter.
 * @author Ondrej Zizka
 */
public class ProfileServiceUtils {

	// Properties
	final ProfileService profileService;
	final ManagementView managementView;
	final DeploymentManager deploymentManager;
	final boolean secured;

	public DeploymentManager getDeploymentManager() {		return deploymentManager;	}
	public ManagementView getManagementView() {		return managementView;	}
	public ProfileService getProfileService() {		return profileService;	}
	public boolean isSecured() {		return secured;	}
	


	// Ctor.

	public ProfileServiceUtils( boolean secured ) throws NamingException
	{
		this.secured = secured;

		SecuredProfileServiceAccess jaasUtils = null;

		jaasUtils = secured ? SecuredProfileServiceAccess.getSecuredInstance() : SecuredProfileServiceAccess.getUnsecuredInstance();

		// NPE will occur here if exception happens above.
		// TODO: Move ejtt's members initialization to some init() method to avoid this nasty thing.
		this.profileService = jaasUtils.getProfileService();
		this.managementView = jaasUtils.getManagementView();
		this.deploymentManager = jaasUtils.getDeploymentManager();
	}


}
// class ProfileServiceUtils
