/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.jboss.jopr.jsfunit.util;

import org.apache.commons.lang.StringUtils;

/**
 * Value validators.
 * @author Ondrej Zizka
 */
public interface ValuesValidator {
	/** Returns null if values are OK, or validation error description when validation fails. */
	public String validate( String a, String b );

	public static final ValuesValidator EQUAL = new ValuesValidator() {
		public String validate( String exp, String act ){
			if( !StringUtils.equals( exp, act ) ){
				return String.format( "Expected '%s', actual '%s'.", exp, act );
			}
			return null;
		}
	};


	public static final ValuesValidator NOT_EQUAL = new ValuesValidator() {
		public String validate( String exp, String act ){
			if( StringUtils.equals( exp, act ) ){
				return String.format( "Expected different values, but both were '%s'.", act );
			}
			return null;
		}
	};

	
	public static final ValuesValidator CONTAINS = new ValuesValidator() {
		public String validate( String exp, String act ){
			if( ! StringUtils.contains( act, exp ) ){
				return String.format( "String does not contain '%s': %s", exp, act );
			}
			return null;
		}
	};

	public static final ValuesValidator MATCHES = new ValuesValidator() {
		public String validate( String exp, String act ){
			if( ! act.matches(exp) ){
				return String.format( "String does not match '%s': %s", exp, act );
			}
			return null;
		}
	};

}

